import { vectorAdd } from '../../../utils/math/vector.js';
import { isInverseSlider } from '../../helpers/core/baseSlider.js';
import { mirrorCoordinate, resolveGridPosition } from '../../helpers/core/gridObject.js';
import { LINE_COUNT } from '../../shared/constants.js';
import { BaseNote } from './baseNote.js';
/**
 * Base slider beatmap object.
 *
 * @abstract
 */
export class BaseSlider extends BaseNote {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "tailTime", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailPosX", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailPosY", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailLaneRotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
    }
    setTailTime(value) {
        this.tailTime = value;
        return this;
    }
    setTailPosX(value) {
        this.tailPosX = value;
        return this;
    }
    setTailPosY(value) {
        this.tailPosY = value;
        return this;
    }
    setTailLaneRotation(value) {
        this.tailLaneRotation = value;
        return this;
    }
    mirror(flipColor = true, fn) {
        fn?.(this);
        this.tailPosX = mirrorCoordinate(this.tailPosX, LINE_COUNT);
        return super.mirror(flipColor);
    }
    getTailPosition(fn) {
        return fn?.(this) ??
            vectorAdd(resolveGridPosition({ posX: this.tailPosX, posY: this.tailPosY }), [-2]);
    }
    isInverse() {
        return isInverseSlider(this);
    }
}
