import { vectorAdd } from '../../../utils/math/vector.js';
import { isAdjacent, isDiagonal, isHorizontal, isInline, isSlantedWindow, isVertical, isWindow, mirrorCoordinate, resolveGridDistance, resolveGridPosition, } from '../../helpers/core/gridObject.js';
import { LINE_COUNT } from '../../shared/constants.js';
import { BaseObject } from './baseObject.js';
/**
 * Base grid beatmap object.
 *
 * This appear within beatmap lane.
 *
 * @abstract
 */
export class GridObject extends BaseObject {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "posX", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "posY", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "laneRotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
    }
    setPosX(value) {
        this.posX = value;
        return this;
    }
    setPosY(value) {
        this.posY = value;
        return this;
    }
    setLaneRotation(value) {
        this.laneRotation = value;
        return this;
    }
    /**
     * Mirror a grid object, apply alternative flip and Noodle Extensions if available.
     * ```ts
     * obj.mirror(false, optionalFn);
     * ```
     *
     * Alternative flip is true by default. This implementation is typically used to flip color of note.
     */
    mirror(_flipAlt, fn) {
        fn?.(this);
        this.posX = mirrorCoordinate(this.posX, LINE_COUNT);
        return this;
    }
    /**
     * Get object position and return the Beatwalls' position x and y value in tuple.
     * ```ts
     * const objPos = obj.getPosition(optionalFn);
     * ```
     */
    getPosition(fn) {
        return fn?.(this) ?? vectorAdd(resolveGridPosition(this), [-2]);
    }
    getDistance(compareTo, fn) {
        return resolveGridDistance(this, compareTo, (o) => o.getPosition(fn));
    }
    isVertical(compareTo, fn) {
        return isVertical(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isHorizontal(compareTo, fn) {
        return isHorizontal(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isDiagonal(compareTo, fn) {
        return isDiagonal(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isInline(compareTo, lapping, fn) {
        return isInline(this, compareTo, lapping ?? 0.5, (o) => o.getPosition(fn));
    }
    isAdjacent(compareTo, fn) {
        return isAdjacent(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isWindow(compareTo, distance, fn) {
        return isWindow(this, compareTo, distance ?? 1.8, (o) => o.getPosition(fn));
    }
    isSlantedWindow(compareTo, fn) {
        return isSlantedWindow(this, compareTo, (o) => o.getPosition(fn));
    }
}
