import type { GetAngleFn, MirrorFn } from '../../types/beatmap/shared/functions.js';
import type { IWrapArc } from '../../types/beatmap/wrapper/arc.js';
import type { DeepPartial } from '../../types/utils.js';
import { BaseSlider } from './abstract/baseSlider.js';
export declare function createArc(data?: DeepPartial<IWrapArc>): IWrapArc;
/**
 * Core beatmap arc.
 */
export declare class Arc extends BaseSlider implements IWrapArc {
    static defaultValue: IWrapArc;
    static createOne(data?: Partial<IWrapArc>): Arc;
    static create(...data: Partial<IWrapArc>[]): Arc[];
    constructor(data?: Partial<IWrapArc>);
    isValid(fn?: (object: this) => boolean, override?: boolean): boolean;
    lengthMultiplier: IWrapArc['lengthMultiplier'];
    tailLengthMultiplier: IWrapArc['tailLengthMultiplier'];
    tailDirection: IWrapArc['tailDirection'];
    midAnchor: IWrapArc['midAnchor'];
    setLengthMultiplier(value: this['lengthMultiplier']): this;
    setTailLengthMultiplier(value: this['tailLengthMultiplier']): this;
    setTailDirection(value: this['tailDirection']): this;
    setMidAnchor(value: this['midAnchor']): this;
    mirror(flipColor?: boolean, fn?: MirrorFn<this>): this;
    /**
     * Get standardised tail note angle.
     *
     * @example
     * ```ts
     * import type { IArc } from './arc.ts';
     * let arc!: IArc;
     * const optionalFn = (object: IArc) => object.customData.value;
     * const arcTailAngle = arc.getTailAngle(optionalFn);
     * ```
     *
     * Custom function are used to return any arbitrary data first if value exist, otherwise returns base value.
     */
    getTailAngle(fn?: GetAngleFn<this>): number;
}
//# sourceMappingURL=arc.d.ts.map