import { deepCopy } from '../../utils/misc/json.js';
import { BaseItem } from './abstract/baseItem.js';
export function createBasicEventTypesForKeywords(data = {}) {
    return {
        keyword: data.keyword ?? '',
        events: data.events ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap basic event types for keywords.
 */
export class BasicEventTypesForKeywords extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "keyword", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.keyword = data.keyword ?? BasicEventTypesForKeywords.defaultValue.keyword;
        this.events = (data.events ?? BasicEventTypesForKeywords.defaultValue.events)
            .map((e) => e)
            .filter((e) => e);
        this.customData = deepCopy(data.customData ?? BasicEventTypesForKeywords.defaultValue.customData);
    }
    setKeyword(value) {
        this.keyword = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    addEvent(value) {
        this.events.push(value);
        return this;
    }
    removeEvent(value) {
        const index = this.events.indexOf(value, 0);
        if (index > -1) {
            this.events.splice(index, 1);
        }
        return this;
    }
}
Object.defineProperty(BasicEventTypesForKeywords, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBasicEventTypesForKeywords()
});
