import type { GenericBeatmapFilename, GenericLightshowFilename } from '../../types/beatmap/shared/filename.js';
import type { IWrapArc } from '../../types/beatmap/wrapper/arc.js';
import type { IWrapBasicEvent } from '../../types/beatmap/wrapper/basicEvent.js';
import type { IWrapBeatmap } from '../../types/beatmap/wrapper/beatmap.js';
import type { IWrapBombNote } from '../../types/beatmap/wrapper/bombNote.js';
import type { IWrapBPMEvent } from '../../types/beatmap/wrapper/bpmEvent.js';
import type { IWrapChain } from '../../types/beatmap/wrapper/chain.js';
import type { IWrapColorBoostEvent } from '../../types/beatmap/wrapper/colorBoostEvent.js';
import type { IWrapColorNote } from '../../types/beatmap/wrapper/colorNote.js';
import type { IWrapFxEventBoxGroup } from '../../types/beatmap/wrapper/fxEventBoxGroup.js';
import type { IWrapLightColorEventBoxGroup } from '../../types/beatmap/wrapper/lightColorEventBoxGroup.js';
import type { IWrapLightRotationEventBoxGroup } from '../../types/beatmap/wrapper/lightRotationEventBoxGroup.js';
import type { IWrapLightTranslationEventBoxGroup } from '../../types/beatmap/wrapper/lightTranslationEventBoxGroup.js';
import type { IWrapNJSEvent } from '../../types/beatmap/wrapper/njsEvent.js';
import type { IWrapObstacle } from '../../types/beatmap/wrapper/obstacle.js';
import type { IWrapRotationEvent } from '../../types/beatmap/wrapper/rotationEvent.js';
import type { IWrapWaypoint } from '../../types/beatmap/wrapper/waypoint.js';
import type { DeepPartial, LooseAutocomplete } from '../../types/utils.js';
import { BaseItem } from './abstract/baseItem.js';
import type { BaseObject } from './abstract/baseObject.js';
import type { Arc } from './arc.js';
import type { BasicEvent } from './basicEvent.js';
import type { BombNote } from './bombNote.js';
import type { BPMEvent } from './bpmEvent.js';
import type { Chain } from './chain.js';
import type { ColorBoostEvent } from './colorBoostEvent.js';
import type { ColorNote } from './colorNote.js';
import { Difficulty } from './difficulty.js';
import type { FxEventBoxGroup } from './fxEventBoxGroup.js';
import type { LightColorEventBoxGroup } from './lightColorEventBoxGroup.js';
import type { LightRotationEventBoxGroup } from './lightRotationEventBoxGroup.js';
import { Lightshow } from './lightshow.js';
import type { LightTranslationEventBoxGroup } from './lightTranslationEventBoxGroup.js';
import type { NJSEvent } from './njsEvent.js';
import type { Obstacle } from './obstacle.js';
import type { RotationEvent } from './rotationEvent.js';
import type { Waypoint } from './waypoint.js';
export declare function createBeatmap(data?: DeepPartial<IWrapBeatmap>): IWrapBeatmap;
/**
 * Core beatmap container.
 *
 * Contains `Difficulty` and `Lightshow`.
 * This object is writable into 2 files.
 * Both may be combined given schema.
 *
 * **WARN:** Do not use `customData` for anything related to either `difficulty` or `lightshow`,
 * access them directly from `difficulty` and `lightshow` if you wish to modify them.
 * This object exists solely for arbitrary data.
 */
export declare class Beatmap extends BaseItem implements IWrapBeatmap {
    static defaultValue: IWrapBeatmap;
    static createOne(data?: DeepPartial<IWrapBeatmap>): Beatmap;
    static create(...data: DeepPartial<IWrapBeatmap>[]): Beatmap[];
    constructor(data?: DeepPartial<IWrapBeatmap>);
    isValid(fn?: (object: this) => boolean, _override?: boolean): boolean;
    version: number;
    difficulty: Difficulty;
    lightshow: Lightshow;
    filename: LooseAutocomplete<GenericBeatmapFilename>;
    lightshowFilename: LooseAutocomplete<GenericLightshowFilename>;
    get bpmEvents(): BPMEvent[];
    set bpmEvents(value: this['bpmEvents']);
    get rotationEvents(): RotationEvent[];
    set rotationEvents(value: this['rotationEvents']);
    get colorNotes(): ColorNote[];
    set colorNotes(value: this['colorNotes']);
    get bombNotes(): BombNote[];
    set bombNotes(value: this['bombNotes']);
    get obstacles(): Obstacle[];
    set obstacles(value: this['obstacles']);
    get arcs(): Arc[];
    set arcs(value: this['arcs']);
    get chains(): Chain[];
    set chains(value: this['chains']);
    get njsEvents(): NJSEvent[];
    set njsEvents(value: this['njsEvents']);
    get waypoints(): Waypoint[];
    set waypoints(value: this['waypoints']);
    get basicEvents(): BasicEvent[];
    set basicEvents(value: this['basicEvents']);
    get colorBoostEvents(): ColorBoostEvent[];
    set colorBoostEvents(value: this['colorBoostEvents']);
    get lightColorEventBoxGroups(): LightColorEventBoxGroup[];
    set lightColorEventBoxGroups(value: this['lightColorEventBoxGroups']);
    get lightRotationEventBoxGroups(): LightRotationEventBoxGroup[];
    set lightRotationEventBoxGroups(value: this['lightRotationEventBoxGroups']);
    get lightTranslationEventBoxGroups(): LightTranslationEventBoxGroup[];
    set lightTranslationEventBoxGroups(value: this['lightTranslationEventBoxGroups']);
    get fxEventBoxGroups(): FxEventBoxGroup[];
    set fxEventBoxGroups(value: this['fxEventBoxGroups']);
    get basicEventTypesWithKeywords(): Lightshow['basicEventTypesWithKeywords'];
    set basicEventTypesWithKeywords(value: this['basicEventTypesWithKeywords']);
    get useNormalEventsAsCompatibleEvents(): boolean;
    set useNormalEventsAsCompatibleEvents(value: this['useNormalEventsAsCompatibleEvents']);
    setFilename(filename: LooseAutocomplete<GenericBeatmapFilename>): this;
    setLightshowFilename(filename: LooseAutocomplete<GenericLightshowFilename>): this;
    setVersion(version: number): this;
    sort(fn?: (object: this) => void): this;
    allObjects(): BaseObject[];
    addBpmEvents(...data: DeepPartial<IWrapBPMEvent>[]): this;
    addRotationEvents(...data: DeepPartial<IWrapRotationEvent>[]): this;
    addColorNotes(...data: DeepPartial<IWrapColorNote>[]): this;
    addBombNotes(...data: DeepPartial<IWrapBombNote>[]): this;
    addObstacles(...data: DeepPartial<IWrapObstacle>[]): this;
    addArcs(...data: DeepPartial<IWrapArc>[]): this;
    addChains(...data: DeepPartial<IWrapChain>[]): this;
    addNjsEvents(...data: DeepPartial<IWrapNJSEvent>[]): this;
    addWaypoints(...data: DeepPartial<IWrapWaypoint>[]): this;
    addBasicEvents(...data: DeepPartial<IWrapBasicEvent>[]): this;
    addColorBoostEvents(...data: DeepPartial<IWrapColorBoostEvent>[]): this;
    addLightColorEventBoxGroups(...data: DeepPartial<IWrapLightColorEventBoxGroup>[]): this;
    addLightRotationEventBoxGroups(...data: DeepPartial<IWrapLightRotationEventBoxGroup>[]): this;
    addLightTranslationEventBoxGroups(...data: DeepPartial<IWrapLightTranslationEventBoxGroup>[]): this;
    addFxEventBoxGroups(...data: DeepPartial<IWrapFxEventBoxGroup>[]): this;
}
//# sourceMappingURL=beatmap.d.ts.map