import { deepCopy } from '../../utils/misc/json.js';
import { BaseNote } from './abstract/baseNote.js';
export function createBombNote(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        color: -1,
        direction: data.direction ?? 0,
        laneRotation: data.laneRotation ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap bomb note.
 */
export class BombNote extends BaseNote {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        this.time = data.time ?? BombNote.defaultValue.time;
        this.posX = data.posX ?? BombNote.defaultValue.posX;
        this.posY = data.posY ?? BombNote.defaultValue.posY;
        this.color = -1;
        this.direction = data.direction ?? BombNote.defaultValue.direction;
        this.laneRotation = data.laneRotation ?? BombNote.defaultValue.laneRotation;
        this.customData = deepCopy(data.customData ?? BombNote.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.posX >= 0 &&
            this.posX <= 3 &&
            this.posY >= 0 &&
            this.posY <= 2;
    }
}
Object.defineProperty(BombNote, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBombNote()
});
