import { deepCopy } from '../../utils/misc/json.js';
import { BaseObject } from './abstract/baseObject.js';
export function createBPMEvent(data = {}) {
    return {
        time: data.time ?? 0,
        bpm: data.bpm ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap BPM event.
 */
export class BPMEvent extends BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "bpm", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? BPMEvent.defaultValue.time;
        this.bpm = data.bpm ?? BPMEvent.defaultValue.bpm;
        this.customData = deepCopy(data.customData ?? BPMEvent.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) && this.bpm > 0;
    }
    setBPM(value) {
        this.bpm = value;
        return this;
    }
}
Object.defineProperty(BPMEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createBPMEvent()
});
