import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { EventBox } from './abstract/eventBox.js';
import { createFxEventFloat, FxEventFloat } from './fxEventFloat.js';
import { createIndexFilter, IndexFilter } from './indexFilter.js';
export function createFxEventBox(data = {}) {
    return {
        filter: createIndexFilter(data.filter),
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        fxDistribution: data.fxDistribution ?? 0,
        fxDistributionType: data.fxDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map(createFxEventFloat) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap FX event box.
 */
export class FxEventBox extends EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "fxDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fxDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new IndexFilter(data.filter ?? FxEventBox.defaultValue.filter);
        this.beatDistribution = data.beatDistribution ?? FxEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            FxEventBox.defaultValue.beatDistributionType;
        this.fxDistribution = data.fxDistribution ?? FxEventBox.defaultValue.fxDistribution;
        this.fxDistributionType = data.fxDistributionType ??
            FxEventBox.defaultValue.fxDistributionType;
        this.affectFirst = data.affectFirst ?? FxEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? FxEventBox.defaultValue.easing;
        this.events = (data.events ?? FxEventBox.defaultValue.events).map((obj) => new FxEventFloat(obj));
        this.customData = deepCopy(data.customData ?? FxEventBox.defaultValue.customData);
    }
    reconcile() {
        this.filter = reconcileClassObject(this.filter, IndexFilter);
        this.events = reconcileClassObject(this.events, FxEventFloat);
        return this;
    }
    setFxDistribution(value) {
        this.fxDistribution = value;
        return this;
    }
    setFxDistributionType(value) {
        this.fxDistributionType = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.fxDistributionType === 1 || this.fxDistributionType === 2) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
Object.defineProperty(FxEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createFxEventBox()
});
