import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { EventBoxGroup } from './abstract/eventBoxGroup.js';
import { createFxEventBox, FxEventBox } from './fxEventBox.js';
export function createFxEventBoxGroup(data = {}) {
    return {
        time: data.time ?? 0,
        id: data.id ?? 0,
        boxes: data.boxes?.map(createFxEventBox) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap FX event box group.
 */
export class FxEventBoxGroup extends EventBoxGroup {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "boxes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? FxEventBoxGroup.defaultValue.time;
        this.id = data.id ?? FxEventBoxGroup.defaultValue.id;
        this.boxes = (data.boxes ?? FxEventBoxGroup.defaultValue.boxes).map((e) => new FxEventBox(e));
        this.customData = deepCopy(data.customData ?? FxEventBoxGroup.defaultValue.customData);
    }
    reconcile() {
        this.boxes = reconcileClassObject(this.boxes, FxEventBox);
        for (let j = 0; j < this.boxes.length; j++) {
            this.boxes[j].reconcile();
        }
        return this;
    }
}
Object.defineProperty(FxEventBoxGroup, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createFxEventBoxGroup()
});
