import { deepCopy } from '../../utils/misc/json.js';
import { BaseObject } from './abstract/baseObject.js';
export function createFxEventFloat(data = {}) {
    return {
        time: data.time ?? 0,
        easing: data.easing ?? 0,
        previous: data.previous ?? 0,
        value: data.value ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap FX event float.
 */
export class FxEventFloat extends BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "easing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "value", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? FxEventFloat.defaultValue.time;
        this.easing = data.easing ?? FxEventFloat.defaultValue.easing;
        this.previous = data.previous ?? FxEventFloat.defaultValue.previous;
        this.value = data.value ?? FxEventFloat.defaultValue.value;
        this.customData = deepCopy(data.customData ?? FxEventFloat.defaultValue.customData);
    }
    setEasing(value) {
        this.easing = value;
        return this;
    }
    setPrevious(value) {
        this.previous = value;
        return this;
    }
    setValue(value) {
        this.value = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.previous === 0 || this.previous === 1) &&
            this.easing >= -1 &&
            this.easing <= 103;
    }
}
Object.defineProperty(FxEventFloat, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createFxEventFloat()
});
