import { deepCopy } from '../../utils/misc/json.js';
import { BaseObject } from './abstract/baseObject.js';
export function createFxEventInt(data = {}) {
    return {
        time: data.time ?? 0,
        previous: data.previous ?? 0,
        value: data.value ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap FX event int.
 */
export class FxEventInt extends BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "value", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? FxEventInt.defaultValue.time;
        this.previous = data.previous ?? FxEventInt.defaultValue.previous;
        this.value = data.value ?? FxEventInt.defaultValue.value;
        this.customData = deepCopy(data.customData ?? FxEventInt.defaultValue.customData);
    }
    setPrevious(value) {
        this.previous = value;
        return this;
    }
    setValue(value) {
        this.value = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.previous === 0 || this.previous === 1);
    }
}
Object.defineProperty(FxEventInt, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createFxEventInt()
});
