import { LimitAlsoAffectsType, RandomType } from '../../types/beatmap/shared/constants.js';
import { deepCopy } from '../../utils/misc/json.js';
import { BaseItem } from './abstract/baseItem.js';
export function createIndexFilter(data = {}) {
    return {
        type: data.type ?? 1,
        p0: data.p0 ?? 0,
        p1: data.p1 ?? 0,
        reverse: data.reverse ?? 0,
        chunks: data.chunks ?? 0,
        random: data.random ?? 0,
        seed: data.seed ?? 0,
        limit: data.limit ?? 0,
        limitAffectsType: data.limitAffectsType ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap index filter.
 */
export class IndexFilter extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "type", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "p0", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "p1", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "reverse", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "chunks", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "limit", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "limitAffectsType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "random", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "seed", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.type = data.type ?? IndexFilter.defaultValue.type;
        this.p0 = data.p0 ?? IndexFilter.defaultValue.p0;
        this.p1 = data.p1 ?? IndexFilter.defaultValue.p1;
        this.reverse = data.reverse ?? IndexFilter.defaultValue.reverse;
        this.chunks = data.chunks ?? IndexFilter.defaultValue.chunks;
        this.random = data.random ?? IndexFilter.defaultValue.random;
        this.seed = data.seed ?? IndexFilter.defaultValue.seed;
        this.limit = data.limit ?? IndexFilter.defaultValue.limit;
        this.limitAffectsType = data.limitAffectsType ?? IndexFilter.defaultValue.limitAffectsType;
        this.customData = deepCopy(data.customData ?? IndexFilter.defaultValue.customData);
    }
    setType(value) {
        this.type = value;
        return this;
    }
    setP0(value) {
        this.p0 = value;
        return this;
    }
    setP1(value) {
        this.p1 = value;
        return this;
    }
    setReverse(value) {
        this.reverse = value;
        return this;
    }
    setChunks(value) {
        this.chunks = value;
        return this;
    }
    setRandom(value) {
        this.random = value;
        return this;
    }
    setSeed(value) {
        this.seed = value;
        return this;
    }
    setLimit(value) {
        this.limit = value;
        return this;
    }
    setLimitAffectsType(value) {
        this.limitAffectsType = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.type === 1 || this.type === 2) &&
            this.p0 >= 0 &&
            this.p1 >= 0 &&
            (this.reverse === 0 || this.reverse === 1) &&
            this.chunks >= 0 &&
            this.random >= 0 &&
            this.random <= RandomType.ALL &&
            this.limit >= 0 &&
            this.limit <= 1 &&
            this.limitAffectsType >= 0 &&
            this.limitAffectsType <= LimitAlsoAffectsType.ALL;
    }
}
Object.defineProperty(IndexFilter, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createIndexFilter()
});
