import { deepCopy } from '../../utils/misc/json.js';
import { BaseItem } from './abstract/baseItem.js';
export function createInfoBeatmap(data = {}) {
    return {
        characteristic: data.characteristic ?? 'Standard',
        difficulty: data.difficulty ?? 'Easy',
        filename: data.filename ?? 'Unnamed.beatmap.dat',
        lightshowFilename: data.lightshowFilename ?? 'Unnamed.lightshow.dat',
        authors: {
            mappers: data.authors?.mappers ?? [],
            lighters: data.authors?.lighters ?? [],
        },
        njs: data.njs ?? 0,
        njsOffset: data.njsOffset ?? 0,
        colorSchemeId: data.colorSchemeId ?? -1,
        environmentId: data.environmentId ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap info beatmap.
 */
export class InfoBeatmap extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "characteristic", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "difficulty", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "filename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightshowFilename", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "authors", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "njs", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "njsOffset", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "colorSchemeId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "environmentId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.characteristic = data.characteristic ?? InfoBeatmap.defaultValue.characteristic;
        this.difficulty = data.difficulty ?? InfoBeatmap.defaultValue.difficulty;
        this.filename = data.filename ?? InfoBeatmap.defaultValue.filename;
        this.lightshowFilename = data.lightshowFilename ?? InfoBeatmap.defaultValue.lightshowFilename;
        this.authors = {
            mappers: (data.authors?.mappers ?? InfoBeatmap.defaultValue.authors.mappers).map((e) => e),
            lighters: (data.authors?.lighters ?? InfoBeatmap.defaultValue.authors.lighters).map((e) => e),
        };
        this.njs = data.njs ?? InfoBeatmap.defaultValue.njs;
        this.njsOffset = data.njsOffset ?? InfoBeatmap.defaultValue.njsOffset;
        this.colorSchemeId = data.colorSchemeId ?? InfoBeatmap.defaultValue.colorSchemeId;
        this.environmentId = data.environmentId ?? InfoBeatmap.defaultValue.environmentId;
        this.customData = deepCopy(data.customData ?? InfoBeatmap.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.njs > 0 &&
            this.colorSchemeId >= -1 &&
            this.environmentId >= 0;
    }
}
Object.defineProperty(InfoBeatmap, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createInfoBeatmap()
});
