import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { EventBox } from './abstract/eventBox.js';
import { createIndexFilter, IndexFilter } from './indexFilter.js';
import { createLightColorEvent, LightColorEvent } from './lightColorEvent.js';
export function createLightColorEventBox(data = {}) {
    return {
        filter: createIndexFilter(data.filter),
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        brightnessDistribution: data.brightnessDistribution ?? 0,
        brightnessDistributionType: data.brightnessDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map((o) => createLightColorEvent(o)) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap light color event box.
 */
export class LightColorEventBox extends EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "brightnessDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "brightnessDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new IndexFilter(data.filter ?? LightColorEventBox.defaultValue.filter);
        this.beatDistribution = data.beatDistribution ??
            LightColorEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            LightColorEventBox.defaultValue.beatDistributionType;
        this.brightnessDistribution = data.brightnessDistribution ??
            LightColorEventBox.defaultValue.brightnessDistribution;
        this.brightnessDistributionType = data.brightnessDistributionType ??
            LightColorEventBox.defaultValue.brightnessDistributionType;
        this.affectFirst = data.affectFirst ?? LightColorEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? LightColorEventBox.defaultValue.easing;
        this.events = (data.events ?? LightColorEventBox.defaultValue.events).map((obj) => new LightColorEvent(obj));
        this.customData = deepCopy(data.customData ?? LightColorEventBox.defaultValue.customData);
    }
    reconcile() {
        this.events = reconcileClassObject(this.events, LightColorEvent);
        return this;
    }
    setBrightnessDistribution(value) {
        this.brightnessDistribution = value;
        return this;
    }
    setBrightnessDistributionType(value) {
        this.brightnessDistributionType = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.brightnessDistributionType === 1 || this.brightnessDistributionType === 2) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
Object.defineProperty(LightColorEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightColorEventBox()
});
