import { deepCopy } from '../../utils/misc/json.js';
import { BaseObject } from './abstract/baseObject.js';
export function createLightRotationEvent(data = {}) {
    return {
        time: data.time ?? 0,
        easing: data.easing ?? 0,
        loop: data.loop ?? 0,
        direction: data.direction ?? 0,
        previous: data.previous ?? 0,
        rotation: data.rotation ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap light rotation event.
 */
export class LightRotationEvent extends BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "easing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "loop", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "rotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "direction", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? LightRotationEvent.defaultValue.time;
        this.easing = data.easing ?? LightRotationEvent.defaultValue.easing;
        this.loop = data.loop ?? LightRotationEvent.defaultValue.loop;
        this.direction = data.direction ?? LightRotationEvent.defaultValue.direction;
        this.previous = data.previous ?? LightRotationEvent.defaultValue.previous;
        this.rotation = data.rotation ?? LightRotationEvent.defaultValue.rotation;
        this.customData = deepCopy(data.customData ?? LightRotationEvent.defaultValue.customData);
    }
    setPrevious(value) {
        this.previous = value;
        return this;
    }
    setEasing(value) {
        this.easing = value;
        return this;
    }
    setLoop(value) {
        this.loop = value;
        return this;
    }
    setRotation(value) {
        this.rotation = value;
        return this;
    }
    setDirection(value) {
        this.direction = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.previous === 0 || this.previous === 1) &&
            this.easing >= -1 &&
            this.easing <= 103 &&
            this.loop >= 0 &&
            this.direction >= 0 &&
            this.direction <= 2;
    }
}
Object.defineProperty(LightRotationEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightRotationEvent()
});
