import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { EventBox } from './abstract/eventBox.js';
import { createIndexFilter, IndexFilter } from './indexFilter.js';
import { createLightRotationEvent, LightRotationEvent } from './lightRotationEvent.js';
export function createLightRotationEventBox(data = {}) {
    return {
        filter: createIndexFilter(data.filter),
        axis: data.axis ?? 0,
        flip: data.flip ?? 0,
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        rotationDistribution: data.rotationDistribution ?? 0,
        rotationDistributionType: data.rotationDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map((o) => createLightRotationEvent(o)) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap light rotation event box.
 */
export class LightRotationEventBox extends EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "rotationDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "rotationDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "axis", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "flip", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new IndexFilter(data.filter ?? LightRotationEventBox.defaultValue.filter);
        this.axis = data.axis ?? LightRotationEventBox.defaultValue.axis;
        this.flip = data.flip ?? LightRotationEventBox.defaultValue.flip;
        this.beatDistribution = data.beatDistribution ??
            LightRotationEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            LightRotationEventBox.defaultValue.beatDistributionType;
        this.rotationDistribution = data.rotationDistribution ??
            LightRotationEventBox.defaultValue.rotationDistribution;
        this.rotationDistributionType = data.rotationDistributionType ??
            LightRotationEventBox.defaultValue.rotationDistributionType;
        this.affectFirst = data.affectFirst ?? LightRotationEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? LightRotationEventBox.defaultValue.easing;
        this.events = (data.events ?? LightRotationEventBox.defaultValue.events).map((obj) => new LightRotationEvent(obj));
        this.customData = deepCopy(data.customData ?? LightRotationEventBox.defaultValue.customData);
    }
    reconcile() {
        this.events = reconcileClassObject(this.events, LightRotationEvent);
        return this;
    }
    setRotationDistribution(value) {
        this.rotationDistribution = value;
        return this;
    }
    setRotationDistributionType(value) {
        this.rotationDistributionType = value;
        return this;
    }
    setAxis(value) {
        this.axis = value;
        return this;
    }
    setFlip(value) {
        this.flip = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.rotationDistributionType === 1 || this.rotationDistributionType === 2) &&
            (this.axis === 0 || this.axis === 1 || this.axis === 2) &&
            (this.flip === 0 || this.flip === 1) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
Object.defineProperty(LightRotationEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightRotationEventBox()
});
