import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { EventBoxGroup } from './abstract/eventBoxGroup.js';
import { createLightTranslationEventBox, LightTranslationEventBox, } from './lightTranslationEventBox.js';
export function createLightTranslationEventBoxGroup(data = {}) {
    return {
        time: data.time ?? 0,
        id: data.id ?? 0,
        boxes: data.boxes?.map((e) => createLightTranslationEventBox(e)) ?? [],
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap light translation event box group.
 */
export class LightTranslationEventBoxGroup extends EventBoxGroup {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "boxes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? LightTranslationEventBoxGroup.defaultValue.time;
        this.id = data.id ?? LightTranslationEventBoxGroup.defaultValue.id;
        this.boxes = (data.boxes ?? LightTranslationEventBoxGroup.defaultValue.boxes).map((e) => new LightTranslationEventBox(e));
        this.customData = deepCopy(data.customData ??
            LightTranslationEventBoxGroup.defaultValue.customData);
    }
    reconcile() {
        this.boxes = reconcileClassObject(this.boxes, LightTranslationEventBox);
        for (let j = 0; j < this.boxes.length; j++) {
            this.boxes[j].reconcile();
        }
        return this;
    }
}
Object.defineProperty(LightTranslationEventBoxGroup, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightTranslationEventBoxGroup()
});
