import { deepCopy } from '../../utils/misc/json.js';
import { reconcileClassObject } from '../helpers/core/misc.js';
import { sortObjectFn } from '../helpers/sort.js';
import { BaseItem } from './abstract/baseItem.js';
import { BasicEvent, createBasicEvent } from './basicEvent.js';
import { BasicEventTypesForKeywords, createBasicEventTypesForKeywords, } from './basicEventTypesForKeywords.js';
import { ColorBoostEvent, createColorBoostEvent } from './colorBoostEvent.js';
import { createFxEventBoxGroup, FxEventBoxGroup } from './fxEventBoxGroup.js';
import { createLightColorEventBoxGroup, LightColorEventBoxGroup, } from './lightColorEventBoxGroup.js';
import { createLightRotationEventBoxGroup, LightRotationEventBoxGroup, } from './lightRotationEventBoxGroup.js';
import { createLightTranslationEventBoxGroup, LightTranslationEventBoxGroup, } from './lightTranslationEventBoxGroup.js';
import { createWaypoint, Waypoint } from './waypoint.js';
export function createLightshow(data = {}) {
    return {
        waypoints: data.waypoints?.map(createWaypoint) ?? [],
        basicEvents: data.basicEvents?.map(createBasicEvent) ?? [],
        colorBoostEvents: data.colorBoostEvents?.map(createColorBoostEvent) ?? [],
        lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map(createLightColorEventBoxGroup) ??
            [],
        lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map(createLightRotationEventBoxGroup) ?? [],
        lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map(createLightTranslationEventBoxGroup) ?? [],
        fxEventBoxGroups: data.fxEventBoxGroups?.map(createFxEventBoxGroup) ?? [],
        basicEventTypesWithKeywords: {
            list: data.basicEventTypesWithKeywords?.list?.map(createBasicEventTypesForKeywords) ?? [],
        },
        useNormalEventsAsCompatibleEvents: !!data.useNormalEventsAsCompatibleEvents,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap lightshow.
 */
export class Lightshow extends BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "waypoints", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "basicEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "colorBoostEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightColorEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightRotationEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightTranslationEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fxEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "basicEventTypesWithKeywords", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "useNormalEventsAsCompatibleEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.waypoints = (data.waypoints ?? Lightshow.defaultValue.waypoints).map((e) => new Waypoint(e));
        this.basicEvents = (data.basicEvents ?? Lightshow.defaultValue.basicEvents).map((e) => new BasicEvent(e));
        // shut the fuck up, ts, it's not that deep
        // deno-lint-ignore ban-ts-comment
        // @ts-ignore
        this.colorBoostEvents = (data.colorBoostEvents ?? Lightshow.defaultValue.colorBoostEvents).map((e) => new ColorBoostEvent(e));
        this.lightColorEventBoxGroups = (data.lightColorEventBoxGroups ??
            Lightshow.defaultValue.lightColorEventBoxGroups).map((e) => new LightColorEventBoxGroup(e));
        this.lightRotationEventBoxGroups = (data.lightRotationEventBoxGroups ??
            Lightshow.defaultValue.lightRotationEventBoxGroups).map((e) => new LightRotationEventBoxGroup(e));
        this.lightTranslationEventBoxGroups = (data.lightTranslationEventBoxGroups ??
            Lightshow.defaultValue.lightTranslationEventBoxGroups).map((e) => new LightTranslationEventBoxGroup(e));
        this.fxEventBoxGroups = (data.fxEventBoxGroups ?? Lightshow.defaultValue.fxEventBoxGroups).map((e) => new FxEventBoxGroup(e));
        this.basicEventTypesWithKeywords = {
            list: (data.basicEventTypesWithKeywords?.list ??
                Lightshow.defaultValue.basicEventTypesWithKeywords.list).map((e) => new BasicEventTypesForKeywords(e)),
        };
        this.useNormalEventsAsCompatibleEvents = !!data.useNormalEventsAsCompatibleEvents;
        this.customData = deepCopy(data.customData ?? Lightshow.defaultValue.customData);
    }
    reconcile() {
        this.waypoints = reconcileClassObject(this.waypoints, Waypoint);
        this.basicEvents = reconcileClassObject(this.basicEvents, BasicEvent);
        this.colorBoostEvents = reconcileClassObject(this.colorBoostEvents, ColorBoostEvent);
        this.lightColorEventBoxGroups = reconcileClassObject(this.lightColorEventBoxGroups, LightColorEventBoxGroup);
        this.lightRotationEventBoxGroups = reconcileClassObject(this.lightRotationEventBoxGroups, LightRotationEventBoxGroup);
        this.lightTranslationEventBoxGroups = reconcileClassObject(this.lightTranslationEventBoxGroups, LightTranslationEventBoxGroup);
        this.fxEventBoxGroups = reconcileClassObject(this.fxEventBoxGroups, FxEventBoxGroup);
        this.basicEventTypesWithKeywords.list = reconcileClassObject(this.basicEventTypesWithKeywords.list, BasicEventTypesForKeywords);
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.waypoints.every((e) => e.isValid()) &&
            this.basicEvents.every((e) => e.isValid()) &&
            this.colorBoostEvents.every((e) => e.isValid()) &&
            this.lightColorEventBoxGroups.every((e) => e.isValid()) &&
            this.lightRotationEventBoxGroups.every((e) => e.isValid()) &&
            this.lightTranslationEventBoxGroups.every((e) => e.isValid()) &&
            this.fxEventBoxGroups.every((e) => e.isValid()) &&
            this.basicEventTypesWithKeywords.list.every((e) => e.isValid());
    }
    sort() {
        this.waypoints.sort(sortObjectFn);
        this.basicEvents.sort(sortObjectFn);
        this.colorBoostEvents.sort(sortObjectFn);
        this.lightColorEventBoxGroups.sort(sortObjectFn);
        this.lightRotationEventBoxGroups.sort(sortObjectFn);
        this.lightTranslationEventBoxGroups.sort(sortObjectFn);
        this.fxEventBoxGroups.sort(sortObjectFn);
        for (let i = 0; i < this.lightColorEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightColorEventBoxGroups[i].boxes.length; j++) {
                this.lightColorEventBoxGroups[i].boxes[j].events.sort(sortObjectFn);
            }
        }
        for (let i = 0; i < this.lightRotationEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightRotationEventBoxGroups[i].boxes.length; j++) {
                this.lightRotationEventBoxGroups[i].boxes[j].events.sort(sortObjectFn);
            }
        }
        for (let i = 0; i < this.lightTranslationEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightTranslationEventBoxGroups[i].boxes.length; j++) {
                this.lightTranslationEventBoxGroups[i].boxes[j].events.sort(sortObjectFn);
            }
        }
        for (let i = 0; i < this.fxEventBoxGroups.length; i++) {
            for (let j = 0; j < this.fxEventBoxGroups[i].boxes.length; j++) {
                this.fxEventBoxGroups[i].boxes[j].events.sort(sortObjectFn);
            }
        }
        return this;
    }
    allObjects() {
        return [
            ...this.waypoints,
            ...this.basicEvents,
            ...this.colorBoostEvents,
            ...this.lightColorEventBoxGroups,
            ...this.lightRotationEventBoxGroups,
            ...this.lightTranslationEventBoxGroups,
            ...this.fxEventBoxGroups,
        ];
    }
    addWaypoints(...data) {
        for (const d of data) {
            this.waypoints.push(new Waypoint(d));
        }
        return this;
    }
    addBasicEvents(...data) {
        for (const d of data) {
            this.basicEvents.push(new BasicEvent(d));
        }
        return this;
    }
    addColorBoostEvents(...data) {
        for (const d of data) {
            this.colorBoostEvents.push(new ColorBoostEvent(d));
        }
        return this;
    }
    addLightColorEventBoxGroups(...data) {
        for (const d of data) {
            this.lightColorEventBoxGroups.push(new LightColorEventBoxGroup(d));
        }
        return this;
    }
    addLightRotationEventBoxGroups(...data) {
        for (const d of data) {
            this.lightRotationEventBoxGroups.push(new LightRotationEventBoxGroup(d));
        }
        return this;
    }
    addLightTranslationEventBoxGroups(...data) {
        for (const d of data) {
            this.lightTranslationEventBoxGroups.push(new LightTranslationEventBoxGroup(d));
        }
        return this;
    }
    addFxEventBoxGroups(...data) {
        for (const d of data) {
            this.fxEventBoxGroups.push(new FxEventBoxGroup(d));
        }
        return this;
    }
}
Object.defineProperty(Lightshow, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightshow()
});
