import { deepCopy } from '../../utils/misc/json.js';
import { mirrorCoordinate } from '../helpers/core/gridObject.js';
import { LINE_COUNT } from '../shared/constants.js';
import { GridObject } from './abstract/gridObject.js';
export function createWaypoint(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        direction: data.direction ?? 0,
        laneRotation: data.laneRotation ?? 0,
        customData: deepCopy({ ...data.customData }),
    };
}
/**
 * Core beatmap waypoint.
 */
export class Waypoint extends GridObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "direction", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? Waypoint.defaultValue.time;
        this.posX = data.posX ?? Waypoint.defaultValue.posX;
        this.posY = data.posY ?? Waypoint.defaultValue.posY;
        this.direction = data.direction ?? Waypoint.defaultValue.direction;
        this.laneRotation = data.laneRotation ?? Waypoint.defaultValue.laneRotation;
        this.customData = deepCopy(data.customData ?? Waypoint.defaultValue.customData);
    }
    setDirection(value) {
        this.direction = value;
        return this;
    }
    mirror(_flipAlt, fn) {
        fn?.(this);
        this.posX = mirrorCoordinate(this.posX, LINE_COUNT);
        switch (this.direction) {
            case 2:
                this.direction = 3;
                break;
            case 3:
                this.direction = 2;
                break;
            case 6:
                this.direction = 7;
                break;
            case 7:
                this.direction = 6;
                break;
            case 4:
                this.direction = 5;
                break;
            case 5:
                this.direction = 4;
                break;
        }
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.direction >= 0 &&
            this.direction <= 9 &&
            this.direction !== 8;
    }
}
Object.defineProperty(Waypoint, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createWaypoint()
});
