import type { ISwingAnalysisBeatmap } from '../../extensions/swing/types/swing.js';
import type { IWrapBeatmapSubset } from '../../types/beatmap/wrapper/beatmap.js';
import type { TimeProcessor } from './timeProcessor.js';
/** Calculates the number of notes per second. */
export declare function calculateNps<T extends IWrapBeatmapSubset<'colorNotes', never>>(beatmap: T, duration: number): number;
/** Calculates the peak number of notes per second given beat count. */
export declare function calculateNpsPeak<T extends IWrapBeatmapSubset<'colorNotes', 'time'>>(beatmap: T, beatCount: number, timeProcessor: TimeProcessor): number;
/**
 * Get the first interactive time.
 *
 * Color notes, bomb notes, chains and obstacles in middle lane are considered interactive object.
 */
export declare function getFirstInteractiveTime<T extends ISwingAnalysisBeatmap>(beatmap: T): number;
/**
 * Get the last interactive time.
 *
 * Color notes, bomb notes, chains and obstacles in middle lane are considered interactive object.
 */
export declare function getLastInteractiveTime<T extends ISwingAnalysisBeatmap>(beatmap: T): number;
/**
 * Get the first obstacle interactive time.
 */
export declare function findFirstInteractiveObstacleTime<T extends IWrapBeatmapSubset<'obstacles', 'time' | 'posX' | 'width'>>(beatmap: T): number;
/**
 * Get the last obstacle interactive time.
 */
export declare function findLastInteractiveObstacleTime<T extends IWrapBeatmapSubset<'obstacles', 'time' | 'posX' | 'duration' | 'width'>>(beatmap: T): number;
//# sourceMappingURL=beatmap.d.ts.map