import { NoteColor, NoteDirection, NoteDirectionAngle } from '../../shared/constants.js';
import { cycle } from '../../../utils/misc/iterator.js';
/** Check if note color is red. */
export function isRedNoteColor(color) {
    return color === NoteColor.RED;
}
/** Check if note color is blue. */
export function isBlueNoteColor(color) {
    return color === NoteColor.BLUE;
}
/** Check if note color is not set (-1). */
export function isUnsetNoteColor(color) {
    return color === NoteColor.NONE;
}
/** Compare two notes and return if they would form a double. */
export function isDouble(object, compareTo, tolerance = 0.01) {
    return (compareTo.time > object.time - tolerance &&
        compareTo.time < object.time + tolerance &&
        object.color !== compareTo.color);
}
/** Get standardised note angle. */
export function resolveNoteAngle(direction) {
    return NoteDirectionAngle[direction] ?? 0;
}
/** Mirror note color. */
export function mirrorNoteColor(color) {
    return cycle([NoteColor.RED, NoteColor.BLUE], color);
}
/** Mirror note direction horizontally. */
export function mirrorNoteDirectionHorizontally(direction) {
    switch (direction) {
        case NoteDirection.UP_LEFT:
        case NoteDirection.LEFT:
        case NoteDirection.DOWN_LEFT: {
            return direction + 1;
        }
        case NoteDirection.UP_RIGHT:
        case NoteDirection.RIGHT:
        case NoteDirection.DOWN_RIGHT: {
            return direction - 1;
        }
        default: {
            return direction;
        }
    }
}
/** Mirror note direction vertically. */
export function mirrorNoteDirectionVertically(direction) {
    switch (direction) {
        case NoteDirection.UP_LEFT:
        case NoteDirection.UP_RIGHT: {
            return direction + 2;
        }
        case NoteDirection.UP: {
            return direction + 1;
        }
        case NoteDirection.DOWN: {
            return direction - 1;
        }
        case NoteDirection.DOWN_LEFT:
        case NoteDirection.DOWN_RIGHT: {
            return direction - 2;
        }
        default: {
            return direction;
        }
    }
}
