import { vectorDistance, vectorIsDiagonal, vectorIsHorizontal, vectorIsVertical, } from '../../../utils/math/vector.js';
/** Get standardized grid position. */
export function resolveGridPosition(object) {
    return [object.posX, object.posY];
}
/** Get standardized grid distance. */
export function resolveGridDistance(o1, o2, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return vectorDistance(v1, v2);
}
/** Check if two objects are in horizontal alignment. */
export function isHorizontal(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return vectorIsHorizontal(v1, v2, epsilon);
}
/** Check if two objects are in vertical alignment. */
export function isVertical(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return vectorIsVertical(v1, v2, epsilon);
}
/** Check if two objects are in diagonal alignment. */
export function isDiagonal(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return vectorIsDiagonal(v1, v2, epsilon);
}
/** Check if two objects are inline. */
export function isInline(o1, o2, lapping = 0.5, fn) {
    return resolveGridDistance(o1, o2, fn) <= lapping;
}
/** Check if two objects are adjacent. */
export function isAdjacent(o1, o2, epsilon = 0.001, fn) {
    const distance = resolveGridDistance(o1, o2, fn);
    return distance > (0.5 - epsilon) && distance < (1 + epsilon);
}
/** Check if two objects form a "window". */
export function isWindow(o1, o2, distance = 1.8, fn) {
    return resolveGridDistance(o1, o2, fn) > distance;
}
/** Check if two objects form a "slanted window". */
export function isSlantedWindow(o1, o2, fn) {
    return (isWindow(o1, o2, 1.8, fn) &&
        !isDiagonal(o1, o2, 0.001, fn) &&
        !isHorizontal(o1, o2, 0.001, fn) &&
        !isVertical(o1, o2, 0.001, fn));
}
export function mirrorCoordinate(x, axis) {
    return axis - 1 - x;
}
