/**
 * Pass this to wrapper object array `sort` function as an argument.
 *
 * ```ts
 * data.basicEvents.sort(sortObjectFn);
 * ```
 */
export function sortObjectFn(a, b) {
    return a.time - b.time;
}
/**
 * Pass this to wrapper note type array `sort` function as an argument.
 *
 * ```ts
 * data.chains.sort(sortNoteFn);
 * ```
 */
export function sortNoteFn(a, b) {
    if (Array.isArray(a.customData.coordinates) && Array.isArray(b.customData.coordinates)) {
        return (a.time - b.time ||
            a.customData.coordinates[0] - b.customData.coordinates[0] ||
            a.customData.coordinates[1] - b.customData.coordinates[1]);
    }
    if (Array.isArray(a.customData._position) && Array.isArray(b.customData._position)) {
        return (a.time - b.time ||
            a.customData._position[0] - b.customData._position[0] ||
            a.customData._position[1] - b.customData._position[1]);
    }
    return a.time - b.time || a.posX - b.posX || a.posY - b.posY;
}
/**
 * Pass this to schema v1 or v2 object array `sort` function as an argument.
 *
 * ```ts
 * data._events.sort(sortV2ObjectFn);
 * ```
 */
export function sortV2ObjectFn(a, b) {
    return a._time - b._time;
}
/**
 * Pass this to schema v1 or v2 note type array `sort` function as an argument.
 *
 * ```ts
 * data._notes.sort(sortV2NoteFn);
 * ```
 */
export function sortV2NoteFn(a, b) {
    if (Array.isArray(a._customData?._position) && Array.isArray(b._customData?._position)) {
        return (a._time - b._time ||
            a._customData._position[0] - b._customData._position[0] ||
            a._customData._position[1] - b._customData._position[1]);
    }
    return a._time - b._time || a._lineIndex - b._lineLayer || a._lineIndex - b._lineLayer;
}
/**
 * Pass this to schema v3 object array `sort` function as an argument.
 *
 * ```ts
 * data.basicBeatmapEvents.sort(sortV3ObjectFn);
 * ```
 */
export function sortV3ObjectFn(a, b) {
    return a.b - b.b;
}
/**
 * Pass this to schema v3 note type array `sort` function as an argument.
 *
 * ```ts
 * data.arcs.sort(sortV3NoteFn);
 * ```
 */
export function sortV3NoteFn(a, b) {
    if (Array.isArray(a.customData?.coordinates) && Array.isArray(b.customData?.coordinates)) {
        return (a.b - b.b ||
            a.customData.coordinates[0] - b.customData.coordinates[0] ||
            a.customData.coordinates[1] - b.customData.coordinates[1]);
    }
    return a.b - b.b || a.x - b.x || a.y - b.y;
}
