/**
 * Get implicit version based on beatmap type.
 *
 * The implicit version is used when the version is not specified
 * in the beatmap due to certain issues related within time period.
 */
export function implicitVersion(type) {
    switch (type) {
        case 'info':
        case 'difficulty':
            return '2.0.0';
        case 'lightshow':
            return '3.0.0';
        default:
            return '4.0.0';
    }
}
/** Get beatmap version from JSON. */
export function retrieveVersion(json) {
    const ver = json._version ?? json.version;
    if (typeof ver !== 'string') {
        return null;
    }
    return ver;
}
/**
 * Compare version between two versions.
 *
 * @returns -1 if current version is lower, 1 if higher, 0 if equal
 */
export function compareVersion(current, compareTo) {
    const verCurrent = getVersionArray(current);
    const verCompareTo = getVersionArray(compareTo);
    for (const num in verCurrent) {
        if (verCurrent[num] < verCompareTo[num]) {
            return -1;
        }
        if (verCurrent[num] > verCompareTo[num]) {
            return 1;
        }
    }
    return 0;
}
function getVersionArray(version) {
    return version.split('.').map((el) => parseInt(el));
}
