import { toV1Beatmap } from '../converter/toV1/beatmap.js';
import { toV1Info } from '../converter/toV1/info.js';
import { toV2AudioData } from '../converter/toV2/audioData.js';
import { toV2Beatmap } from '../converter/toV2/beatmap.js';
import { toV2Info } from '../converter/toV2/info.js';
import { toV3Beatmap } from '../converter/toV3/beatmap.js';
import { toV4AudioData } from '../converter/toV4/audioData.js';
import { toV4Beatmap } from '../converter/toV4/beatmap.js';
import { toV4Info } from '../converter/toV4/info.js';
/** Conversion function version map for beatmap info. */
export const infoConvertMap = {
    1: toV1Info,
    2: toV2Info,
    4: toV4Info,
};
/** Conversion function version map for beatmap audio data. */
export const audioDataConvertMap = {
    2: toV2AudioData,
    4: toV4AudioData,
};
/** Conversion function version map for beatmap data. */
export const beatmapConvertMap = {
    1: toV1Beatmap,
    2: toV2Beatmap,
    3: toV3Beatmap,
    4: toV4Beatmap,
};
export function convertBeatmap(type, targetVersion, data, sourceVersion) {
    switch (type) {
        case 'info': {
            const convert = infoConvertMap[targetVersion];
            return convert(data, sourceVersion);
        }
        case 'audioData': {
            const convert = audioDataConvertMap[targetVersion];
            return convert(data, sourceVersion);
        }
        case 'difficulty': {
            const convert = beatmapConvertMap[targetVersion];
            return convert(data, sourceVersion);
        }
        case 'lightshow': {
            const convert = beatmapConvertMap[targetVersion];
            return convert(data, sourceVersion);
        }
        default: {
            throw new Error('');
        }
    }
}
