import { optimizeDifficulty as optimizeV1Difficulty } from '../schema/v1/optimize/difficulty.js';
import { optimizeInfo as optimizeV1Info } from '../schema/v1/optimize/info.js';
import { optimizeDifficulty as optimizeV2Difficulty } from '../schema/v2/optimize/difficulty.js';
import { optimizeInfo as optimizeV2Info } from '../schema/v2/optimize/info.js';
import { optimizeDifficulty as optimizeV3Difficulty } from '../schema/v3/optimize/difficulty.js';
import { optimizeLightshow as optimizeV3Lightshow } from '../schema/v3/optimize/lightshow.js';
import { optimizeDifficulty as optimizeV4Difficulty } from '../schema/v4/optimize/difficulty.js';
import { optimizeInfo as optimizeV4Info } from '../schema/v4/optimize/info.js';
import { optimizeLightshow as optimizeV4Lightshow } from '../schema/v4/optimize/lightshow.js';
/** Optimize function version map for schema beatmap info. */
export const infoOptimizeMap = {
    1: optimizeV1Info,
    2: optimizeV2Info,
    4: optimizeV4Info,
};
/** Optimize function version map for schema beatmap difficulty. */
export const difficultyOptimizeMap = {
    1: optimizeV1Difficulty,
    2: optimizeV2Difficulty,
    3: optimizeV3Difficulty,
    4: optimizeV4Difficulty,
};
/** Optimize function version map for schema beatmap lightshow. */
export const lightshowOptimizeMap = {
    3: optimizeV3Lightshow,
    4: optimizeV4Lightshow,
};
export function optimizeBeatmap(type, ver, data, options) {
    switch (type) {
        case 'info': {
            const optimize = infoOptimizeMap[ver];
            return optimize(data, options);
        }
        case 'difficulty': {
            const optimize = difficultyOptimizeMap[ver];
            return optimize(data, options);
        }
        case 'lightshow': {
            const optimize = lightshowOptimizeMap[ver];
            return optimize(data, options);
        }
        default: {
            return data;
        }
    }
}
