import { difficulty as V1Difficulty } from '../schema/v1/difficulty.js';
import { info as V1Info } from '../schema/v1/info.js';
import { audioData as V2AudioData } from '../schema/v2/audioData.js';
import { difficulty as V2Difficulty } from '../schema/v2/difficulty.js';
import { info as V2Info } from '../schema/v2/info.js';
import { difficulty as V3Difficulty } from '../schema/v3/difficulty.js';
import { lightshow as V3Lightshow } from '../schema/v3/lightshow.js';
import { audioData as V4AudioData } from '../schema/v4/audioData.js';
import { difficulty as V4Difficulty } from '../schema/v4/difficulty.js';
import { info as V4Info } from '../schema/v4/info.js';
import { lightshow as V4Lightshow } from '../schema/v4/lightshow.js';
/** Schema version map for beatmap info. */
export const infoSchemaMap = {
    1: V1Info,
    2: V2Info,
    4: V4Info,
};
/** Schema version map for beatmap audio data. */
export const audioDataSchemaMap = {
    2: V2AudioData,
    4: V4AudioData,
};
/** Schema version map for beatmap difficulty. */
export const difficultySchemaMap = {
    1: V1Difficulty,
    2: V2Difficulty,
    3: V3Difficulty,
    4: V4Difficulty,
};
/** Schema version map for beatmap lightshow. */
export const lightshowSchemaMap = {
    3: V3Lightshow,
    4: V4Lightshow,
};
export function serializeBeatmap(type, ver, data) {
    switch (type) {
        case 'info': {
            const container = infoSchemaMap[ver];
            return container.serialize(data);
        }
        case 'audioData': {
            const container = audioDataSchemaMap[ver];
            return container.serialize(data);
        }
        case 'difficulty': {
            const container = difficultySchemaMap[ver];
            return container.serialize(data);
        }
        case 'lightshow': {
            const container = lightshowSchemaMap[ver];
            return container.serialize(data);
        }
        default: {
            throw new Error('');
        }
    }
}
export function deserializeBeatmap(type, ver, data) {
    switch (type) {
        case 'info': {
            const container = infoSchemaMap[ver];
            return container.deserialize(data);
        }
        case 'audioData': {
            const container = audioDataSchemaMap[ver];
            return container.deserialize(data);
        }
        case 'difficulty': {
            const container = difficultySchemaMap[ver];
            return container.deserialize(data);
        }
        case 'lightshow': {
            const container = lightshowSchemaMap[ver];
            return container.deserialize(data);
        }
        default: {
            throw new Error('');
        }
    }
}
