import { v } from '../../../../_deps.js';
import { entity, field, mask } from '../../helpers.js';
import { CharacteristicNameSchema, CustomContributorSchema, DifficultyNameSchema, } from '../../shared/declaration/mod.js';
/**
 * Schema declaration for v1 `Color`.
 */
export const CustomColorObjectSchema = v.object({
    r: v.number(),
    g: v.number(),
    b: v.number(),
});
/**
 * Schema declaration for v1 `Info Difficulty`.
 */
export const InfoDifficultySchema = v.object({
    difficulty: field(DifficultyNameSchema, {
        version: '1.0.0',
    }),
    difficultyRank: field(v.picklist([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]), {
        version: '1.0.0',
    }),
    audioPath: field(v.string(), {
        version: '1.0.0',
    }),
    jsonPath: field(v.string(), {
        version: '1.0.0',
    }),
    characteristic: field(CharacteristicNameSchema, {
        version: '1.0.0',
    }),
    offset: field(v.optional(v.number())),
    oldOffset: field(v.optional(v.number())),
    chromaToggle: field(v.optional(v.string())),
    customColors: field(v.optional(v.boolean())),
    difficultyLabel: field(v.optional(v.string())),
    colorLeft: field(v.optional(CustomColorObjectSchema)),
    colorRight: field(v.optional(CustomColorObjectSchema)),
    envColorLeft: field(v.optional(CustomColorObjectSchema)),
    envColorRight: field(v.optional(CustomColorObjectSchema)),
    obstacleColor: field(v.optional(CustomColorObjectSchema)),
});
/**
 * Schema declaration for v1 `Info`.
 */
export const InfoSchema = entity(() => '1.0.0', {
    songName: field(v.string(), {
        version: '1.0.0',
    }),
    songSubName: field(v.string(), {
        version: '1.0.0',
    }),
    authorName: field(v.string(), {
        version: '1.0.0',
    }),
    beatsPerMinute: field(v.number(), {
        version: '1.0.0',
    }),
    previewStartTime: field(v.number(), {
        version: '1.0.0',
    }),
    previewDuration: field(v.number(), {
        version: '1.0.0',
    }),
    coverImagePath: field(v.string(), {
        version: '1.0.0',
    }),
    environmentName: field(mask(v.string()), {
        version: '1.0.0',
    }),
    difficultyLevels: field(v.array(InfoDifficultySchema), {
        version: '1.0.0',
    }),
    oneSaber: field(v.boolean(), {
        version: '1.0.0',
    }),
    contributors: field(v.optional(v.array(CustomContributorSchema)), {
        version: '1.0.0',
    }),
    customEnvironment: field(v.optional(v.string()), {
        version: '1.0.0',
    }),
    customEnvironmentHash: field(v.optional(v.string()), {
        version: '1.0.0',
    }),
});
