import { deepCopy } from '../../../utils/misc/json.js';
import { createBeatmap } from '../../core/beatmap.js';
import { createDifficulty } from '../../core/difficulty.js';
import { createLightshow } from '../../core/lightshow.js';
import { FxType } from '../../shared/constants.js';
import { arc } from './arc.js';
import { basicEvent } from './basicEvent.js';
import { basicEventTypesWithKeywords } from './basicEventTypesWithKeywords.js';
import { bombNote } from './bombNote.js';
import { bpmEvent } from './bpmEvent.js';
import { chain } from './chain.js';
import { colorBoostEvent } from './colorBoostEvent.js';
import { colorNote } from './colorNote.js';
import { fxEventBoxGroup } from './fxEventBoxGroup.js';
import { lightColorEventBoxGroup } from './lightColorEventBoxGroup.js';
import { lightRotationEventBoxGroup } from './lightRotationEventBoxGroup.js';
import { lightTranslationEventBoxGroup } from './lightTranslationEventBoxGroup.js';
import { obstacle } from './obstacle.js';
import { rotationEvent } from './rotationEvent.js';
import { waypoint } from './waypoint.js';
/**
 * Schema serialization for v3 `Difficulty`.
 */
export const difficulty = {
    serialize(data) {
        const json = {
            version: '3.3.0',
            bpmEvents: data.difficulty.bpmEvents.map((x) => {
                return bpmEvent.serialize(x);
            }),
            rotationEvents: data.difficulty.rotationEvents.map((x) => {
                return rotationEvent.serialize(x);
            }),
            colorNotes: data.difficulty.colorNotes.map((x) => {
                return colorNote.serialize(x);
            }),
            bombNotes: data.difficulty.bombNotes.map((x) => {
                return bombNote.serialize(x);
            }),
            obstacles: data.difficulty.obstacles.map((x) => {
                return obstacle.serialize(x);
            }),
            sliders: data.difficulty.arcs.map((x) => {
                return arc.serialize(x);
            }),
            burstSliders: data.difficulty.chains.map((x) => {
                return chain.serialize(x);
            }),
            waypoints: data.lightshow.waypoints.map((x) => {
                return waypoint.serialize(x);
            }),
            basicBeatmapEvents: data.lightshow.basicEvents.map((x) => {
                return basicEvent.serialize(x);
            }),
            colorBoostBeatmapEvents: data.lightshow.colorBoostEvents.map((x) => {
                return colorBoostEvent.serialize(x);
            }),
            lightColorEventBoxGroups: data.lightshow.lightColorEventBoxGroups.map((x) => {
                return lightColorEventBoxGroup.serialize(x);
            }),
            lightRotationEventBoxGroups: data.lightshow.lightRotationEventBoxGroups.map((x) => {
                return lightRotationEventBoxGroup.serialize(x);
            }),
            lightTranslationEventBoxGroups: data.lightshow.lightTranslationEventBoxGroups.map((x) => {
                return lightTranslationEventBoxGroup.serialize(x);
            }),
            vfxEventBoxGroups: [],
            basicEventTypesWithKeywords: basicEventTypesWithKeywords.serialize(data.lightshow.basicEventTypesWithKeywords),
            _fxEventsCollection: {
                _fl: [],
                _il: [],
            },
            useNormalEventsAsCompatibleEvents: data.lightshow.useNormalEventsAsCompatibleEvents,
            customData: deepCopy(data.difficulty.customData),
        };
        for (const obj of data.lightshow.fxEventBoxGroups.map((x) => {
            return fxEventBoxGroup.serialize(x);
        })) {
            json.vfxEventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                obj.object.e.push(box.data);
                for (const evt of box.eventData) {
                    box.data.l.push(json._fxEventsCollection._fl.length);
                    json._fxEventsCollection._fl.push(evt);
                }
            }
        }
        return json;
    },
    deserialize(data, options) {
        const fx = data._fxEventsCollection?._fl;
        return createBeatmap({
            version: 3,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: createDifficulty({
                colorNotes: data.colorNotes?.map((x) => {
                    return colorNote.deserialize(x);
                }),
                bombNotes: data.bombNotes?.map((x) => {
                    return bombNote.deserialize(x);
                }),
                obstacles: data.obstacles?.map((x) => {
                    return obstacle.deserialize(x);
                }),
                arcs: data.sliders?.map((x) => {
                    return arc.deserialize(x);
                }),
                chains: data.burstSliders?.map((x) => {
                    return chain.deserialize(x);
                }),
                rotationEvents: data.rotationEvents?.map((x) => {
                    return rotationEvent.deserialize(x);
                }),
                bpmEvents: data.bpmEvents?.map((x) => {
                    return bpmEvent.deserialize(x);
                }),
                customData: data.customData,
            }),
            lightshow: createLightshow({
                waypoints: data.waypoints?.map((x) => {
                    return waypoint.deserialize(x);
                }),
                basicEvents: data.basicBeatmapEvents?.map((x) => {
                    return basicEvent.deserialize(x);
                }),
                colorBoostEvents: data.colorBoostBeatmapEvents?.map((x) => {
                    return colorBoostEvent.deserialize(x);
                }),
                lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map((x) => {
                    return lightColorEventBoxGroup.deserialize(x);
                }),
                lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map((x) => {
                    return lightRotationEventBoxGroup.deserialize(x);
                }),
                lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map((x) => {
                    return lightTranslationEventBoxGroup.deserialize(x);
                }),
                fxEventBoxGroups: data.vfxEventBoxGroups?.map((obj) => fxEventBoxGroup.deserialize({
                    object: { ...obj, t: FxType.FLOAT },
                    boxData: obj.e?.map((box) => ({
                        data: box,
                        eventData: box.l?.map((idx) => fx[idx]) ?? [],
                    })) ?? [],
                })),
                basicEventTypesWithKeywords: basicEventTypesWithKeywords.deserialize(data.basicEventTypesWithKeywords ?? {}),
                useNormalEventsAsCompatibleEvents: data.useNormalEventsAsCompatibleEvents,
            }),
        });
    },
};
