import { deepCopy } from '../../../utils/misc/json.js';
import { createLightTranslationEventBox } from '../../core/lightTranslationEventBox.js';
import { indexFilter } from './indexFilter.js';
import { lightTranslationEvent } from './lightTranslationEvent.js';
/**
 * Schema serialization for v3 `Light Translation Event Box`.
 */
export const lightTranslationEventBox = {
    serialize(data) {
        return {
            f: indexFilter.serialize(data.filter),
            w: data.beatDistribution,
            d: data.beatDistributionType,
            s: data.gapDistribution,
            t: data.gapDistributionType,
            a: data.axis,
            r: data.flip,
            b: data.affectFirst,
            i: data.easing,
            l: data.events.map((x) => {
                return lightTranslationEvent.serialize(x);
            }),
            customData: deepCopy(data.customData),
        };
    },
    deserialize(data) {
        return createLightTranslationEventBox({
            filter: indexFilter.deserialize(data.f ?? {}),
            beatDistribution: data.w,
            beatDistributionType: data.d,
            gapDistribution: data.s,
            gapDistributionType: data.t,
            axis: data.a,
            flip: data.r,
            affectFirst: data.b,
            easing: data.i,
            events: data.l?.map((x) => {
                return lightTranslationEvent.deserialize(x);
            }),
            customData: data.customData,
        });
    },
};
