import { deepCopy } from '../../../utils/misc/json.js';
import { createBeatmap } from '../../core/beatmap.js';
import { createDifficulty } from '../../core/difficulty.js';
import { createLightshow } from '../../core/lightshow.js';
import { FxType } from '../../shared/constants.js';
import { basicEvent } from './basicEvent.js';
import { colorBoostEvent } from './colorBoostEvent.js';
import { fxEventBoxGroup } from './fxEventBoxGroup.js';
import { lightColorEventBoxGroup } from './lightColorEventBoxGroup.js';
import { lightRotationEventBoxGroup } from './lightRotationEventBoxGroup.js';
import { lightTranslationEventBoxGroup } from './lightTranslationEventBoxGroup.js';
/**
 * Schema serialization for v3 `Lightshow`.
 */
export const lightshow = {
    serialize(data) {
        const json = {
            basicBeatmapEvents: data.lightshow.basicEvents.map((x) => {
                return basicEvent.serialize(x);
            }),
            colorBoostBeatmapEvents: data.lightshow.colorBoostEvents.map((x) => {
                return colorBoostEvent.serialize(x);
            }),
            lightColorEventBoxGroups: data.lightshow.lightColorEventBoxGroups.map((x) => {
                return lightColorEventBoxGroup.serialize(x);
            }),
            lightRotationEventBoxGroups: data.lightshow.lightRotationEventBoxGroups.map((x) => {
                return lightRotationEventBoxGroup.serialize(x);
            }),
            lightTranslationEventBoxGroups: data.lightshow.lightTranslationEventBoxGroups.map((x) => {
                return lightTranslationEventBoxGroup.serialize(x);
            }),
            vfxEventBoxGroups: [],
            _fxEventsCollection: {
                _fl: [],
                _il: [],
            },
            customData: deepCopy(data.lightshow.customData),
        };
        for (const obj of data.lightshow.fxEventBoxGroups.map((x) => {
            return fxEventBoxGroup.serialize(x);
        })) {
            json.vfxEventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                obj.object.e.push(box.data);
                for (const evt of box.eventData) {
                    box.data.l.push(json._fxEventsCollection._fl.length);
                    json._fxEventsCollection._fl.push(evt);
                }
            }
        }
        return json;
    },
    deserialize(data, options) {
        const fx = data._fxEventsCollection?._fl;
        return createBeatmap({
            version: 3,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: createDifficulty(),
            lightshow: createLightshow({
                basicEvents: data.basicBeatmapEvents?.map((x) => {
                    return basicEvent.deserialize(x);
                }),
                colorBoostEvents: data.colorBoostBeatmapEvents?.map((x) => {
                    return colorBoostEvent.deserialize(x);
                }),
                lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map((x) => {
                    return lightColorEventBoxGroup.deserialize(x);
                }),
                lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map((x) => {
                    return lightRotationEventBoxGroup.deserialize(x);
                }),
                lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map((x) => {
                    return lightTranslationEventBoxGroup.deserialize(x);
                }),
                fxEventBoxGroups: data.vfxEventBoxGroups?.map((obj) => fxEventBoxGroup.deserialize({
                    object: { ...obj, t: FxType.FLOAT },
                    boxData: obj.e?.map((box) => ({
                        data: box,
                        eventData: box.l?.map((idx) => fx[idx]) ?? [],
                    })) ?? [],
                })),
                customData: data.customData,
            }),
        });
    },
};
