import { v } from '../../../../_deps.js';
import { entity, field, mask } from '../../helpers.js';
import { VersionSchema } from '../../shared/declaration/mod.js';
/**
 * Schema declaration for v4 `Audio Data BPM`.
 */
export const AudioDataBPMSchema = v.object({
    si: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    ei: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    sb: field(v.number(), {
        version: '4.0.0',
    }),
    eb: field(v.number(), {
        version: '4.0.0',
    }),
});
/**
 * Schema declaration for v4 `Audio Data LUFS`.
 */
export const AudioDataLUFSSchema = v.object({
    si: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    ei: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    l: field(v.number(), {
        version: '4.0.0',
    }),
});
/**
 * Schema declaration for v4 `Audio`.
 */
export const AudioDataSchema = entity((x) => x.version || '4.0.0', {
    version: field(mask(VersionSchema), {
        version: '4.0.0',
    }),
    songChecksum: field(v.string(), {
        version: '4.0.0',
    }),
    songSampleCount: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    songFrequency: field(v.pipe(v.number(), v.integer(), v.minValue(0)), {
        version: '4.0.0',
    }),
    bpmData: field(v.array(AudioDataBPMSchema), {
        version: '4.0.0',
    }),
    lufsData: field(v.array(AudioDataLUFSSchema), {
        version: '4.0.0',
    }),
});
