import { deepCopy } from '../../../utils/misc/json.js';
import { createLightTranslationEventBox } from '../../core/lightTranslationEventBox.js';
import { indexFilter } from './indexFilter.js';
import { lightTranslationEvent } from './lightTranslationEvent.js';
/**
 * Schema serialization for v4 `Light Translation Event Box`.
 */
export const lightTranslationEventBox = {
    serialize(data) {
        return {
            data: {
                w: data.beatDistribution,
                d: data.beatDistributionType,
                s: data.gapDistribution,
                t: data.gapDistributionType,
                b: data.affectFirst,
                e: data.easing,
                a: data.axis,
                f: data.flip,
                customData: deepCopy(data.customData),
            },
            eventData: data.events.map((x) => {
                return lightTranslationEvent.serialize(x);
            }),
            filterData: indexFilter.serialize(data.filter),
        };
    },
    deserialize(data) {
        return createLightTranslationEventBox({
            filter: indexFilter.deserialize(data.filterData ?? {}),
            beatDistribution: data.data?.w,
            beatDistributionType: data.data?.d,
            gapDistribution: data.data?.s,
            gapDistributionType: data.data?.t,
            affectFirst: data.data?.b,
            easing: data.data?.e,
            axis: data.data?.a,
            flip: data.data?.f,
            events: data.eventData?.map((x) => {
                return lightTranslationEvent.deserialize(x);
            }),
            customData: data.data?.customData,
        });
    },
};
