import type { Vector2 } from '../../types/vector.js';
import { NoteDirection } from '../../types/beatmap/shared/constants.js';
export * from '../../types/beatmap/shared/custom/constants.js';
export * from '../../types/beatmap/shared/constants.js';
/** Also known as lane count. */
export declare const LINE_COUNT = 4;
/** Size of grid in unity unit */
export declare const LANE_SIZE = 0.6;
/** Size of regular note in unity unit */
export declare const NOTE_SIZE: readonly [0.5, 0.5];
/** Y-offset of note on grid in unity unit */
export declare const NoteYOffset: {
    0: number;
    1: number;
    2: number;
};
/**
 * Cut angle corresponding to the `direction`.
 * ```ts
 * 0 (UP) -> 180
 * 1 (DOWN) -> 0
 * 2 (LEFT) -> 270
 * 3 (RIGHT) -> 90
 * 4 (UP_LEFT) -> 225
 * 5 (UP_RIGHT) -> 135
 * 6 (DOWN_LEFT) -> 315
 * 7 (DOWN_RIGHT) -> 45
 * 8 (ANY) -> 0
 * ```
 *
 * **NOTE:** Rotation in counter-clockwise.
 */
export declare const NoteDirectionAngle: {
    readonly [d in NoteDirection]: number;
};
/**
 * Opposite direction corresponding to the `direction`.
 * ```ts
 * 0 (UP) -> 1 (DOWN)
 * 1 (DOWN) -> 0 (UP)
 * 2 (LEFT) -> 3 (RIGHT)
 * 3 (RIGHT) -> 2 (LEFT)
 * 4 (UP_LEFT) -> 7 (DOWN_RIGHT)
 * 5 (UP_RIGHT) -> 6 (DOWN_LEFT)
 * 6 (DOWN_LEFT) -> 5 (UP_RIGHT)
 * 7 (DOWN_RIGHT) -> 4 (UP_LEFT)
 * 8 (ANY) -> 8 (ANY)
 * ```
 */
export declare const NoteDirectionFlip: {
    readonly [d in NoteDirection]: NoteDirection;
};
/**
 * Array index mapped to tuple of `posX` and `posY` corresponding to the `direction`.
 * ```ts
 * 0 -> [0, 1]
 * 1 -> [0, -1]
 * 2 -> [-1, 0]
 * 3 -> [1, 0]
 * 4 -> [-1, 1]
 * 5 -> [1, 1]
 * 6 -> [-1, -1]
 * 7 -> [1, -1]
 * 8 -> [0, 0]
 * ```
 */
export declare const NoteDirectionSpace: {
    readonly [d in NoteDirection]: Readonly<Vector2>;
};
/** Unused as of beatmap v3, useful for conversion reason. */
export declare const EventLaneRotationValue: {
    readonly [key: number]: number;
};
/** Unused as of beatmap v3, useful for conversion reason. */
export declare const RotationValueEventValue: {
    readonly [key: number]: number;
};
//# sourceMappingURL=constants.d.ts.map