// deno-lint-ignore-file no-explicit-any
import { logger } from '../../logger.js';
import { audioDataCompatibilityMap, difficultyCompatibilityMap, infoCompatibilityMap, lightshowCompatibilityMap, } from '../mapping/compatibility.js';
function tag(name) {
    return ['validator', name];
}
const defaultOptions = {
    enabled: true,
    throwOn: { incompatibleObject: true, mappingExtensions: false },
};
/**
 * Check if the beatmap is compatible with the version schema.
 */
export function compatibilityCheck(type, data, version, options) {
    const opt = {
        enabled: options?.enabled ?? defaultOptions.enabled,
        throwOn: {
            ...defaultOptions.throwOn,
            ...options?.throwOn,
        },
    };
    let compatibilityMap = {};
    switch (type) {
        case 'info':
            compatibilityMap = infoCompatibilityMap;
            break;
        case 'audioData':
            compatibilityMap = audioDataCompatibilityMap;
            break;
        case 'difficulty':
            compatibilityMap = difficultyCompatibilityMap;
            break;
        case 'lightshow':
            compatibilityMap = lightshowCompatibilityMap;
            break;
    }
    const compatFn = compatibilityMap[version];
    if (compatFn) {
        logger.tInfo(tag('compatibilityCheck'), 'Checking beatmap compatibility for ' + type + ' with version', version);
        compatFn(data, opt);
    }
    return data;
}
