import { logger } from '../../logger.js';
import { audioSchemaMap, difficultyCheckMap, infoCheckMap, lightshowCheckMap, } from '../mapping/validator.js';
import { schemaCheck } from './schema.js';
function tag(name) {
    return ['validator', name];
}
const defaultOptions = {
    enabled: true,
    throwOn: {
        unused: false,
        missing: true,
        ignoreOptional: false,
        nullish: true,
        wrongType: true,
        notInt: false,
        notUnsigned: false,
    },
};
/**
 * Validate the beatmap JSON data against the schema.
 */
export function validateJSON(type, data, version, options) {
    const opt = {
        enabled: options?.enabled ?? defaultOptions.enabled,
        throwOn: {
            ...defaultOptions.throwOn,
            ...options?.throwOn,
        },
    };
    logger.tInfo(tag('validateJSON'), 'Validating beatmap JSON for ' + type + ' with version', version);
    let schema;
    switch (type) {
        case 'info': {
            schema = infoCheckMap[version];
            break;
        }
        case 'audioData': {
            schema = audioSchemaMap[version];
            break;
        }
        case 'difficulty': {
            schema = difficultyCheckMap[version];
            break;
        }
        case 'lightshow': {
            schema = lightshowCheckMap[version];
            break;
        }
    }
    schemaCheck(data, schema, type, undefined, opt.throwOn);
    return data;
}
