import { deepCopy } from '../../../../utils/misc/json.js';
/** **IMPORTANT:** Manually adjust block to be exactly 1x1x1 unity unit (1x1x1 scale does usually not work) */
export class EnvironmentBlock {
    constructor(data, anchor) {
        Object.defineProperty(this, "data", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "anchor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.data = data;
        this.anchor = anchor;
    }
    static create(data, anchor) {
        return new this(data, anchor);
    }
    place(options, insertTo) {
        const d = deepCopy(this.data);
        const scale = options.scale
            ? (d.scale ? d.scale : [1, 1, 1]).map((s, i) => s * options.scale[i])
            : d.scale;
        const position = options.position
            ? options.position.map((p, i) => p + this.anchor[i] * (options.scale?.[i] || 1))
            : (d.position ? d.position : [0, 0, 0]).map((p, i) => p + this.anchor[i] * (options.scale?.[i] ?? 1));
        const rotation = options.rotation
            ? options.rotation.map((r, i) => r + (d.rotation?.[i] ?? 0))
            : d.rotation;
        const components = {
            ...d.components,
        };
        if (typeof components.ILightWithId?.type === 'number') {
            components.ILightWithId.lightID = EnvironmentBlock.startLightID + EnvironmentBlock.index++;
            if (typeof options.type === 'number') {
                components.ILightWithId.type = options.type;
                components.ILightWithId.lightID = options.id;
            }
        }
        if (options.track) {
            d.track = options.track;
        }
        const data = {
            ...d,
            scale,
            position,
            rotation,
        };
        if (Object.entries(components).length) {
            data.components = components;
        }
        if (insertTo) {
            insertTo.push(data);
            return;
        }
        return data;
    }
}
Object.defineProperty(EnvironmentBlock, "startLightID", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 100
});
Object.defineProperty(EnvironmentBlock, "index", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 0
});
