import { deepCopy } from '../../../../utils/misc/json.js';
import { vectorAdd, vectorMul } from '../../../../utils/math/vector.js';
import { degToRad } from '../../../../utils/math/trigonometry.js';
export class EnvironmentGroup {
    constructor(data, anchor = [0, 0, 0]) {
        Object.defineProperty(this, "data", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "anchor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.data = data;
        this.anchor = anchor;
    }
    static create(data, anchor = [0, 0, 0]) {
        return new this(data, anchor);
    }
    place(options, insertTo) {
        const data = deepCopy(this.data);
        data.forEach((d) => {
            if (d.rotation) {
                d.rotation = vectorAdd(d.rotation, options.rotation);
            }
            else
                d.rotation = options.rotation;
            // im only doing Y rotation for now
            if (options.rotation && d.position) {
                const yRad = degToRad(options.rotation[1]);
                const cos = Math.cos(yRad);
                const sin = Math.sin(yRad);
                const nx = cos * d.position[0] + sin * d.position[2];
                const ny = cos * d.position[2] - sin * d.position[0];
                d.position[0] = nx;
                d.position[2] = ny;
            }
            d.position ??= this.anchor;
            d.position = d.position.map((p, i) => (this.anchor[i] + p) * (options.scale?.[i] ?? 1) +
                (options.position?.[i] ?? 0));
            d.scale = vectorMul(d.scale, options.scale);
            if (typeof d.components?.ILightWithId?.type === 'number' &&
                typeof options.type === 'number') {
                d.components.ILightWithId.type = options.type;
            }
        });
        if (insertTo) {
            insertTo.push(...data);
            return;
        }
        return data;
    }
}
