import { logger } from '../../../logger.js';
import { convertColorType } from '../../../utils/colors/convertor.js';
import { lerpColor } from '../../../utils/colors/helpers.js';
import { normalize } from '../../../utils/math/helpers.js';
import { settings } from './settings.js';
function tag(name) {
    return ['ext', 'chroma', 'color', name];
}
export function setColor(objects, options) {
    const opt = {
        color: options.color,
        colorType: options.colorType ?? (settings.colorType || 'hsva'),
    };
    const color = convertColorType(opt.color, opt.colorType);
    objects.forEach((obj) => {
        obj.customData.color = color;
    });
}
export function setColorGradient(objects, options) {
    if (!objects.length) {
        logger.tWarn(tag('setColorGradient'), 'No object(s) received.');
        return;
    }
    const opt = {
        offsetStart: options.offsetStart,
        offsetEnd: options.offsetEnd,
        colorStart: options.colorStart,
        colorEnd: options.colorEnd,
        colorType: options.colorType ?? 'rgba',
        easingColor: options.easingColor ??
            function (x) {
                return x;
            },
    };
    const startTime = objects.at(0).time + opt.offsetStart;
    const endTime = objects.at(-1).time + opt.offsetEnd;
    objects.forEach((obj) => {
        const norm = normalize(obj.time, startTime, endTime);
        const color = lerpColor(opt.colorStart, opt.colorEnd, opt.easingColor(norm), opt.colorType);
        obj.customData.color = color;
    });
}
export function setColorRandom(objects, options) {
    const opt = {
        offsetStart: options.offsetStart,
        offsetEnd: options.offsetEnd,
        color1: options.color1,
        color2: options.color2,
        colorType: options.colorType ?? 'rgba',
    };
    let random = 0;
    let prevTime = 0;
    for (let i = 0, l = objects.length; i < l; i++) {
        if (objects[i].time > prevTime + 0.001) {
            random = Math.random();
        }
        const color = lerpColor(opt.color1, opt.color2, random, opt.colorType);
        objects[i].customData._color = color;
        prevTime = objects[i].time;
    }
}
