import { NoteJumpSpeed } from '../../../beatmap/helpers/njs.js';
import { TimeProcessor } from '../../../beatmap/helpers/timeProcessor.js';
import type { EasingFunction } from '../../../types/easings.js';
import type { INEObject } from './types/object.js';
/**
 * Set NJS to object from start to end object.
 *
 * **NOTE:** JD input will override NJS offset.
 */
export declare function setNjs<T extends Pick<INEObject, 'customData'>>(objects: T[], options: {
    timeProc: TimeProcessor;
    njs: NoteJumpSpeed | number;
    offset?: number;
    jd?: number;
}): void;
/**
 * Simultaneously spawn the object from start to end object.
 *
 * Speed determines how fast should note spawn from start to end. (1 is regular speed)
 *
 * **NOTE:** JD input will override NJS offset.
 */
export declare function simultaneousSpawn<T extends Pick<INEObject, 'time' | 'customData'>>(objects: T[], options: {
    timeProc: TimeProcessor;
    njs: NoteJumpSpeed | number;
    njsOverride?: boolean;
    jd?: number;
    spawnBeatOffset?: number;
    speed: number;
}): void;
/**
 * Gradually change NJS for objects from start to end objects.
 *
 * **NOTE:** JD input will override NJS offset.
 */
export declare function gradientNjs<T extends Pick<INEObject, 'time' | 'customData'>>(objects: T[], options: {
    timeProc: TimeProcessor | number;
    njsStart: number;
    njsEnd: number;
    njsOffset?: NoteJumpSpeed | number | null;
    jd?: number;
    easing?: EasingFunction;
}): void;
//# sourceMappingURL=njs.d.ts.map