import type { IWrapBaseNote } from '../../types/beatmap/wrapper/baseNote.js';
/** Check if current note can be swung into from previous note. */
export declare function isEndNote<T extends Pick<IWrapBaseNote, 'posX' | 'posY' | 'direction'>>(endNote: T, startNote: T, maybeDirection: number, angleTolerance?: number): boolean;
/**
 * Check if the note intersect on swing path by angle and distance.
 *
 * Can be used to check for hitbox collision or handclap.
 *
 * ```ts
 * if (isIntersect(note1, note2, [[20, 1.5]])) {}
 * ```
 */
export declare function isIntersectNote<T extends Pick<IWrapBaseNote, 'posX' | 'posY' | 'direction'>>(noteA: T, noteB: T, angleDistances: [range: number, distance: number, offset?: number][], ahead?: boolean): [noteAIntersect: boolean, noteBIntersect: boolean];
export declare function predictDirection<T extends Pick<IWrapBaseNote, 'time' | 'posX' | 'posY' | 'direction'>>(endNote: T, startNote: T): number;
/**
 * Check the angle equality of the two notes.
 * @param {(T|number|null)} noteA - First beatmap note, note `direction`, or null value
 * @param {(T|number|null)} noteB - Second beatmap note, note `direction`, or null value
 * @param {number} angleTol - Angle tolerance
 * @param {boolean} equal - If it should check inner or outer angle
 * @returns {boolean} If condition is met
 */
export declare function checkDirection<T extends Pick<IWrapBaseNote, 'direction'>>(noteA: T | number | null, noteB: T | number | null, angleTol: number, equal: boolean): boolean;
//# sourceMappingURL=note.d.ts.map