import { isOldChromaEventValue } from '../../beatmap/helpers/core/basicEvent.js';
import { hasChromaEventV2, hasChromaEventV3 } from '../../beatmap/helpers/modded/has.js';
import { EventList } from '../../beatmap/shared/environment.js';
import { isValidEventType } from '../../mod.js';
/**
 * Count number of type of events with their properties in given array and return a event count object.
 * ```ts
 * const list = count(events);
 * console.log(list);
 * ```
 */
export function countEvent(events, boost, environment = 'DefaultEnvironment', version = 2) {
    const commonEvent = EventList[environment]?.[0] ?? EventList['DefaultEnvironment'][0];
    const eventCount = {};
    for (let i = commonEvent.length - 1; i >= 0; i--) {
        eventCount[commonEvent[i]] = {
            total: 0,
            chroma: 0,
            chromaOld: 0,
        };
    }
    const hasChroma = version >= 3 ? hasChromaEventV3 : hasChromaEventV2;
    eventCount[5] = {
        total: boost.length,
        chroma: 0,
        chromaOld: 0,
    };
    for (let i = events.length - 1; i >= 0; i--) {
        if (isValidEventType(events[i].type)) {
            if (!eventCount[events[i].type]) {
                eventCount[events[i].type] = {
                    total: 0,
                    chroma: 0,
                    chromaOld: 0,
                };
            }
            eventCount[events[i].type].total++;
            if (hasChroma(events[i])) {
                eventCount[events[i].type].chroma++;
            }
            if (isOldChromaEventValue(events[i].value)) {
                eventCount[events[i].type].chromaOld++;
            }
        }
    }
    return eventCount;
}
