declare enum LogLevels {
    NONE = 0,
    ERROR = 1,
    WARN = 2,
    INFO = 3,
    DEBUG = 4,
    TRACE = 5
}
/** Simple logging system class. */
export declare class Logger {
    #private;
    static readonly LogLevels: typeof LogLevels;
    static LogPrefixes: Map<LogLevels, string>;
    set logLevel(value: LogLevels);
    get logLevel(): LogLevels;
    set tagPrint(fn: (tags: string[], level: LogLevels) => string);
    get tagPrint(): (tags: string[], level: LogLevels) => string;
    set untagged(value: string);
    get untagged(): string;
    private log;
    /**
     * Set logging level to filter various information.
     * ```ts
     * 0 -> None
     * 1 -> Error
     * 2 -> Warn
     * 3 -> Info
     * 4 -> Debug
     * 5 -> Trace
     * ```
     */
    setLevel(level: LogLevels): void;
    tTrace(tags: string[], ...args: any[]): void;
    tDebug(tags: string[], ...args: any[]): void;
    tInfo(tags: string[], ...args: any[]): void;
    tWarn(tags: string[], ...args: any[]): void;
    tError(tags: string[], ...args: any[]): void;
    trace(...args: any[]): void;
    debug(...args: any[]): void;
    info(...args: any[]): void;
    warn(...args: any[]): void;
    error(...args: any[]): void;
}
/**
 * Logging instance used in the module.
 *
 * Can be replaced as needed.
 */
export declare let logger: Logger;
export {};
//# sourceMappingURL=logger.d.ts.map