// deno-lint-ignore-file no-explicit-any
import { loadBeatmap } from '../beatmap/loader/_main.js';
import { globals } from '../globals.js';
import { readJSONFile, readJSONFileSync } from '../shims/_json.js';
import { path } from '../shims/path.js';
const defaultOptions = {
    directory: '',
    load: {},
};
export function tag(name) {
    return ['reader', name];
}
export function handleRead(type, src, version, options = {}) {
    const ver = typeof version === 'number' ? version : null;
    const opt = (typeof version !== 'number' ? version : options) ?? {};
    const p = path.resolve(opt.directory ?? (defaultOptions.directory || globals.directory), src);
    const [posttransformer, ...postprocesses] = [
        ...(opt.load?.postprocess ?? []),
    ].reverse();
    return readJSONFile(p).then((data) => {
        return loadBeatmap(type, data, ver, {
            ...opt.load,
            postprocess: [
                ...postprocesses,
                (x) => {
                    if (type === 'lightshow' && 'lightshowFilename' in x) {
                        x.lightshowFilename = path.basename(p);
                    }
                    else
                        x.filename = path.basename(p);
                    return posttransformer ? posttransformer(x) : x;
                },
            ],
        });
    });
}
export function handleReadSync(type, src, version, options = {}) {
    const ver = typeof version === 'number' ? version : null;
    const opt = (typeof version !== 'number' ? version : options) ?? {};
    const p = path.resolve(opt.directory ?? (defaultOptions.directory || globals.directory), src);
    const [posttransformer, ...postprocesses] = [
        ...(opt.load?.postprocess ?? []),
    ].reverse();
    const d = loadBeatmap(type, readJSONFileSync(p), ver, {
        ...opt.load,
        postprocess: [
            ...postprocesses,
            (x) => {
                if (type === 'lightshow' && 'lightshowFilename' in x) {
                    x.lightshowFilename = path.basename(p);
                }
                else
                    x.filename = path.basename(p);
                return posttransformer ? posttransformer(x) : x;
            },
        ],
    });
    return d;
}
