import { readDifficultyFile, readDifficultyFileSync } from './difficulty.js';
import { readLightshowFileSync } from './lightshow.js';
export async function readFromInfo(info, options = {}) {
    const ary = [];
    for (const d of info.difficulties) {
        const [posttransformer, ...postprocesses] = [
            ...(options.load?.postprocess ?? []),
        ].reverse();
        const beatmap = await readDifficultyFile(d.filename, {
            ...options,
            load: {
                postprocess: [
                    ...postprocesses,
                    (x) => {
                        if (x.version === 4) {
                            const light = readLightshowFileSync(d.lightshowFilename, {
                                directory: options.directory ?? options.directory,
                            });
                            x.lightshow = light.lightshow;
                            x.lightshowFilename = light.lightshowFilename;
                        }
                        return x;
                    },
                    posttransformer,
                ],
            },
        });
        ary.push({ info: d, beatmap });
    }
    return ary;
}
export function readFromInfoSync(info, options = {}) {
    const ary = [];
    for (const d of info.difficulties) {
        const [posttransformer, ...postprocesses] = [
            ...(options.load?.postprocess ?? []),
        ].reverse();
        const beatmap = readDifficultyFileSync(d.filename, {
            ...options,
            load: {
                postprocess: [
                    ...postprocesses,
                    (x) => {
                        if (x.version === 4) {
                            const light = readLightshowFileSync(d.lightshowFilename, {
                                directory: options.directory,
                            });
                            x.lightshow = light.lightshow;
                            x.lightshowFilename = light.lightshowFilename;
                        }
                        return x;
                    },
                    posttransformer,
                ],
            },
        });
        ary.push({ info: d, beatmap });
    }
    return ary;
}
