import type { InferBeatmap, InferBeatmapSerial, InferBeatmapVersion } from '../types/beatmap/shared/infer.js';
import type { IReadOptions } from '../types/bsmap/reader.js';
import type { LooseAutocomplete } from '../types/utils.js';
/**
 * Asynchronously read beatmap info file.
 *
 * @example
 * ```ts
 * readInfoFile('Info.dat', 4).then((data) => console.log(data));
 * ```
 *
 * Default to `Info.dat` if path not provided.
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readInfoFile<TVersion extends InferBeatmapVersion<'info'>, TWrapper extends Record<string, any> = InferBeatmap<'info'>, TSerial extends Record<string, any> = InferBeatmapSerial<'info', TVersion>>(path?: LooseAutocomplete<'Info.dat' | 'info.dat'>, version?: TVersion | null, options?: IReadOptions<'info', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
export declare function readInfoFile<TVersion extends InferBeatmapVersion<'info'>, TWrapper extends Record<string, any> = InferBeatmap<'info'>, TSerial extends Record<string, any> = InferBeatmapSerial<'info', TVersion>>(path?: LooseAutocomplete<'Info.dat' | 'info.dat'>, options?: IReadOptions<'info', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
/**
 * Synchronously read beatmap info file.
 *
 * @example
 * ```ts
 * const info = readInfoFileSync('Info.dat', 4);
 * console.log(info);
 * ```
 *
 * Default to `Info.dat` if path not provided.
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readInfoFileSync<TVersion extends InferBeatmapVersion<'info'>, TWrapper extends Record<string, any> = InferBeatmap<'info'>, TSerial extends Record<string, any> = InferBeatmapSerial<'info', TVersion>>(path?: LooseAutocomplete<'Info.dat' | 'info.dat'>, version?: TVersion | null, options?: IReadOptions<'info', TVersion, TWrapper, TSerial>): TWrapper;
export declare function readInfoFileSync<TVersion extends InferBeatmapVersion<'info'>, TWrapper extends Record<string, any> = InferBeatmap<'info'>, TSerial extends Record<string, any> = InferBeatmapSerial<'info', TVersion>>(path?: LooseAutocomplete<'Info.dat' | 'info.dat'>, options?: IReadOptions<'info', TVersion, TWrapper, TSerial>): TWrapper;
//# sourceMappingURL=info.d.ts.map