import type { GenericLightshowFilename } from '../types/beatmap/shared/filename.js';
import type { InferBeatmap, InferBeatmapSerial, InferBeatmapVersion } from '../types/beatmap/shared/infer.js';
import type { IReadOptions } from '../types/bsmap/reader.js';
import type { LooseAutocomplete } from '../types/utils.js';
/**
 * Asynchronously read beatmap lightshow file.
 * ```ts
 * readLightshowFile('Lightshow.dat', 4).then((data) => console.log(data));
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readLightshowFile<TVersion extends InferBeatmapVersion<'lightshow'>, TWrapper extends Record<string, any> = InferBeatmap<'lightshow'>, TSerial extends Record<string, any> = InferBeatmapSerial<'lightshow', TVersion>>(path: LooseAutocomplete<GenericLightshowFilename>, version?: TVersion | null, options?: IReadOptions<'lightshow', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
export declare function readLightshowFile<TVersion extends InferBeatmapVersion<'lightshow'>, TWrapper extends Record<string, any> = InferBeatmap<'lightshow'>, TSerial extends Record<string, any> = InferBeatmapSerial<'lightshow', TVersion>>(path: LooseAutocomplete<GenericLightshowFilename>, options?: IReadOptions<'lightshow', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
/**
 * Synchronously read beatmap lightshow file.
 * ```ts
 * const lightshow = readLightshowFileSync('Lightshow.dat', 4);
 * console.log(lightshow);
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readLightshowFileSync<TVersion extends InferBeatmapVersion<'lightshow'>, TWrapper extends Record<string, any> = InferBeatmap<'lightshow'>, TSerial extends Record<string, any> = InferBeatmapSerial<'lightshow', TVersion>>(path: LooseAutocomplete<GenericLightshowFilename>, version?: TVersion | null, options?: IReadOptions<'lightshow', TVersion, TWrapper, TSerial>): TWrapper;
export declare function readLightshowFileSync<TVersion extends InferBeatmapVersion<'lightshow'>, TWrapper extends Record<string, any> = InferBeatmap<'lightshow'>, TSerial extends Record<string, any> = InferBeatmapSerial<'lightshow', TVersion>>(path: LooseAutocomplete<GenericLightshowFilename>, options?: IReadOptions<'lightshow', TVersion, TWrapper, TSerial>): TWrapper;
//# sourceMappingURL=lightshow.d.ts.map