// @ts-ignore: trick
import fsMod from './_fs.js';
// @ts-ignore: trick
import fspMod from './_fsp.js';
function noFsFunctionProvided() {
    throw new Error('`fs` function not provided; please supply `fs` function inside the `fs` object from the module');
}
// @ts-ignore: trick
const _fs = fsMod;
// @ts-ignore: trick
const _fsp = fspMod;
/**
 * Wrapper for use in `read` and `write`.
 *
 * Do not use this as a mean to perform `fs` operations.
 * This module only serve to provide cross-platform compatibility.
 *
 * If your runtime or vendor does not contain `node:` built-in module,
 * you should provide `fs` function inside the `fs` object from the module.
 * If you are creating a web app, you may ignore this and use `load` and `save`.
 */
export const fs = {
    readTextFile: noFsFunctionProvided,
    readTextFileSync: noFsFunctionProvided,
    writeTextFile: noFsFunctionProvided,
    writeTextFileSync: noFsFunctionProvided,
};
if (typeof Deno !== 'undefined') {
    fs.readTextFile = Deno.readTextFile || noFsFunctionProvided;
    fs.readTextFileSync = Deno.readTextFileSync || noFsFunctionProvided;
    fs.writeTextFile = Deno.writeTextFile || noFsFunctionProvided;
    fs.writeTextFileSync = Deno.writeTextFileSync || noFsFunctionProvided;
}
else {
    if (typeof Bun !== 'undefined') {
        fs.readTextFile = (path) => {
            return Bun.file(path).text();
        };
        fs.writeTextFile = Bun.write;
    }
    else if (_fsp) {
        fs.readTextFile = (path) => {
            return _fsp.readFile(path, 'utf8');
        };
        fs.writeTextFile = (path, data) => {
            return _fsp.writeFile(path, data, 'utf8');
        };
    }
    if (_fs) {
        fs.readTextFileSync = (path) => {
            return _fs?.readFileSync(path, 'utf8');
        };
        fs.writeTextFileSync = (path, data) => {
            return _fs?.writeFileSync(path, data, 'utf8');
        };
    }
}
