import type { CharacteristicName } from '../shared/characteristic.js';
import type { EnvironmentAllName } from '../shared/environment.js';
import type { DifficultyName } from '../shared/difficulty.js';
import type { IItem } from './item.js';
import type { ICustomDataInfo } from './custom/info.js';
import type { ICustomDataInfoBeatmap } from './custom/info.js';
/**
 * Schema for v4 `Info`.
 */
export interface IInfo extends IItem {
    version: '4.0.0' | '4.0.1';
    song: IInfoSong;
    audio: IInfoAudio;
    songPreviewFilename: string;
    coverImageFilename: string;
    environmentNames: EnvironmentAllName[];
    colorSchemes: IInfoColorScheme[];
    difficultyBeatmaps: IInfoBeatmap[];
    customData?: ICustomDataInfo;
}
/**
 * Song schema for v4 `Info`.
 */
export interface IInfoSong {
    title: string;
    subTitle: string;
    author: string;
}
/**
 * Audio schema for v4 `Info`.
 */
export interface IInfoAudio {
    songFilename: string;
    songDuration: number;
    audioDataFilename: string;
    bpm: number;
    lufs: number;
    previewStartTime: number;
    previewDuration: number;
}
/**
 * Color scheme schema for v4 `Info`.
 */
export interface IInfoColorScheme {
    colorSchemeName: string;
    overrideNotes: boolean;
    saberAColor: string;
    saberBColor: string;
    obstaclesColor: string;
    overrideLights: boolean;
    environmentColor0: string;
    environmentColor1: string;
    environmentColorW?: string;
    environmentColor0Boost: string;
    environmentColor1Boost: string;
    environmentColorWBoost?: string;
}
/**
 * Beatmap authors schema for v4 `Info Beatmap`.
 */
export interface IInfoBeatmapAuthors {
    mappers: string[];
    lighters: string[];
}
/**
 * Schema for v4 `Info Beatmap`.
 */
export interface IInfoBeatmap extends IItem {
    characteristic: CharacteristicName;
    difficulty: DifficultyName;
    beatmapAuthors: IInfoBeatmapAuthors;
    environmentNameIdx: number;
    beatmapColorSchemeIdx: number;
    noteJumpMovementSpeed: number;
    noteJumpStartBeatOffset: number;
    lightshowDataFilename: string;
    beatmapDataFilename: string;
    customData?: ICustomDataInfoBeatmap;
}
//# sourceMappingURL=info.d.ts.map