/** Clamp value between min and max. */
export declare function clamp(value: number, min: number, max: number): number;
/**
 * Normalize value to 0-1 from given min and max value.
 *
 * Returns 1 if `min == max`, values beyond will be extrapolated.
 */
export declare function normalize(value: number, min: number, max: number): number;
/**
 * Linear interpolate between start to end time given alpha value.
 * ```ts
 * const num = lerp(0.5, 4, 8); // returns 6
 * ```
 *
 * Alpha value must be in range of `0-1`, otherwise extrapolated.
 */
export declare function lerp(alpha: number, from: number, to: number): number;
/**
 * Returns alpha value from interpolated value.
 * ```ts
 * const num = invLerp(6, 4, 8); // returns 0.5
 * ```
 */
export declare function invLerp(value: number, from: number, to: number): number;
/**
 * Remap the value from original range to target range.
 * ```ts
 * const num = remap(6, 4, 8, 40, 60); // returns 50
 * ```
 */
export declare function remap(value: number, origFrom: number, origTo: number, targetFrom: number, targetTo: number): number;
/** Returns true if both value are approximately equal within given tolerance. */
export declare function nearEqual(n1: number, n2: number, tolerance?: number): boolean;
/** Generate 32-bit hash with Java implementation. */
export declare function hashCode(str: string | number | bigint): number;
/** Return number tuple in order. */
export declare function rearrangeTuple(min: number, max: number, inverse?: boolean): [number, number];
/** Round number to given decimal places. */
export declare function round(num: number, d?: number): number;
/** Sum numbers in an array. */
export declare function sumAry(nums: number[]): number;
/** Product numbers in an array. */
export declare function productAry(nums: number[]): number;
/** Mean of numbers in an array. */
export declare function mean(nums: number[]): number;
/** Median of numbers in an array. */
export declare function median(nums: number[]): number;
//# sourceMappingURL=helpers.d.ts.map