/**
 * Seeded pseudorandom generator.
 *
 * Based on Mulberry32 PRNG algorithm.
 *
 * **NOTE:** This is globally scoped, any random call elsewhere will affect the consequent call.
 * Consider creating instance of pseudorandom with `pRandomFn` if you need consistency across usage.
 * Reset the random seed to retain same randomness if needed.
 *
 * **WARNING:** This is not meant to be used for security, rather quick and simple for pseudorandom purpose.
 */
export declare function pRandom(max: number, int?: boolean): number;
export declare function pRandom(int?: boolean): number;
export declare function pRandom(min: number, max: number): number;
export declare function pRandom(min: number, max: number, rounding: boolean): number;
export declare function pRandom(min: number, max: number, rounding: number): number;
/**
 * Create instance of pseudorandom function.
 * ```ts
 * const pRandom = utils.pRandomFn('seed');
 * console.log(pRandom());
 * ```
 *
 * **NOTE:** Seed cannot be reset.
 */
export declare function pRandomFn(seed?: string | number | bigint): (min?: number | boolean, max?: number | boolean, rounding?: number | boolean) => number;
/**
 * Set seed for pseudorandom generator.
 *
 * Recalling this resets the seed.
 *
 * If this is never called, defaults to randomly generated seed.
 */
export declare function pRandomSeed(seed: string | number | bigint): void;
/** Random number generator helpers using built-in JS Math. */
export declare function random(max: number, int?: boolean): number;
export declare function random(int?: boolean): number;
export declare function random(min: number, max: number): number;
export declare function random(min: number, max: number, rounding: boolean): number;
export declare function random(min: number, max: number, rounding: number): number;
/**
 * JS implementation of gaussian random using Box-Muller transformation and skew-normal.
 *
 * This returns value closer to `[-6, 6]` range but can be completely outside of the range due to precision.
 *
 * @link https://spin.atomicobject.com/skew-normal-prng-javascript/
 */
export declare function randomNormal(mean?: number, stdev?: number, skew?: number, rand?: () => number): number;
//# sourceMappingURL=random.d.ts.map