/**
 * Return array of numbers given arguments.
 * ```ts
 * range(5); // [0, 1, 2, 3, 4]
 * range(5, true); // [0, 1, 2, 3, 4, 5]
 * range(5, 10); // [5, 6, 7, 8, 9]
 * range(5, 10, true); // [5, 6, 7, 8, 9, 10]
 * range(5, 10, 2); // [5, 7, 9]
 * ```
 */
export declare function range(n: number, inclusive?: boolean): number[];
export declare function range(start: number, end: number, inclusive?: boolean): number[];
export declare function range(start: number, end: number, step: number, inclusive?: boolean): number[];
/**
 * Shuffle array in-place.
 *
 * Using Fisher–Yates shuffle algorithm.
 */
export declare function shuffle<T>(array: T[], fn?: () => number): T[];
/**
 * Interleave two arrays.
 *
 * ```ts
 * interleave([1, 2, 3], [4, 5, 6]); // [1, 4, 2, 5, 3, 6]
 * ```
 */
export declare function interleave<T, U>([x, ...xs]: T[], ys?: U[]): (T | U)[];
/** Pick random element from array */
export declare function pickRandom<T>(ary: T[], fn?: () => number): T;
/** Move to the next item in an iterable object programatically. */
export declare function cycle<T extends string | number | symbol>(iter: Iterable<T>, current: T, step?: number): T;
//# sourceMappingURL=iterator.d.ts.map