export function shallowCopy(object) {
    if (object === null || object === undefined || typeof object !== 'object') {
        return object;
    }
    if (Array.isArray(object))
        return [...object];
    return { ...object };
}
export function deepCopy(object) {
    if (object === null || object === undefined || typeof object !== 'object') {
        return object;
    }
    // deno-lint-ignore no-explicit-any
    const newObj = Array.isArray(object) ? new Array(object.length) : {};
    if (Array.isArray(object)) {
        for (let i = 0; i < object.length; i++) {
            newObj[i] = deepCopy(object[i]);
        }
        return newObj;
    }
    for (const k in object) {
        newObj[k] = deepCopy(object[k]);
    }
    return newObj;
}
export function jsonCopy(object) {
    return JSON.parse(JSON.stringify(object));
}
/** Check if an object has key/value pairs */
export function isRecord(data) {
    return !!data && typeof data === 'object';
}
/** Check if object is empty. */
export function isEmpty(obj) {
    for (const key in obj) {
        if (Object.hasOwn(obj, key)) {
            return false;
        }
    }
    return true;
}
