"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.chromaLightGradientToVanillaGradient = chromaLightGradientToVanillaGradient;
const logger_js_1 = require("../../logger.js");
const helpers_js_1 = require("../../utils/colors/helpers.js");
const easings_js_1 = require("../../utils/math/easings.js");
const helpers_js_2 = require("../../utils/math/helpers.js");
const basicEvent_js_1 = require("../helpers/core/basicEvent.js");
function tag(name) {
    return ['convert', name];
}
function isLightGradient(obj) {
    return (typeof obj === 'object' &&
        obj != null &&
        '_startColor' in obj &&
        '_endColor' in obj);
}
/**
 * Convert Chroma light gradient to transition event.
 * ```ts
 * const newData = convert.V2ogChromaToChroma(oldData);
 * ```
 */
function chromaLightGradientToVanillaGradient(data) {
    logger_js_1.logger.tWarn(tag('chromaLightGradientToVanillaGradient'), 'Converting chroma light gradient is not fully tested and may break certain lightshow effect!');
    const events = data.lightshow.basicEvents;
    const newEvents = [];
    for (let curr = 0, len = events.length; curr < len; curr++) {
        const ev = events[curr];
        if (!(0, basicEvent_js_1.isLightEventType)(ev.type)) {
            newEvents.push({ ...ev, floatValue: 1 });
            continue;
        }
        if (isLightGradient(ev.customData._lightGradient)) {
            const eventInGradient = [];
            for (let next = curr + 1; next < len; next++) {
                if (ev.type !== events[next].type) {
                    continue;
                }
                if (ev.time + ev.customData._lightGradient._duration >=
                    events[next].time) {
                    eventInGradient.push(events[next]);
                }
            }
            if (eventInGradient.length) {
                ev.customData._color = ev.customData._lightGradient._startColor;
                ev.customData._easing = ev.customData._lightGradient._easing;
                ev.value = ev.value >= 1 && ev.value <= 4 ? 1 : ev.value >= 5 && ev.value <= 8 ? 5 : 9;
                const easing = easings_js_1.EasingsFn[ev.customData._lightGradient._easing ??
                    'easeLinear'];
                let hasOff = false;
                let previousEvent = ev;
                for (const eig of eventInGradient) {
                    if (!hasOff &&
                        eig.time >
                            ev.time + ev.customData._lightGradient._duration - 0.001) {
                        newEvents.push({
                            time: ev.time +
                                ev.customData._lightGradient._duration -
                                0.001,
                            type: ev.type,
                            value: ev.value >= 1 && ev.value <= 4
                                ? 4
                                : ev.value >= 5 && ev.value <= 8
                                    ? 8
                                    : 12,
                            floatValue: 1,
                            customData: {
                                _color: ev.customData._lightGradient._endColor,
                            },
                        });
                    }
                    else {
                        eig.value = hasOff
                            ? eig.value >= 1 && eig.value <= 4
                                ? 1
                                : eig.value >= 5 && eig.value <= 8
                                    ? 5
                                    : 9
                            : eig.value >= 1 && eig.value <= 4
                                ? 4
                                : eig.value >= 5 && eig.value <= 8
                                    ? 8
                                    : 12;
                        eig.customData._color = (0, helpers_js_1.lerpColor)(ev.customData._lightGradient._startColor, ev.customData._lightGradient._endColor, easing((0, helpers_js_2.normalize)(eig.time, ev.time, ev.time + ev.customData._lightGradient._duration)), 'rgba');
                        if (eig.value === 0) {
                            if (eig.customData['_color']) {
                                delete eig.customData['_color'];
                            }
                            if (!hasOff) {
                                newEvents.push({
                                    time: eig.time - 0.001,
                                    type: ev.type,
                                    value: previousEvent.value >= 1 &&
                                        previousEvent.value <= 4
                                        ? 4
                                        : previousEvent.value >= 5 &&
                                            previousEvent.value <= 8
                                            ? 8
                                            : 12,
                                    floatValue: 1,
                                    customData: {
                                        _color: (0, helpers_js_1.lerpColor)(ev.customData._lightGradient._startColor, ev.customData._lightGradient._endColor, easing((0, helpers_js_2.normalize)(eig.time - 0.001, ev.time, ev.time +
                                            ev.customData._lightGradient._duration)), 'rgba'),
                                    },
                                });
                            }
                            hasOff = true;
                        }
                        else {
                            hasOff = false;
                        }
                    }
                    previousEvent = eig;
                }
                if (ev.customData['_lightGradient']) {
                    delete ev.customData['_lightGradient'];
                }
            }
            else {
                ev.customData._color = ev.customData._lightGradient._startColor;
                ev.customData._easing = ev.customData._lightGradient._easing;
                ev.value = ev.value >= 1 && ev.value <= 4 ? 1 : ev.value >= 5 && ev.value <= 8 ? 5 : 9;
                newEvents.push({
                    time: ev.time + ev.customData._lightGradient._duration,
                    type: ev.type,
                    value: ev.value >= 1 && ev.value <= 4 ? 4 : ev.value >= 5 && ev.value <= 8 ? 8 : 12,
                    floatValue: 1,
                    customData: {
                        _color: ev.customData._lightGradient._endColor,
                    },
                });
                if (ev.customData['_lightGradient']) {
                    delete ev.customData['_lightGradient'];
                }
            }
        }
        newEvents.push(ev);
    }
    data.lightshow.basicEvents = newEvents;
    return data;
}
