"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toV1Beatmap = toV1Beatmap;
const logger_js_1 = require("../../../logger.js");
const sort_js_1 = require("../../helpers/sort.js");
function tag(name) {
    return ['convert', 'toV1Beatmap', name];
}
/**
 * Feeling nostalgic?
 * ```ts
 * const converted = toV1Beatmap(data);
 * ```
 *
 * **WARNING:** Guess you should know this legacy version does not have modern features.
 */
function toV1Beatmap(data, fromVersion = data.version) {
    logger_js_1.logger.tWarn(tag('main'), 'Converting beatmap to v1 may lose certain data!');
    switch (fromVersion) {
        case 1:
        case 2:
            data.version = 1;
            break;
        case 3:
            data.version = 1;
            fromV3(data);
            break;
        case 4:
            data.version = 1;
            fromV4(data);
            break;
        default:
            logger_js_1.logger.tWarn(tag('main'), 'Unknown version: version not supported; misinput? Returning original data.');
    }
    return data;
}
function fromV3(bm) {
    bm.difficulty.customData._time = bm.difficulty.customData.time;
    bm.difficulty.customData._BPMChanges = bm.difficulty.customData.BPMChanges?.map((bpmc) => {
        return {
            _time: bpmc.b,
            _BPM: bpmc.m,
            _beatsPerBar: bpmc.p,
            _metronomeOffset: bpmc.o,
        };
    });
    bm.difficulty.customData._bookmarks = bm.difficulty.customData.bookmarks?.map((b) => {
        return { _time: b.b, _name: b.n };
    });
    delete bm.difficulty.customData.time;
    delete bm.difficulty.customData.BPMChanges;
    delete bm.difficulty.customData.bookmarks;
}
function fromV4(bm) {
    bm.difficulty.customData._time = bm.difficulty.customData.time ?? 0;
    let impossibleRotationEvt = false;
    const mapTime = {};
    const objects = [
        bm.difficulty.arcs,
        bm.difficulty.bombNotes,
        bm.difficulty.chains,
        bm.difficulty.colorNotes,
        bm.difficulty.obstacles,
        bm.lightshow.waypoints,
    ]
        .flat()
        .sort(sort_js_1.sortObjectFn);
    for (let i = 0; i < objects.length; i++) {
        const obj = objects[i];
        if (!(obj.time in mapTime)) {
            mapTime[obj.time] = obj.laneRotation;
        }
        else if (mapTime[obj.time] !== obj.laneRotation) {
            impossibleRotationEvt = true;
            break;
        }
    }
    if (impossibleRotationEvt) {
        logger_js_1.logger.warn(tag('fromV4'), 'Impossible rotation event cannot be represented in v1!');
    }
    else {
        bm.difficulty.rotationEvents = [];
        let currentRotation = 0;
        for (const time in mapTime) {
            const t = +time;
            const r = mapTime[time];
            const difference = r - currentRotation;
            if (difference === 0)
                continue;
            currentRotation = r;
            bm.difficulty.rotationEvents.push({
                time: t,
                rotation: difference,
                executionTime: 0,
                customData: {},
            });
        }
    }
    for (let i = 0; i < objects.length; i++) {
        const obj = objects[i];
        obj.laneRotation = 0;
        if ('tailLaneRotation' in obj)
            obj.tailLaneRotation = 0;
    }
}
