"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toV4Info = toV4Info;
const logger_js_1 = require("../../../logger.js");
function tag(name) {
    return ['convert', 'toV4Info', name];
}
/**
 * Convert to beatmap v4.
 * ```ts
 * const converted = toV4Info(data);
 * ```
 *
 * **WARNING:** Custom data may be lost on conversion, as well as other incompatible attributes.
 */
function toV4Info(data, fromVersion = data.version) {
    logger_js_1.logger.tWarn(tag('main'), 'Converting to beatmap v4 may lose certain data!');
    switch (fromVersion) {
        case 1:
            data.environmentNames = [
                data.environmentBase.normal || 'DefaultEnvironment',
            ];
            data.difficulties.forEach((d) => {
                d.environmentId = 0;
            }); /** Falls through */
        case 2:
        case 3:
        case 4:
            data.version = 4;
            break;
        default:
            logger_js_1.logger.tWarn(tag('main'), 'Unknown version: version not supported; misinput? Returning original data.');
    }
    return data;
}
