import type { IWrapBaseItem } from '../../../types/beatmap/wrapper/baseItem.js';
import { Cloneable } from '../../shared/cloneable.js';
/**
 * Basic building block of beatmap item.
 *
 * Items contains basic information and utilities.
 *
 * @abstract
 */
export declare abstract class BaseItem extends Cloneable implements IWrapBaseItem {
    customData: IWrapBaseItem['customData'];
    /**
     * Convert object into class.
     *
     * **WARNING:** If you ever need to use this,
     * you may have used function that used wrapper interface and add onto array/object when are supposed to be concrete.
     */
    reconcile(): this;
    setCustomData(value: this['customData']): this;
    resetCustomData(): this;
    removeCustomData(key: string | string[]): this;
    addCustomData(object: this['customData']): this;
    /**
     * Sort beatmap object(s) accordingly.
     *
     * Certain objects may not contain sortable array.
     *
     * Custom function can be provided and will run after base implemention.
     */
    sort(fn?: (object: this) => void): this;
    /** Allow for advanced custom function. */
    perform(fn: (object: this) => void): this;
    /**
     * Check if object is valid in vanilla game.
     * This may also allow for value outside of intended value as long as it does not cause
     * weird, inconsistent behaviour or unable to load the map.
     *
     * However, this validity may not be applicable to older version of the game or schema.
     *
     * Override allow for custom function to take over rather than run alongside.
     *
     * @example
     * ```ts
     * if (!obj.isValid(optionalFn)) {}
     * ```
     */
    isValid(fn?: (object: this) => boolean, _override?: boolean): boolean;
    /**
     * Check an object given a custom function.
     *
     * @example
     * ```ts
     * if (obj.check(hasChromaFn)) {}
     * ```
     */
    check(fn?: (object: this) => boolean): boolean;
}
//# sourceMappingURL=baseItem.d.ts.map