import type { GetAngleFn, MirrorFn } from '../../../types/beatmap/shared/functions.js';
import type { IWrapBaseNote } from '../../../types/beatmap/wrapper/baseNote.js';
import { GridObject } from './gridObject.js';
/**
 * Base note beatmap object.
 *
 * @abstract
 */
export declare abstract class BaseNote extends GridObject implements IWrapBaseNote {
    color: IWrapBaseNote['color'];
    direction: IWrapBaseNote['direction'];
    setColor(value: this['color']): this;
    setDirection(value: this['direction']): this;
    mirror(flipColor?: boolean, fn?: MirrorFn<this>): this;
    /**
     * Check if note is red note.
     * ```ts
     * if (note.isBlue()) {}
     * ```
     */
    isRed(): boolean;
    /**
     * Check if note is blue note.
     * ```ts
     * if (note.isBlue()) {}
     * ```
     */
    isBlue(): boolean;
    /**
     * Get standardised note angle.
     *
     * @example
     * ```ts
     * import type { IBaseNote } from './baseNote.ts';
     * let note!: IBaseNote;
     * const optionalFn = (object: IBaseNote) => object.customData.value;
     * const tailAngle = note.getTailAngle(optionalFn);
     * ```
     *
     * Custom function are used to return any arbitrary data first if value exist, otherwise returns base value.
     */
    getAngle(fn?: GetAngleFn<this>): number;
    /**
     * Compare current note with the note ahead of it and return if the notes is a double.
     * ```ts
     * if (note.isDouble(otherNote, tol, optionalFn)) {}
     * ```
     */
    isDouble(compareTo: IWrapBaseNote, tolerance?: number): boolean;
    isValid(fn?: (object: this) => boolean, override?: boolean): boolean;
}
//# sourceMappingURL=baseNote.d.ts.map