"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseSlider = void 0;
const vector_js_1 = require("../../../utils/math/vector.js");
const baseSlider_js_1 = require("../../helpers/core/baseSlider.js");
const gridObject_js_1 = require("../../helpers/core/gridObject.js");
const constants_js_1 = require("../../shared/constants.js");
const baseNote_js_1 = require("./baseNote.js");
/**
 * Base slider beatmap object.
 *
 * @abstract
 */
class BaseSlider extends baseNote_js_1.BaseNote {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "tailTime", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailPosX", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailPosY", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "tailLaneRotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
    }
    setTailTime(value) {
        this.tailTime = value;
        return this;
    }
    setTailPosX(value) {
        this.tailPosX = value;
        return this;
    }
    setTailPosY(value) {
        this.tailPosY = value;
        return this;
    }
    setTailLaneRotation(value) {
        this.tailLaneRotation = value;
        return this;
    }
    mirror(flipColor = true, fn) {
        fn?.(this);
        this.tailPosX = (0, gridObject_js_1.mirrorCoordinate)(this.tailPosX, constants_js_1.LINE_COUNT);
        return super.mirror(flipColor);
    }
    getTailPosition(fn) {
        return fn?.(this) ??
            (0, vector_js_1.vectorAdd)((0, gridObject_js_1.resolveGridPosition)({ posX: this.tailPosX, posY: this.tailPosY }), [-2]);
    }
    isInverse() {
        return (0, baseSlider_js_1.isInverseSlider)(this);
    }
}
exports.BaseSlider = BaseSlider;
