"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GridObject = void 0;
const vector_js_1 = require("../../../utils/math/vector.js");
const gridObject_js_1 = require("../../helpers/core/gridObject.js");
const constants_js_1 = require("../../shared/constants.js");
const baseObject_js_1 = require("./baseObject.js");
/**
 * Base grid beatmap object.
 *
 * This appear within beatmap lane.
 *
 * @abstract
 */
class GridObject extends baseObject_js_1.BaseObject {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "posX", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "posY", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "laneRotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
    }
    setPosX(value) {
        this.posX = value;
        return this;
    }
    setPosY(value) {
        this.posY = value;
        return this;
    }
    setLaneRotation(value) {
        this.laneRotation = value;
        return this;
    }
    /**
     * Mirror a grid object, apply alternative flip and Noodle Extensions if available.
     * ```ts
     * obj.mirror(false, optionalFn);
     * ```
     *
     * Alternative flip is true by default. This implementation is typically used to flip color of note.
     */
    mirror(_flipAlt, fn) {
        fn?.(this);
        this.posX = (0, gridObject_js_1.mirrorCoordinate)(this.posX, constants_js_1.LINE_COUNT);
        return this;
    }
    /**
     * Get object position and return the Beatwalls' position x and y value in tuple.
     * ```ts
     * const objPos = obj.getPosition(optionalFn);
     * ```
     */
    getPosition(fn) {
        return fn?.(this) ?? (0, vector_js_1.vectorAdd)((0, gridObject_js_1.resolveGridPosition)(this), [-2]);
    }
    getDistance(compareTo, fn) {
        return (0, gridObject_js_1.resolveGridDistance)(this, compareTo, (o) => o.getPosition(fn));
    }
    isVertical(compareTo, fn) {
        return (0, gridObject_js_1.isVertical)(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isHorizontal(compareTo, fn) {
        return (0, gridObject_js_1.isHorizontal)(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isDiagonal(compareTo, fn) {
        return (0, gridObject_js_1.isDiagonal)(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isInline(compareTo, lapping, fn) {
        return (0, gridObject_js_1.isInline)(this, compareTo, lapping ?? 0.5, (o) => o.getPosition(fn));
    }
    isAdjacent(compareTo, fn) {
        return (0, gridObject_js_1.isAdjacent)(this, compareTo, 0.001, (o) => o.getPosition(fn));
    }
    isWindow(compareTo, distance, fn) {
        return (0, gridObject_js_1.isWindow)(this, compareTo, distance ?? 1.8, (o) => o.getPosition(fn));
    }
    isSlantedWindow(compareTo, fn) {
        return (0, gridObject_js_1.isSlantedWindow)(this, compareTo, (o) => o.getPosition(fn));
    }
}
exports.GridObject = GridObject;
