import type { EnvironmentAllName } from '../../types/beatmap/shared/environment.js';
import type { IWrapBasicEvent } from '../../types/beatmap/wrapper/basicEvent.js';
import type { DeepPartial } from '../../types/utils.js';
import { BaseObject } from './abstract/baseObject.js';
export declare function createBasicEvent(data?: DeepPartial<IWrapBasicEvent>): IWrapBasicEvent;
/**
 * Core beatmap basic event.
 */
export declare class BasicEvent extends BaseObject implements IWrapBasicEvent {
    static defaultValue: IWrapBasicEvent;
    static createOne(data?: Partial<IWrapBasicEvent>): BasicEvent;
    static create(...data: Partial<IWrapBasicEvent>[]): BasicEvent[];
    constructor(data?: Partial<IWrapBasicEvent>);
    /**
     * Check if event is a valid type.
     * ```ts
     * if (event.isValidType()) {}
     * ```
     */
    isValidType(): boolean;
    isValid(fn?: (object: this) => boolean, override?: boolean): boolean;
    type: IWrapBasicEvent['type'];
    value: IWrapBasicEvent['value'];
    floatValue: IWrapBasicEvent['floatValue'];
    setType(value: this['type']): this;
    setValue(value: this['value']): this;
    setFloatValue(value: this['floatValue']): this;
    /**
     * Check if light event is an off event.
     * ```ts
     * if (event.isOff()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOff(): boolean;
    /**
     * Check if light event is an on event.
     * ```ts
     * if (event.isOn()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isOn(): boolean;
    /**
     * Check if light event is a flash event.
     * ```ts
     * if (event.isFlash()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFlash(): boolean;
    /**
     * Check if light event is a fade event.
     * ```ts
     * if (event.isFade()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isFade(): boolean;
    /**
     * Check if light event is a transition event.
     * ```ts
     * if (event.isTransition()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isTransition(): boolean;
    /**
     * Check if light event is a blue light.
     * ```ts
     * if (event.isBlue()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isBlue(): boolean;
    /**
     * Check if light event is a red light.
     * ```ts
     * if (event.isRed()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isRed(): boolean;
    /**
     * Check if light event is a white light.
     * ```ts
     * if (event.isWhite()) {}
     * ```
     *
     * This may check non-light event too.
     */
    isWhite(): boolean;
    /**
     * Check if event is a light event.
     * ```ts
     * if (event.isLightEvent()) {}
     * ```
     */
    isLightEvent(environment?: EnvironmentAllName): boolean;
    /**
     * Check if event is a boost event.
     * ```ts
     * if (event.isColorBoost()) {}
     * ```
     */
    isColorBoost(): boolean;
    /**
     * Check if event is a ring event.
     * ```ts
     * if (event.isRingEvent()) {}
     * ```
     *
     * This does not check for ring zoom.
     */
    isRingEvent(environment?: EnvironmentAllName): boolean;
    /**
     * Check if event is a laser rotation event.
     * ```ts
     * if (event.isLaserRotationEvent()) {}
     * ```
     */
    isLaserRotationEvent(environment?: EnvironmentAllName): boolean;
    /**
     * Check if event is a lane rotation event.
     * ```ts
     * if (event.isLaneRotationEvent()) {}
     * ```
     */
    isLaneRotationEvent(): boolean;
    /**
     * Check if event is a extra event.
     * ```ts
     * if (event.isExtraEvent()) {}
     * ```
     */
    isExtraEvent(environment?: EnvironmentAllName): boolean;
    /**
     * Check if event is a special event.
     * ```ts
     * if (event.isSpecialEvent()) {}
     * ```
     */
    isSpecialEvent(environment?: EnvironmentAllName): boolean;
    /**
     * Check if event is a BPM change event.
     * ```ts
     * if (event.isBpmEvent()) {}
     * ```
     */
    isBpmEvent(): boolean;
    isNjsEvent(): boolean;
    /**
     * Not to be confused with `isLightEvent`, this checks for event that affects the environment/lighting.
     * ```ts
     * if (event.isLightingEvent()) {}
     * ```
     */
    isLightingEvent(): boolean;
    /**
     * Check if event has old Chroma properties.
     * ```ts
     * if (event.isOldChroma()) {}
     * ```
     */
    isOldChroma(): boolean;
}
//# sourceMappingURL=basicEvent.d.ts.map