"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chain = void 0;
exports.createChain = createChain;
const json_js_1 = require("../../utils/misc/json.js");
const baseSlider_js_1 = require("./abstract/baseSlider.js");
function createChain(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        color: data.color ?? 0,
        direction: data.direction ?? 0,
        laneRotation: data.laneRotation ?? 0,
        tailTime: data.tailTime ?? 0,
        tailPosX: data.tailPosX ?? 0,
        tailPosY: data.tailPosY ?? 0,
        tailLaneRotation: data.tailLaneRotation ?? 0,
        sliceCount: data.sliceCount ?? 0,
        squish: data.squish ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap chain.
 */
class Chain extends baseSlider_js_1.BaseSlider {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "sliceCount", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "squish", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? Chain.defaultValue.time;
        this.posX = data.posX ?? Chain.defaultValue.posX;
        this.posY = data.posY ?? Chain.defaultValue.posY;
        this.color = data.color ?? Chain.defaultValue.color;
        this.direction = data.direction ?? Chain.defaultValue.direction;
        this.laneRotation = data.laneRotation ?? Chain.defaultValue.laneRotation;
        this.tailTime = data.tailTime ?? Chain.defaultValue.tailTime;
        this.tailPosX = data.tailPosX ?? Chain.defaultValue.tailPosX;
        this.tailPosY = data.tailPosY ?? Chain.defaultValue.tailPosY;
        this.tailLaneRotation = data.tailLaneRotation ?? Chain.defaultValue.tailLaneRotation;
        this.sliceCount = data.sliceCount ?? Chain.defaultValue.sliceCount;
        this.squish = data.squish ?? Chain.defaultValue.squish;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? Chain.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (!this.isInverse() ||
                this.posY >= 0 ||
                this.posY <= 2 ||
                this.tailPosY >= 0 ||
                this.tailPosY <= 2);
    }
    setSliceCount(value) {
        this.sliceCount = value;
        return this;
    }
    setSquish(value) {
        this.squish = value;
        return this;
    }
}
exports.Chain = Chain;
Object.defineProperty(Chain, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createChain()
});
