"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorNote = void 0;
exports.createColorNote = createColorNote;
const json_js_1 = require("../../utils/misc/json.js");
const baseNote_js_1 = require("../helpers/core/baseNote.js");
const baseNote_js_2 = require("./abstract/baseNote.js");
function createColorNote(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        color: data.color ?? 0,
        direction: data.direction ?? 0,
        angleOffset: data.angleOffset ?? 0,
        laneRotation: data.laneRotation ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap color note.
 */
class ColorNote extends baseNote_js_2.BaseNote {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "angleOffset", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "customData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? ColorNote.defaultValue.time;
        this.posX = data.posX ?? ColorNote.defaultValue.posX;
        this.posY = data.posY ?? ColorNote.defaultValue.posY;
        this.color = data.color ?? ColorNote.defaultValue.color;
        this.direction = data.direction ?? ColorNote.defaultValue.direction;
        this.angleOffset = data.angleOffset ?? ColorNote.defaultValue.angleOffset;
        this.laneRotation = data.laneRotation ?? ColorNote.defaultValue.laneRotation;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? ColorNote.defaultValue.customData);
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.posX >= 0 &&
            this.posX <= 3 &&
            this.posY >= 0 &&
            this.posY <= 2 &&
            this.direction >= 0 &&
            this.direction <= 8;
    }
    setAngleOffset(value) {
        this.angleOffset = value;
        return this;
    }
    mirror(flipColor = true, fn) {
        fn?.(this);
        return super.mirror(flipColor);
    }
    getAngle(fn) {
        return fn?.(this) ?? (0, baseNote_js_1.resolveNoteAngle)(this.direction) + this.angleOffset;
    }
}
exports.ColorNote = ColorNote;
Object.defineProperty(ColorNote, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createColorNote()
});
