"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Difficulty = void 0;
exports.createDifficulty = createDifficulty;
const json_js_1 = require("../../utils/misc/json.js");
const misc_js_1 = require("../helpers/core/misc.js");
const sort_js_1 = require("../helpers/sort.js");
const baseItem_js_1 = require("./abstract/baseItem.js");
const arc_js_1 = require("./arc.js");
const bombNote_js_1 = require("./bombNote.js");
const bpmEvent_js_1 = require("./bpmEvent.js");
const chain_js_1 = require("./chain.js");
const colorNote_js_1 = require("./colorNote.js");
const njsEvent_js_1 = require("./njsEvent.js");
const obstacle_js_1 = require("./obstacle.js");
const rotationEvent_js_1 = require("./rotationEvent.js");
function createDifficulty(data = {}) {
    return {
        bpmEvents: data.bpmEvents?.map(bpmEvent_js_1.createBPMEvent) ?? [],
        rotationEvents: data.rotationEvents?.map(rotationEvent_js_1.createRotationEvent) ?? [],
        colorNotes: data.colorNotes?.map(colorNote_js_1.createColorNote) ?? [],
        bombNotes: data.bombNotes?.map(bombNote_js_1.createBombNote) ?? [],
        obstacles: data.obstacles?.map(obstacle_js_1.createObstacle) ?? [],
        arcs: data.arcs?.map(arc_js_1.createArc) ?? [],
        chains: data.chains?.map(chain_js_1.createChain) ?? [],
        njsEvents: data.njsEvents?.map(njsEvent_js_1.createNJSEvent) ?? [],
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap difficulty.
 */
class Difficulty extends baseItem_js_1.BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "bpmEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "rotationEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "colorNotes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "bombNotes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "obstacles", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "arcs", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "chains", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "njsEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "customData", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.bpmEvents = (data.bpmEvents ?? Difficulty.defaultValue.bpmEvents).map((e) => new bpmEvent_js_1.BPMEvent(e));
        this.rotationEvents = (data.rotationEvents ?? Difficulty.defaultValue.rotationEvents).map((e) => new rotationEvent_js_1.RotationEvent(e));
        this.colorNotes = (data.colorNotes ?? Difficulty.defaultValue.colorNotes).map((e) => new colorNote_js_1.ColorNote(e));
        this.bombNotes = (data.bombNotes ?? Difficulty.defaultValue.bombNotes).map((e) => new bombNote_js_1.BombNote(e));
        this.obstacles = (data.obstacles ?? Difficulty.defaultValue.obstacles).map((e) => new obstacle_js_1.Obstacle(e));
        this.arcs = (data.arcs ?? Difficulty.defaultValue.arcs).map((e) => new arc_js_1.Arc(e));
        this.chains = (data.chains ?? Difficulty.defaultValue.chains).map((e) => new chain_js_1.Chain(e));
        this.njsEvents = (data.njsEvents ?? Difficulty.defaultValue.njsEvents).map((e) => new njsEvent_js_1.NJSEvent(e));
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? Difficulty.defaultValue.customData);
    }
    reconcile() {
        this.bpmEvents = (0, misc_js_1.reconcileClassObject)(this.bpmEvents, bpmEvent_js_1.BPMEvent);
        this.rotationEvents = (0, misc_js_1.reconcileClassObject)(this.rotationEvents, rotationEvent_js_1.RotationEvent);
        this.colorNotes = (0, misc_js_1.reconcileClassObject)(this.colorNotes, colorNote_js_1.ColorNote);
        this.bombNotes = (0, misc_js_1.reconcileClassObject)(this.bombNotes, bombNote_js_1.BombNote);
        this.obstacles = (0, misc_js_1.reconcileClassObject)(this.obstacles, obstacle_js_1.Obstacle);
        this.arcs = (0, misc_js_1.reconcileClassObject)(this.arcs, arc_js_1.Arc);
        this.chains = (0, misc_js_1.reconcileClassObject)(this.chains, chain_js_1.Chain);
        this.njsEvents = (0, misc_js_1.reconcileClassObject)(this.njsEvents, njsEvent_js_1.NJSEvent);
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.bpmEvents.every((e) => e.isValid()) &&
            this.rotationEvents.every((e) => e.isValid()) &&
            this.colorNotes.every((e) => e.isValid()) &&
            this.bombNotes.every((e) => e.isValid()) &&
            this.obstacles.every((e) => e.isValid()) &&
            this.arcs.every((e) => e.isValid()) &&
            this.chains.every((e) => e.isValid());
    }
    sort() {
        this.bpmEvents.sort(sort_js_1.sortObjectFn);
        this.rotationEvents.sort(sort_js_1.sortObjectFn);
        this.colorNotes.sort(sort_js_1.sortNoteFn);
        this.bombNotes.sort(sort_js_1.sortNoteFn);
        this.obstacles.sort(sort_js_1.sortObjectFn);
        this.arcs.sort(sort_js_1.sortNoteFn);
        this.chains.sort(sort_js_1.sortNoteFn);
        return this;
    }
    allObjects() {
        return [
            ...this.bpmEvents,
            ...this.rotationEvents,
            ...this.colorNotes,
            ...this.bombNotes,
            ...this.obstacles,
            ...this.arcs,
            ...this.chains,
            ...this.njsEvents,
        ];
    }
    addBpmEvents(...data) {
        for (const d of data) {
            this.bpmEvents.push(new bpmEvent_js_1.BPMEvent(d));
        }
        return this;
    }
    addRotationEvents(...data) {
        for (const d of data) {
            this.rotationEvents.push(new rotationEvent_js_1.RotationEvent(d));
        }
        return this;
    }
    addColorNotes(...data) {
        for (const d of data) {
            this.colorNotes.push(new colorNote_js_1.ColorNote(d));
        }
        return this;
    }
    addBombNotes(...data) {
        for (const d of data) {
            this.bombNotes.push(new bombNote_js_1.BombNote(d));
        }
        return this;
    }
    addObstacles(...data) {
        for (const d of data) {
            this.obstacles.push(new obstacle_js_1.Obstacle(d));
        }
        return this;
    }
    addArcs(...data) {
        for (const d of data) {
            this.arcs.push(new arc_js_1.Arc(d));
        }
        return this;
    }
    addChains(...data) {
        for (const d of data) {
            this.chains.push(new chain_js_1.Chain(d));
        }
        return this;
    }
    addNjsEvents(...data) {
        for (const d of data) {
            this.njsEvents.push(new njsEvent_js_1.NJSEvent(d));
        }
        return this;
    }
}
exports.Difficulty = Difficulty;
Object.defineProperty(Difficulty, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createDifficulty()
});
