"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FxEventBox = void 0;
exports.createFxEventBox = createFxEventBox;
const json_js_1 = require("../../utils/misc/json.js");
const misc_js_1 = require("../helpers/core/misc.js");
const eventBox_js_1 = require("./abstract/eventBox.js");
const fxEventFloat_js_1 = require("./fxEventFloat.js");
const indexFilter_js_1 = require("./indexFilter.js");
function createFxEventBox(data = {}) {
    return {
        filter: (0, indexFilter_js_1.createIndexFilter)(data.filter),
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        fxDistribution: data.fxDistribution ?? 0,
        fxDistributionType: data.fxDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map(fxEventFloat_js_1.createFxEventFloat) ?? [],
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap FX event box.
 */
class FxEventBox extends eventBox_js_1.EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "fxDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fxDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new indexFilter_js_1.IndexFilter(data.filter ?? FxEventBox.defaultValue.filter);
        this.beatDistribution = data.beatDistribution ?? FxEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            FxEventBox.defaultValue.beatDistributionType;
        this.fxDistribution = data.fxDistribution ?? FxEventBox.defaultValue.fxDistribution;
        this.fxDistributionType = data.fxDistributionType ??
            FxEventBox.defaultValue.fxDistributionType;
        this.affectFirst = data.affectFirst ?? FxEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? FxEventBox.defaultValue.easing;
        this.events = (data.events ?? FxEventBox.defaultValue.events).map((obj) => new fxEventFloat_js_1.FxEventFloat(obj));
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? FxEventBox.defaultValue.customData);
    }
    reconcile() {
        this.filter = (0, misc_js_1.reconcileClassObject)(this.filter, indexFilter_js_1.IndexFilter);
        this.events = (0, misc_js_1.reconcileClassObject)(this.events, fxEventFloat_js_1.FxEventFloat);
        return this;
    }
    setFxDistribution(value) {
        this.fxDistribution = value;
        return this;
    }
    setFxDistributionType(value) {
        this.fxDistributionType = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.fxDistributionType === 1 || this.fxDistributionType === 2) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
exports.FxEventBox = FxEventBox;
Object.defineProperty(FxEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createFxEventBox()
});
