"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightColorEvent = void 0;
exports.createLightColorEvent = createLightColorEvent;
const json_js_1 = require("../../utils/misc/json.js");
const baseObject_js_1 = require("./abstract/baseObject.js");
function createLightColorEvent(data = {}) {
    return {
        time: data.time ?? 0,
        previous: data.previous ?? 0,
        color: data.color ?? 0,
        frequency: data.frequency ?? 0,
        brightness: data.brightness ?? 0,
        strobeBrightness: data.strobeBrightness ?? 0,
        strobeFade: data.strobeFade ?? 0,
        easing: data.easing ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap light color event.
 */
class LightColorEvent extends baseObject_js_1.BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "color", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "brightness", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "frequency", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "strobeBrightness", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "strobeFade", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "easing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.time = data.time ?? LightColorEvent.defaultValue.time;
        this.previous = data.previous ?? LightColorEvent.defaultValue.previous;
        this.color = data.color ?? LightColorEvent.defaultValue.color;
        this.frequency = data.frequency ?? LightColorEvent.defaultValue.frequency;
        this.brightness = data.brightness ?? LightColorEvent.defaultValue.brightness;
        this.strobeBrightness = data.strobeBrightness ??
            LightColorEvent.defaultValue.strobeBrightness;
        this.strobeFade = data.strobeFade ?? LightColorEvent.defaultValue.strobeFade;
        this.easing = data.easing ?? LightColorEvent.defaultValue.easing;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? LightColorEvent.defaultValue.customData);
    }
    setPrevious(value) {
        this.previous = value;
        return this;
    }
    setColor(value) {
        this.color = value;
        return this;
    }
    setBrightness(value) {
        this.brightness = value;
        return this;
    }
    setFrequency(value) {
        this.frequency = value;
        return this;
    }
    setStrobeBrightness(value) {
        this.strobeBrightness = value;
        return this;
    }
    setStrobeFade(value) {
        this.strobeFade = value;
        return this;
    }
    setEasing(value) {
        this.easing = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.previous === 0 || this.previous === 1) &&
            this.easing >= -1 &&
            this.easing <= 103 &&
            this.color >= -1 &&
            this.color <= 2 &&
            this.brightness >= 0 &&
            this.frequency >= 0;
    }
}
exports.LightColorEvent = LightColorEvent;
Object.defineProperty(LightColorEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightColorEvent()
});
