"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightColorEventBox = void 0;
exports.createLightColorEventBox = createLightColorEventBox;
const json_js_1 = require("../../utils/misc/json.js");
const misc_js_1 = require("../helpers/core/misc.js");
const eventBox_js_1 = require("./abstract/eventBox.js");
const indexFilter_js_1 = require("./indexFilter.js");
const lightColorEvent_js_1 = require("./lightColorEvent.js");
function createLightColorEventBox(data = {}) {
    return {
        filter: (0, indexFilter_js_1.createIndexFilter)(data.filter),
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        brightnessDistribution: data.brightnessDistribution ?? 0,
        brightnessDistributionType: data.brightnessDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map((o) => (0, lightColorEvent_js_1.createLightColorEvent)(o)) ?? [],
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap light color event box.
 */
class LightColorEventBox extends eventBox_js_1.EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "brightnessDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "brightnessDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new indexFilter_js_1.IndexFilter(data.filter ?? LightColorEventBox.defaultValue.filter);
        this.beatDistribution = data.beatDistribution ??
            LightColorEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            LightColorEventBox.defaultValue.beatDistributionType;
        this.brightnessDistribution = data.brightnessDistribution ??
            LightColorEventBox.defaultValue.brightnessDistribution;
        this.brightnessDistributionType = data.brightnessDistributionType ??
            LightColorEventBox.defaultValue.brightnessDistributionType;
        this.affectFirst = data.affectFirst ?? LightColorEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? LightColorEventBox.defaultValue.easing;
        this.events = (data.events ?? LightColorEventBox.defaultValue.events).map((obj) => new lightColorEvent_js_1.LightColorEvent(obj));
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? LightColorEventBox.defaultValue.customData);
    }
    reconcile() {
        this.events = (0, misc_js_1.reconcileClassObject)(this.events, lightColorEvent_js_1.LightColorEvent);
        return this;
    }
    setBrightnessDistribution(value) {
        this.brightnessDistribution = value;
        return this;
    }
    setBrightnessDistributionType(value) {
        this.brightnessDistributionType = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.brightnessDistributionType === 1 || this.brightnessDistributionType === 2) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
exports.LightColorEventBox = LightColorEventBox;
Object.defineProperty(LightColorEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightColorEventBox()
});
