"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightTranslationEvent = void 0;
exports.createLightTranslationEvent = createLightTranslationEvent;
const json_js_1 = require("../../utils/misc/json.js");
const baseObject_js_1 = require("./abstract/baseObject.js");
function createLightTranslationEvent(data = {}) {
    return {
        time: data.time ?? 0,
        easing: data.easing ?? 0,
        previous: data.previous ?? 0,
        translation: data.translation ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap light translation event.
 */
class LightTranslationEvent extends baseObject_js_1.BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "previous", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "easing", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "translation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? LightTranslationEvent.defaultValue.time;
        this.easing = data.easing ?? LightTranslationEvent.defaultValue.easing;
        this.previous = data.previous ?? LightTranslationEvent.defaultValue.previous;
        this.translation = data.translation ?? LightTranslationEvent.defaultValue.translation;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? LightTranslationEvent.defaultValue.customData);
    }
    setPrevious(value) {
        this.previous = value;
        return this;
    }
    setEasing(value) {
        this.easing = value;
        return this;
    }
    setTranslation(value) {
        this.translation = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.previous === 0 || this.previous === 1) &&
            this.easing >= -1 &&
            this.easing <= 103;
    }
}
exports.LightTranslationEvent = LightTranslationEvent;
Object.defineProperty(LightTranslationEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightTranslationEvent()
});
