"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LightTranslationEventBox = void 0;
exports.createLightTranslationEventBox = createLightTranslationEventBox;
const json_js_1 = require("../../utils/misc/json.js");
const misc_js_1 = require("../helpers/core/misc.js");
const eventBox_js_1 = require("./abstract/eventBox.js");
const indexFilter_js_1 = require("./indexFilter.js");
const lightTranslationEvent_js_1 = require("./lightTranslationEvent.js");
function createLightTranslationEventBox(data = {}) {
    return {
        filter: (0, indexFilter_js_1.createIndexFilter)(data.filter),
        axis: data.axis ?? 0,
        flip: data.flip ?? 0,
        beatDistribution: data.beatDistribution ?? 0,
        beatDistributionType: data.beatDistributionType ?? 1,
        gapDistribution: data.gapDistribution ?? 0,
        gapDistributionType: data.gapDistributionType ?? 1,
        affectFirst: data.affectFirst ?? 0,
        easing: data.easing ?? 0,
        events: data.events?.map((o) => (0, lightTranslationEvent_js_1.createLightTranslationEvent)(o)) ?? [],
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap light translation event box.
 */
class LightTranslationEventBox extends eventBox_js_1.EventBox {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "gapDistribution", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "gapDistributionType", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "axis", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "flip", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "events", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.filter = new indexFilter_js_1.IndexFilter(data.filter ?? LightTranslationEventBox.defaultValue.filter);
        this.axis = data.axis ?? LightTranslationEventBox.defaultValue.axis;
        this.flip = data.flip ?? LightTranslationEventBox.defaultValue.flip;
        this.beatDistribution = data.beatDistribution ??
            LightTranslationEventBox.defaultValue.beatDistribution;
        this.beatDistributionType = data.beatDistributionType ??
            LightTranslationEventBox.defaultValue.beatDistributionType;
        this.gapDistribution = data.gapDistribution ??
            LightTranslationEventBox.defaultValue.gapDistribution;
        this.gapDistributionType = data.gapDistributionType ??
            LightTranslationEventBox.defaultValue.gapDistributionType;
        this.affectFirst = data.affectFirst ?? LightTranslationEventBox.defaultValue.affectFirst;
        this.easing = data.easing ?? LightTranslationEventBox.defaultValue.easing;
        this.events = (data.events ?? LightTranslationEventBox.defaultValue.events).map((obj) => new lightTranslationEvent_js_1.LightTranslationEvent(obj));
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? LightTranslationEventBox.defaultValue.customData);
    }
    reconcile() {
        this.events = (0, misc_js_1.reconcileClassObject)(this.events, lightTranslationEvent_js_1.LightTranslationEvent);
        return this;
    }
    setGapDistribution(value) {
        this.gapDistribution = value;
        return this;
    }
    setGapDistributionType(value) {
        this.gapDistributionType = value;
        return this;
    }
    setAxis(value) {
        this.axis = value;
        return this;
    }
    setFlip(value) {
        this.flip = value;
        return this;
    }
    setEvents(value) {
        this.events = value;
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            (this.gapDistributionType === 1 || this.gapDistributionType === 2) &&
            (this.axis === 0 || this.axis === 1 || this.axis === 2) &&
            (this.flip === 0 || this.flip === 1) &&
            (this.affectFirst === 0 || this.affectFirst === 1);
    }
}
exports.LightTranslationEventBox = LightTranslationEventBox;
Object.defineProperty(LightTranslationEventBox, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightTranslationEventBox()
});
