"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Lightshow = void 0;
exports.createLightshow = createLightshow;
const json_js_1 = require("../../utils/misc/json.js");
const misc_js_1 = require("../helpers/core/misc.js");
const sort_js_1 = require("../helpers/sort.js");
const baseItem_js_1 = require("./abstract/baseItem.js");
const basicEvent_js_1 = require("./basicEvent.js");
const basicEventTypesForKeywords_js_1 = require("./basicEventTypesForKeywords.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const fxEventBoxGroup_js_1 = require("./fxEventBoxGroup.js");
const lightColorEventBoxGroup_js_1 = require("./lightColorEventBoxGroup.js");
const lightRotationEventBoxGroup_js_1 = require("./lightRotationEventBoxGroup.js");
const lightTranslationEventBoxGroup_js_1 = require("./lightTranslationEventBoxGroup.js");
const waypoint_js_1 = require("./waypoint.js");
function createLightshow(data = {}) {
    return {
        waypoints: data.waypoints?.map(waypoint_js_1.createWaypoint) ?? [],
        basicEvents: data.basicEvents?.map(basicEvent_js_1.createBasicEvent) ?? [],
        colorBoostEvents: data.colorBoostEvents?.map(colorBoostEvent_js_1.createColorBoostEvent) ?? [],
        lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map(lightColorEventBoxGroup_js_1.createLightColorEventBoxGroup) ??
            [],
        lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map(lightRotationEventBoxGroup_js_1.createLightRotationEventBoxGroup) ?? [],
        lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map(lightTranslationEventBoxGroup_js_1.createLightTranslationEventBoxGroup) ?? [],
        fxEventBoxGroups: data.fxEventBoxGroups?.map(fxEventBoxGroup_js_1.createFxEventBoxGroup) ?? [],
        basicEventTypesWithKeywords: {
            list: data.basicEventTypesWithKeywords?.list?.map(basicEventTypesForKeywords_js_1.createBasicEventTypesForKeywords) ?? [],
        },
        useNormalEventsAsCompatibleEvents: !!data.useNormalEventsAsCompatibleEvents,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap lightshow.
 */
class Lightshow extends baseItem_js_1.BaseItem {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "waypoints", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "basicEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "colorBoostEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightColorEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightRotationEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lightTranslationEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fxEventBoxGroups", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "basicEventTypesWithKeywords", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "useNormalEventsAsCompatibleEvents", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.waypoints = (data.waypoints ?? Lightshow.defaultValue.waypoints).map((e) => new waypoint_js_1.Waypoint(e));
        this.basicEvents = (data.basicEvents ?? Lightshow.defaultValue.basicEvents).map((e) => new basicEvent_js_1.BasicEvent(e));
        // shut the fuck up, ts, it's not that deep
        // deno-lint-ignore ban-ts-comment
        // @ts-ignore
        this.colorBoostEvents = (data.colorBoostEvents ?? Lightshow.defaultValue.colorBoostEvents).map((e) => new colorBoostEvent_js_1.ColorBoostEvent(e));
        this.lightColorEventBoxGroups = (data.lightColorEventBoxGroups ??
            Lightshow.defaultValue.lightColorEventBoxGroups).map((e) => new lightColorEventBoxGroup_js_1.LightColorEventBoxGroup(e));
        this.lightRotationEventBoxGroups = (data.lightRotationEventBoxGroups ??
            Lightshow.defaultValue.lightRotationEventBoxGroups).map((e) => new lightRotationEventBoxGroup_js_1.LightRotationEventBoxGroup(e));
        this.lightTranslationEventBoxGroups = (data.lightTranslationEventBoxGroups ??
            Lightshow.defaultValue.lightTranslationEventBoxGroups).map((e) => new lightTranslationEventBoxGroup_js_1.LightTranslationEventBoxGroup(e));
        this.fxEventBoxGroups = (data.fxEventBoxGroups ?? Lightshow.defaultValue.fxEventBoxGroups).map((e) => new fxEventBoxGroup_js_1.FxEventBoxGroup(e));
        this.basicEventTypesWithKeywords = {
            list: (data.basicEventTypesWithKeywords?.list ??
                Lightshow.defaultValue.basicEventTypesWithKeywords.list).map((e) => new basicEventTypesForKeywords_js_1.BasicEventTypesForKeywords(e)),
        };
        this.useNormalEventsAsCompatibleEvents = !!data.useNormalEventsAsCompatibleEvents;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? Lightshow.defaultValue.customData);
    }
    reconcile() {
        this.waypoints = (0, misc_js_1.reconcileClassObject)(this.waypoints, waypoint_js_1.Waypoint);
        this.basicEvents = (0, misc_js_1.reconcileClassObject)(this.basicEvents, basicEvent_js_1.BasicEvent);
        this.colorBoostEvents = (0, misc_js_1.reconcileClassObject)(this.colorBoostEvents, colorBoostEvent_js_1.ColorBoostEvent);
        this.lightColorEventBoxGroups = (0, misc_js_1.reconcileClassObject)(this.lightColorEventBoxGroups, lightColorEventBoxGroup_js_1.LightColorEventBoxGroup);
        this.lightRotationEventBoxGroups = (0, misc_js_1.reconcileClassObject)(this.lightRotationEventBoxGroups, lightRotationEventBoxGroup_js_1.LightRotationEventBoxGroup);
        this.lightTranslationEventBoxGroups = (0, misc_js_1.reconcileClassObject)(this.lightTranslationEventBoxGroups, lightTranslationEventBoxGroup_js_1.LightTranslationEventBoxGroup);
        this.fxEventBoxGroups = (0, misc_js_1.reconcileClassObject)(this.fxEventBoxGroups, fxEventBoxGroup_js_1.FxEventBoxGroup);
        this.basicEventTypesWithKeywords.list = (0, misc_js_1.reconcileClassObject)(this.basicEventTypesWithKeywords.list, basicEventTypesForKeywords_js_1.BasicEventTypesForKeywords);
        return this;
    }
    isValid(fn, override) {
        return override ? super.isValid(fn, override) : super.isValid(fn, override) &&
            this.waypoints.every((e) => e.isValid()) &&
            this.basicEvents.every((e) => e.isValid()) &&
            this.colorBoostEvents.every((e) => e.isValid()) &&
            this.lightColorEventBoxGroups.every((e) => e.isValid()) &&
            this.lightRotationEventBoxGroups.every((e) => e.isValid()) &&
            this.lightTranslationEventBoxGroups.every((e) => e.isValid()) &&
            this.fxEventBoxGroups.every((e) => e.isValid()) &&
            this.basicEventTypesWithKeywords.list.every((e) => e.isValid());
    }
    sort() {
        this.waypoints.sort(sort_js_1.sortObjectFn);
        this.basicEvents.sort(sort_js_1.sortObjectFn);
        this.colorBoostEvents.sort(sort_js_1.sortObjectFn);
        this.lightColorEventBoxGroups.sort(sort_js_1.sortObjectFn);
        this.lightRotationEventBoxGroups.sort(sort_js_1.sortObjectFn);
        this.lightTranslationEventBoxGroups.sort(sort_js_1.sortObjectFn);
        this.fxEventBoxGroups.sort(sort_js_1.sortObjectFn);
        for (let i = 0; i < this.lightColorEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightColorEventBoxGroups[i].boxes.length; j++) {
                this.lightColorEventBoxGroups[i].boxes[j].events.sort(sort_js_1.sortObjectFn);
            }
        }
        for (let i = 0; i < this.lightRotationEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightRotationEventBoxGroups[i].boxes.length; j++) {
                this.lightRotationEventBoxGroups[i].boxes[j].events.sort(sort_js_1.sortObjectFn);
            }
        }
        for (let i = 0; i < this.lightTranslationEventBoxGroups.length; i++) {
            for (let j = 0; j < this.lightTranslationEventBoxGroups[i].boxes.length; j++) {
                this.lightTranslationEventBoxGroups[i].boxes[j].events.sort(sort_js_1.sortObjectFn);
            }
        }
        for (let i = 0; i < this.fxEventBoxGroups.length; i++) {
            for (let j = 0; j < this.fxEventBoxGroups[i].boxes.length; j++) {
                this.fxEventBoxGroups[i].boxes[j].events.sort(sort_js_1.sortObjectFn);
            }
        }
        return this;
    }
    allObjects() {
        return [
            ...this.waypoints,
            ...this.basicEvents,
            ...this.colorBoostEvents,
            ...this.lightColorEventBoxGroups,
            ...this.lightRotationEventBoxGroups,
            ...this.lightTranslationEventBoxGroups,
            ...this.fxEventBoxGroups,
        ];
    }
    addWaypoints(...data) {
        for (const d of data) {
            this.waypoints.push(new waypoint_js_1.Waypoint(d));
        }
        return this;
    }
    addBasicEvents(...data) {
        for (const d of data) {
            this.basicEvents.push(new basicEvent_js_1.BasicEvent(d));
        }
        return this;
    }
    addColorBoostEvents(...data) {
        for (const d of data) {
            this.colorBoostEvents.push(new colorBoostEvent_js_1.ColorBoostEvent(d));
        }
        return this;
    }
    addLightColorEventBoxGroups(...data) {
        for (const d of data) {
            this.lightColorEventBoxGroups.push(new lightColorEventBoxGroup_js_1.LightColorEventBoxGroup(d));
        }
        return this;
    }
    addLightRotationEventBoxGroups(...data) {
        for (const d of data) {
            this.lightRotationEventBoxGroups.push(new lightRotationEventBoxGroup_js_1.LightRotationEventBoxGroup(d));
        }
        return this;
    }
    addLightTranslationEventBoxGroups(...data) {
        for (const d of data) {
            this.lightTranslationEventBoxGroups.push(new lightTranslationEventBoxGroup_js_1.LightTranslationEventBoxGroup(d));
        }
        return this;
    }
    addFxEventBoxGroups(...data) {
        for (const d of data) {
            this.fxEventBoxGroups.push(new fxEventBoxGroup_js_1.FxEventBoxGroup(d));
        }
        return this;
    }
}
exports.Lightshow = Lightshow;
Object.defineProperty(Lightshow, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createLightshow()
});
