import type { GetPositionFn, MirrorFn } from '../../types/beatmap/shared/functions.js';
import type { IWrapObstacle } from '../../types/beatmap/wrapper/obstacle.js';
import type { DeepPartial } from '../../types/utils.js';
import type { Vector2 } from '../../types/vector.js';
import { GridObject } from './abstract/gridObject.js';
export declare function createObstacle(data?: DeepPartial<IWrapObstacle>): IWrapObstacle;
/**
 * Core beatmap obstacle.
 */
export declare class Obstacle extends GridObject implements IWrapObstacle {
    static defaultValue: IWrapObstacle;
    static createOne(data?: Partial<IWrapObstacle>): Obstacle;
    static create(...data: Partial<IWrapObstacle>[]): Obstacle[];
    constructor(data?: Partial<IWrapObstacle>);
    isValid(fn?: (object: this) => boolean, override?: boolean): boolean;
    duration: IWrapObstacle['duration'];
    width: IWrapObstacle['width'];
    height: IWrapObstacle['height'];
    setDuration(value: this['duration']): this;
    setWidth(value: this['width']): this;
    setHeight(value: this['height']): this;
    mirror(_flipAlt?: boolean, fn?: MirrorFn<this>): this;
    getPosition(fn?: GetPositionFn<this>): Vector2;
    /**
     * Check if obstacle is interactive.
     * ```ts
     * if (wall.isInteractive()) {}
     * ```
     */
    isInteractive(): boolean;
    /**
     * Check if obstacle has zero value.
     * ```ts
     * if (wall.hasZero()) {}
     * ```
     */
    hasZero(): boolean;
    /**
     * Check if obstacle has negative value.
     * ```ts
     * if (wall.hasNegative()) {}
     * ```
     */
    hasNegative(): boolean;
}
//# sourceMappingURL=obstacle.d.ts.map