"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Obstacle = void 0;
exports.createObstacle = createObstacle;
const json_js_1 = require("../../utils/misc/json.js");
const vector_js_1 = require("../../utils/math/vector.js");
const gridObject_js_1 = require("../helpers/core/gridObject.js");
const obstacle_js_1 = require("../helpers/core/obstacle.js");
const constants_js_1 = require("../shared/constants.js");
const gridObject_js_2 = require("./abstract/gridObject.js");
function createObstacle(data = {}) {
    return {
        time: data.time ?? 0,
        posX: data.posX ?? 0,
        posY: data.posY ?? 0,
        width: data.width ?? 0,
        height: data.height ?? 0,
        duration: data.duration ?? 0,
        laneRotation: data.laneRotation ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap obstacle.
 */
class Obstacle extends gridObject_js_2.GridObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "duration", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "width", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "height", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? Obstacle.defaultValue.time;
        this.posX = data.posX ?? Obstacle.defaultValue.posX;
        this.posY = data.posY ?? Obstacle.defaultValue.posY;
        this.width = data.width ?? Obstacle.defaultValue.width;
        this.height = data.height ?? Obstacle.defaultValue.height;
        this.duration = data.duration ?? Obstacle.defaultValue.duration;
        this.laneRotation = data.laneRotation ?? Obstacle.defaultValue.laneRotation;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? Obstacle.defaultValue.customData);
    }
    isValid(fn, override) {
        return override
            ? super.isValid(fn, override)
            : super.isValid(fn, override) && !this.hasZero() && !this.hasNegative();
    }
    setDuration(value) {
        this.duration = value;
        return this;
    }
    setWidth(value) {
        this.width = value;
        return this;
    }
    setHeight(value) {
        this.height = value;
        return this;
    }
    mirror(_flipAlt, fn) {
        fn?.(this);
        this.posX = (0, gridObject_js_1.mirrorCoordinate)(this.posX + this.width - 1, constants_js_1.LINE_COUNT);
        return this;
    }
    getPosition(fn) {
        return fn?.(this) ?? (0, vector_js_1.vectorAdd)((0, gridObject_js_1.resolveGridPosition)(this), [-2, -0.5]);
    }
    /**
     * Check if obstacle is interactive.
     * ```ts
     * if (wall.isInteractive()) {}
     * ```
     */
    isInteractive() {
        return (0, obstacle_js_1.isInteractiveObstacle)(this);
    }
    /**
     * Check if obstacle has zero value.
     * ```ts
     * if (wall.hasZero()) {}
     * ```
     */
    hasZero() {
        return (0, obstacle_js_1.isZeroValueObstacle)(this);
    }
    /**
     * Check if obstacle has negative value.
     * ```ts
     * if (wall.hasNegative()) {}
     * ```
     */
    hasNegative() {
        return (0, obstacle_js_1.isNegativeValueObstacle)(this);
    }
}
exports.Obstacle = Obstacle;
Object.defineProperty(Obstacle, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createObstacle()
});
