"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RotationEvent = void 0;
exports.createRotationEvent = createRotationEvent;
const json_js_1 = require("../../utils/misc/json.js");
const baseObject_js_1 = require("./abstract/baseObject.js");
function createRotationEvent(data = {}) {
    return {
        time: data.time ?? 0,
        executionTime: data.executionTime ?? 0,
        rotation: data.rotation ?? 0,
        customData: (0, json_js_1.deepCopy)({ ...data.customData }),
    };
}
/**
 * Core beatmap rotation event.
 */
class RotationEvent extends baseObject_js_1.BaseObject {
    static createOne(data = {}) {
        return new this(data);
    }
    static create(...data) {
        return data.length ? data.map((obj) => new this(obj)) : [new this()];
    }
    constructor(data = {}) {
        super();
        Object.defineProperty(this, "executionTime", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "rotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        this.time = data.time ?? RotationEvent.defaultValue.time;
        this.executionTime = data.executionTime ?? RotationEvent.defaultValue.executionTime;
        this.rotation = data.rotation ?? RotationEvent.defaultValue.rotation;
        this.customData = (0, json_js_1.deepCopy)(data.customData ?? RotationEvent.defaultValue.customData);
    }
    setExecutionTime(value) {
        this.executionTime = value;
        return this;
    }
    setRotation(value) {
        this.rotation = value;
        return this;
    }
    isValid(fn, override) {
        return override
            ? super.isValid(fn, override)
            : super.isValid(fn, override) && (this.executionTime === 0 || this.executionTime === 1);
    }
}
exports.RotationEvent = RotationEvent;
Object.defineProperty(RotationEvent, "defaultValue", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: createRotationEvent()
});
