"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRedNoteColor = isRedNoteColor;
exports.isBlueNoteColor = isBlueNoteColor;
exports.isUnsetNoteColor = isUnsetNoteColor;
exports.isDouble = isDouble;
exports.resolveNoteAngle = resolveNoteAngle;
exports.mirrorNoteColor = mirrorNoteColor;
exports.mirrorNoteDirectionHorizontally = mirrorNoteDirectionHorizontally;
exports.mirrorNoteDirectionVertically = mirrorNoteDirectionVertically;
const constants_js_1 = require("../../shared/constants.js");
const iterator_js_1 = require("../../../utils/misc/iterator.js");
/** Check if note color is red. */
function isRedNoteColor(color) {
    return color === constants_js_1.NoteColor.RED;
}
/** Check if note color is blue. */
function isBlueNoteColor(color) {
    return color === constants_js_1.NoteColor.BLUE;
}
/** Check if note color is not set (-1). */
function isUnsetNoteColor(color) {
    return color === constants_js_1.NoteColor.NONE;
}
/** Compare two notes and return if they would form a double. */
function isDouble(object, compareTo, tolerance = 0.01) {
    return (compareTo.time > object.time - tolerance &&
        compareTo.time < object.time + tolerance &&
        object.color !== compareTo.color);
}
/** Get standardised note angle. */
function resolveNoteAngle(direction) {
    return constants_js_1.NoteDirectionAngle[direction] ?? 0;
}
/** Mirror note color. */
function mirrorNoteColor(color) {
    return (0, iterator_js_1.cycle)([constants_js_1.NoteColor.RED, constants_js_1.NoteColor.BLUE], color);
}
/** Mirror note direction horizontally. */
function mirrorNoteDirectionHorizontally(direction) {
    switch (direction) {
        case constants_js_1.NoteDirection.UP_LEFT:
        case constants_js_1.NoteDirection.LEFT:
        case constants_js_1.NoteDirection.DOWN_LEFT: {
            return direction + 1;
        }
        case constants_js_1.NoteDirection.UP_RIGHT:
        case constants_js_1.NoteDirection.RIGHT:
        case constants_js_1.NoteDirection.DOWN_RIGHT: {
            return direction - 1;
        }
        default: {
            return direction;
        }
    }
}
/** Mirror note direction vertically. */
function mirrorNoteDirectionVertically(direction) {
    switch (direction) {
        case constants_js_1.NoteDirection.UP_LEFT:
        case constants_js_1.NoteDirection.UP_RIGHT: {
            return direction + 2;
        }
        case constants_js_1.NoteDirection.UP: {
            return direction + 1;
        }
        case constants_js_1.NoteDirection.DOWN: {
            return direction - 1;
        }
        case constants_js_1.NoteDirection.DOWN_LEFT:
        case constants_js_1.NoteDirection.DOWN_RIGHT: {
            return direction - 2;
        }
        default: {
            return direction;
        }
    }
}
