"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isValidEventType = isValidEventType;
exports.isLightEventType = isLightEventType;
exports.isColorBoostEventType = isColorBoostEventType;
exports.isRingEventType = isRingEventType;
exports.isLaserRotationEventType = isLaserRotationEventType;
exports.isLaneRotationEventType = isLaneRotationEventType;
exports.isExtraEventType = isExtraEventType;
exports.isSpecialEventType = isSpecialEventType;
exports.isBpmChangeEventType = isBpmChangeEventType;
exports.isNjsChangeEventType = isNjsChangeEventType;
exports.isLightingEventType = isLightingEventType;
exports.isOffEventValue = isOffEventValue;
exports.isOnEventValue = isOnEventValue;
exports.isFlashEventValue = isFlashEventValue;
exports.isFadeEventValue = isFadeEventValue;
exports.isTransitionEventValue = isTransitionEventValue;
exports.isBlueEventValue = isBlueEventValue;
exports.isRedEventValue = isRedEventValue;
exports.isWhiteEventValue = isWhiteEventValue;
exports.isOldChromaEventValue = isOldChromaEventValue;
const constants_js_1 = require("../../shared/constants.js");
/** Check if event type is valid. */
function isValidEventType(type) {
    // todo: replace with enum once schema changes land
    return ((type >= 0 && type <= 19) ||
        (type >= 40 && type <= 43) ||
        type === 100 ||
        type === 1000);
}
/** Check if event type is a light event. */
function isLightEventType(type, _environment) {
    return (type === constants_js_1.EventType.BACK_LASERS ||
        type === constants_js_1.EventType.RING_LIGHTS ||
        type === constants_js_1.EventType.LEFT_LASERS ||
        type === constants_js_1.EventType.RIGHT_LASERS ||
        type === constants_js_1.EventType.CENTER_LIGHTS ||
        type === constants_js_1.EventType.EXTRA_LEFT_LIGHTS ||
        type === constants_js_1.EventType.EXTRA_RIGHT_LIGHTS ||
        type === constants_js_1.EventType.EXTRA_LEFT_LASERS ||
        type === constants_js_1.EventType.EXTRA_RIGHT_LASERS);
}
/** Check if event type is a color boost event. */
function isColorBoostEventType(type) {
    return type === constants_js_1.EventType.COLOR_BOOST;
}
/** Check if event type is a ring event. */
function isRingEventType(type, _environment) {
    return type === constants_js_1.EventType.RING_ROTATION || type === constants_js_1.EventType.RING_ZOOM;
}
/** Check if event type is a laser rotation event. */
function isLaserRotationEventType(type, _environment) {
    return (type === constants_js_1.EventType.LEFT_LASER_ROTATION ||
        type === constants_js_1.EventType.RIGHT_LASER_ROTATION);
}
/** Check if event type is a lane rotation event. */
function isLaneRotationEventType(type) {
    return (type === constants_js_1.EventType.EARLY_LANE_ROTATION ||
        type === constants_js_1.EventType.LATE_LANE_ROTATION);
}
/** Check if event type is an extra event. */
function isExtraEventType(type, _environment) {
    return (type === constants_js_1.EventType.UTILITY_EVENT_0 ||
        type === constants_js_1.EventType.UTILITY_EVENT_1 ||
        type === constants_js_1.EventType.UTILITY_EVENT_2 ||
        type === constants_js_1.EventType.UTILITY_EVENT_3);
}
/** Check if event type is a special event. */
function isSpecialEventType(type, _environment) {
    return (type === constants_js_1.EventType.SPECIAL_EVENT_0 ||
        type === constants_js_1.EventType.SPECIAL_EVENT_1 ||
        type === constants_js_1.EventType.SPECIAL_EVENT_2 ||
        type === constants_js_1.EventType.SPECIAL_EVENT_3);
}
/** Check if event type is a BPM change. */
function isBpmChangeEventType(type) {
    return type === constants_js_1.EventType.BPM_CHANGE;
}
/** Check if event type is an NJS change. */
function isNjsChangeEventType(type) {
    return type === constants_js_1.EventType.NJS_CHANGE;
}
/** Not to be confused with isLightEvent, this checks for event that affects the environment/lighting. */
function isLightingEventType(type) {
    return (isLightEventType(type) ||
        isRingEventType(type) ||
        isLaserRotationEventType(type) ||
        isExtraEventType(type));
}
/** Check if light event is an off event. */
function isOffEventValue(value) {
    return value === constants_js_1.EventLightValue.OFF;
}
/** Check if light event is an on event. */
function isOnEventValue(value) {
    return (value === constants_js_1.EventLightValue.BLUE_ON ||
        value === constants_js_1.EventLightValue.RED_ON ||
        value === constants_js_1.EventLightValue.WHITE_ON);
}
/** Check if light event is a flash event. */
function isFlashEventValue(value) {
    return (value === constants_js_1.EventLightValue.BLUE_FLASH ||
        value === constants_js_1.EventLightValue.RED_FLASH ||
        value === constants_js_1.EventLightValue.WHITE_FLASH);
}
/** Check if light event is a fade event. */
function isFadeEventValue(value) {
    return (value === constants_js_1.EventLightValue.BLUE_FADE ||
        value === constants_js_1.EventLightValue.RED_FADE ||
        value === constants_js_1.EventLightValue.WHITE_FADE);
}
/** Check if light event is a transition event. */
function isTransitionEventValue(value) {
    return (value === constants_js_1.EventLightValue.BLUE_TRANSITION ||
        value === constants_js_1.EventLightValue.RED_TRANSITION ||
        value === constants_js_1.EventLightValue.WHITE_TRANSITION);
}
/** Check if light event is a blue light. */
function isBlueEventValue(value) {
    return (value === constants_js_1.EventLightValue.BLUE_ON ||
        value === constants_js_1.EventLightValue.BLUE_FLASH ||
        value === constants_js_1.EventLightValue.BLUE_FADE ||
        value === constants_js_1.EventLightValue.BLUE_TRANSITION);
}
/** Check if light event is a red light. */
function isRedEventValue(value) {
    return (value === constants_js_1.EventLightValue.RED_ON ||
        value === constants_js_1.EventLightValue.RED_FLASH ||
        value === constants_js_1.EventLightValue.RED_FADE ||
        value === constants_js_1.EventLightValue.RED_TRANSITION);
}
/** Check if light event is a white light. */
function isWhiteEventValue(value) {
    return (value === constants_js_1.EventLightValue.WHITE_ON ||
        value === constants_js_1.EventLightValue.WHITE_FLASH ||
        value === constants_js_1.EventLightValue.WHITE_FADE ||
        value === constants_js_1.EventLightValue.WHITE_TRANSITION);
}
/** Check if event has old Chroma properties. */
function isOldChromaEventValue(value) {
    return value >= 2000000000;
}
