"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveGridPosition = resolveGridPosition;
exports.resolveGridDistance = resolveGridDistance;
exports.isHorizontal = isHorizontal;
exports.isVertical = isVertical;
exports.isDiagonal = isDiagonal;
exports.isInline = isInline;
exports.isAdjacent = isAdjacent;
exports.isWindow = isWindow;
exports.isSlantedWindow = isSlantedWindow;
exports.mirrorCoordinate = mirrorCoordinate;
const vector_js_1 = require("../../../utils/math/vector.js");
/** Get standardized grid position. */
function resolveGridPosition(object) {
    return [object.posX, object.posY];
}
/** Get standardized grid distance. */
function resolveGridDistance(o1, o2, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return (0, vector_js_1.vectorDistance)(v1, v2);
}
/** Check if two objects are in horizontal alignment. */
function isHorizontal(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return (0, vector_js_1.vectorIsHorizontal)(v1, v2, epsilon);
}
/** Check if two objects are in vertical alignment. */
function isVertical(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return (0, vector_js_1.vectorIsVertical)(v1, v2, epsilon);
}
/** Check if two objects are in diagonal alignment. */
function isDiagonal(o1, o2, epsilon = 0.001, fn) {
    const v1 = fn?.(o1) ?? resolveGridPosition(o1);
    const v2 = fn?.(o2) ?? resolveGridPosition(o2);
    return (0, vector_js_1.vectorIsDiagonal)(v1, v2, epsilon);
}
/** Check if two objects are inline. */
function isInline(o1, o2, lapping = 0.5, fn) {
    return resolveGridDistance(o1, o2, fn) <= lapping;
}
/** Check if two objects are adjacent. */
function isAdjacent(o1, o2, epsilon = 0.001, fn) {
    const distance = resolveGridDistance(o1, o2, fn);
    return distance > (0.5 - epsilon) && distance < (1 + epsilon);
}
/** Check if two objects form a "window". */
function isWindow(o1, o2, distance = 1.8, fn) {
    return resolveGridDistance(o1, o2, fn) > distance;
}
/** Check if two objects form a "slanted window". */
function isSlantedWindow(o1, o2, fn) {
    return (isWindow(o1, o2, 1.8, fn) &&
        !isDiagonal(o1, o2, 0.001, fn) &&
        !isHorizontal(o1, o2, 0.001, fn) &&
        !isVertical(o1, o2, 0.001, fn));
}
function mirrorCoordinate(x, axis) {
    return axis - 1 - x;
}
