import type { IWrapArc } from '../../../types/beatmap/wrapper/arc.js';
import type { IWrapBasicEvent } from '../../../types/beatmap/wrapper/basicEvent.js';
import type { IWrapBombNote } from '../../../types/beatmap/wrapper/bombNote.js';
import type { IWrapChain } from '../../../types/beatmap/wrapper/chain.js';
import type { IWrapColorNote } from '../../../types/beatmap/wrapper/colorNote.js';
import type { IWrapObstacle } from '../../../types/beatmap/wrapper/obstacle.js';
import type { IWrapRotationEvent } from '../../../types/beatmap/wrapper/rotationEvent.js';
/**
 * Checks if basic event has Chroma in beatmap v2.
 */
export declare function hasChromaEventV2<T extends Pick<IWrapBasicEvent, 'customData'>>(data: T): boolean;
/**
 * Checks if note has Chroma in beatmap v2.
 */
export declare function hasChromaNoteV2<T extends Pick<IWrapColorNote | IWrapBombNote, 'customData'>>(data: T): boolean;
/**
 * Checks if note has Noodle Extensions in beatmap v2.
 */
export declare function hasNoodleExtensionsNoteV2<T extends Pick<IWrapColorNote | IWrapBombNote, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Chroma in beatmap v2
 */
export declare function hasChromaObstacleV2<T extends Pick<IWrapObstacle, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Noodle Extensions in beatmap v2.
 */
export declare function hasNoodleExtensionsObstacleV2<T extends Pick<IWrapObstacle, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Mapping Extensions in beatmap v2.
 */
export declare function hasMappingExtensionsObstacleV2<T extends Pick<IWrapObstacle, 'posX' | 'posY' | 'width' | 'height'>>(data: T): boolean;
/**
 * Checks if rotation event has Noodle Extensions in beatmap v2.
 */
export declare function hasNoodleExtensionsRotationV2<T extends Pick<IWrapRotationEvent, 'customData'>>(data: T): boolean;
/**
 * Checks if rotation event has Mapping Extensions in beatmap v2.
 */
export declare function hasMappingExtensionsRotationV2<T extends Pick<IWrapRotationEvent, 'rotation'>>(data: T): boolean;
/**
 * Checks if basic event has Chroma in beatmap v3.
 */
export declare function hasChromaEventV3<T extends Pick<IWrapBasicEvent, 'customData'>>(data: T): boolean;
/**
 * Checks if note has Chroma in beatmap v3.
 */
export declare function hasChromaNoteV3<T extends Pick<IWrapColorNote | IWrapBombNote, 'customData'>>(data: T): boolean;
/**
 * Checks if note has Noodle Extensions in beatmap v3.
 */
export declare function hasNoodleExtensionsNoteV3<T extends Pick<IWrapColorNote | IWrapBombNote, 'customData'>>(data: T): boolean;
/**
 * Checks if slider has Noodle Extensions in beatmap v3.
 */
export declare function hasNoodleExtensionsSliderV3<T extends Pick<IWrapArc | IWrapChain, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Chroma in beatmap v3.
 */
export declare function hasChromaObstacleV3<T extends Pick<IWrapObstacle, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Noodle Extensions in beatmap v3.
 */
export declare function hasNoodleExtensionsObstacleV3<T extends Pick<IWrapObstacle, 'customData'>>(data: T): boolean;
/**
 * Checks if obstacle has Mapping Extensions in beatmap v3.
 */
export declare function hasMappingExtensionsObstacleV3<T extends Pick<IWrapObstacle, 'posX' | 'posY' | 'width' | 'height'>>(data: T): boolean;
/**
 * Checks if arc has Mapping Extensions in beatmap v3.
 */
export declare function hasMappingExtensionsArc<T extends Pick<IWrapArc, 'posX' | 'posY' | 'direction' | 'tailDirection'>>(data: T): boolean;
/**
 * Checks if bomb note has Mapping Extensions in beatmap v3.
 */
export declare function hasMappingExtensionsBombNote<T extends Pick<IWrapBombNote, 'posX' | 'posY'>>(data: T): boolean;
/**
 * Checks if chain has Mapping Extensions in beatmap v3.
 */
export declare function hasMappingExtensionsChain<T extends Pick<IWrapChain, 'posX' | 'posY' | 'direction'>>(data: T): boolean;
/**
 * Checks if note has Mapping Extensions.
 */
export declare function hasMappingExtensionsNote<T extends Pick<IWrapColorNote, 'posX' | 'posY' | 'direction'>>(data: T): boolean;
//# sourceMappingURL=has.d.ts.map