"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasChromaEventV2 = hasChromaEventV2;
exports.hasChromaNoteV2 = hasChromaNoteV2;
exports.hasNoodleExtensionsNoteV2 = hasNoodleExtensionsNoteV2;
exports.hasChromaObstacleV2 = hasChromaObstacleV2;
exports.hasNoodleExtensionsObstacleV2 = hasNoodleExtensionsObstacleV2;
exports.hasMappingExtensionsObstacleV2 = hasMappingExtensionsObstacleV2;
exports.hasNoodleExtensionsRotationV2 = hasNoodleExtensionsRotationV2;
exports.hasMappingExtensionsRotationV2 = hasMappingExtensionsRotationV2;
exports.hasChromaEventV3 = hasChromaEventV3;
exports.hasChromaNoteV3 = hasChromaNoteV3;
exports.hasNoodleExtensionsNoteV3 = hasNoodleExtensionsNoteV3;
exports.hasNoodleExtensionsSliderV3 = hasNoodleExtensionsSliderV3;
exports.hasChromaObstacleV3 = hasChromaObstacleV3;
exports.hasNoodleExtensionsObstacleV3 = hasNoodleExtensionsObstacleV3;
exports.hasMappingExtensionsObstacleV3 = hasMappingExtensionsObstacleV3;
exports.hasMappingExtensionsArc = hasMappingExtensionsArc;
exports.hasMappingExtensionsBombNote = hasMappingExtensionsBombNote;
exports.hasMappingExtensionsChain = hasMappingExtensionsChain;
exports.hasMappingExtensionsNote = hasMappingExtensionsNote;
const constants_js_1 = require("../../shared/constants.js");
/**
 * Checks if basic event has Chroma in beatmap v2.
 */
function hasChromaEventV2(data) {
    return (Array.isArray(data.customData._color) ||
        typeof data.customData._lightID === 'number' ||
        Array.isArray(data.customData._lightID) ||
        typeof data.customData._propID === 'number' ||
        typeof data.customData._lightGradient === 'object' ||
        typeof data.customData._easing === 'string' ||
        typeof data.customData._lerpType === 'string' ||
        typeof data.customData._nameFilter === 'string' ||
        typeof data.customData._reset === 'boolean' ||
        typeof data.customData._rotation === 'number' ||
        typeof data.customData._step === 'number' ||
        typeof data.customData._prop === 'number' ||
        typeof data.customData._speed === 'number' ||
        typeof data.customData._direction === 'number' ||
        typeof data.customData._counterSpin === 'boolean' ||
        typeof data.customData._stepMult === 'number' ||
        typeof data.customData._propMult === 'number' ||
        typeof data.customData._speedMult === 'number' ||
        typeof data.customData._lockPosition === 'boolean' ||
        typeof data.customData._speed === 'number' ||
        typeof data.customData._preciseSpeed === 'number' ||
        typeof data.customData._direction === 'number');
}
/**
 * Checks if note has Chroma in beatmap v2.
 */
function hasChromaNoteV2(data) {
    return (Array.isArray(data.customData._color) ||
        typeof data.customData._disableSpawnEffect === 'boolean');
}
/**
 * Checks if note has Noodle Extensions in beatmap v2.
 */
function hasNoodleExtensionsNoteV2(data) {
    return (Array.isArray(data.customData._animation) ||
        typeof data.customData._cutDirection === 'number' ||
        typeof data.customData._disableNoteGravity === 'boolean' ||
        typeof data.customData._disableNoteLook === 'boolean' ||
        typeof data.customData._fake === 'boolean' ||
        Array.isArray(data.customData._flip) ||
        typeof data.customData._interactable === 'boolean' ||
        Array.isArray(data.customData._localRotation) ||
        typeof data.customData._noteJumpMovementSpeed === 'number' ||
        typeof data.customData._noteJumpStartBeatOffset === 'number' ||
        Array.isArray(data.customData._position) ||
        Array.isArray(data.customData._rotation));
}
/**
 * Checks if obstacle has Chroma in beatmap v2
 */
function hasChromaObstacleV2(data) {
    return Array.isArray(data.customData._color);
}
/**
 * Checks if obstacle has Noodle Extensions in beatmap v2.
 */
function hasNoodleExtensionsObstacleV2(data) {
    return (Array.isArray(data.customData._animation) ||
        typeof data.customData._fake === 'boolean' ||
        typeof data.customData._interactable === 'boolean' ||
        Array.isArray(data.customData._localRotation) ||
        typeof data.customData._noteJumpMovementSpeed === 'number' ||
        typeof data.customData._noteJumpStartBeatOffset === 'number' ||
        Array.isArray(data.customData._position) ||
        Array.isArray(data.customData._rotation) ||
        Array.isArray(data.customData._scale));
}
/**
 * Checks if obstacle has Mapping Extensions in beatmap v2.
 */
function hasMappingExtensionsObstacleV2(data) {
    return (data.posX < 0 ||
        data.posX > 3 ||
        data.width < 0 ||
        data.width > 4 ||
        data.posY < 0 ||
        data.posY > 2 ||
        data.posY === 1 ||
        (data.posY === 0 && data.height !== 5) ||
        (data.posY === 2 && data.height !== 3));
}
/**
 * Checks if rotation event has Noodle Extensions in beatmap v2.
 */
function hasNoodleExtensionsRotationV2(data) {
    return typeof data.customData._rotation === 'number';
}
/**
 * Checks if rotation event has Mapping Extensions in beatmap v2.
 */
function hasMappingExtensionsRotationV2(data) {
    return !constants_js_1.RotationValueEventValue[data.rotation];
}
/**
 * Checks if basic event has Chroma in beatmap v3.
 */
function hasChromaEventV3(data) {
    return (Array.isArray(data.customData.color) ||
        typeof data.customData.lightID === 'number' ||
        Array.isArray(data.customData.lightID) ||
        typeof data.customData.easing === 'string' ||
        typeof data.customData.lerpType === 'string' ||
        typeof data.customData.nameFilter === 'string' ||
        typeof data.customData.rotation === 'number' ||
        typeof data.customData.step === 'number' ||
        typeof data.customData.prop === 'number' ||
        typeof data.customData.speed === 'number' ||
        typeof data.customData.direction === 'number' ||
        typeof data.customData.lockRotation === 'boolean' ||
        typeof data.customData.speed === 'number' ||
        typeof data.customData.direction === 'number');
}
/**
 * Checks if note has Chroma in beatmap v3.
 */
function hasChromaNoteV3(data) {
    return (Array.isArray(data.customData.color) ||
        typeof data.customData.spawnEffect === 'boolean' ||
        typeof data.customData.disableDebris === 'boolean');
}
/**
 * Checks if note has Noodle Extensions in beatmap v3.
 */
function hasNoodleExtensionsNoteV3(data) {
    return (Array.isArray(data.customData.animation) ||
        typeof data.customData.disableNoteGravity === 'boolean' ||
        typeof data.customData.disableNoteLook === 'boolean' ||
        typeof data.customData.disableBadCutDirection === 'boolean' ||
        typeof data.customData.disableBadCutSaberType === 'boolean' ||
        typeof data.customData.disableBadCutSpeed === 'boolean' ||
        Array.isArray(data.customData.flip) ||
        typeof data.customData.uninteractable === 'boolean' ||
        Array.isArray(data.customData.localRotation) ||
        typeof data.customData.noteJumpMovementSpeed === 'number' ||
        typeof data.customData.noteJumpStartBeatOffset === 'number' ||
        Array.isArray(data.customData.coordinates) ||
        Array.isArray(data.customData.worldRotation) ||
        typeof data.customData.worldRotation === 'number' ||
        typeof data.customData.link === 'string');
}
/**
 * Checks if slider has Noodle Extensions in beatmap v3.
 */
function hasNoodleExtensionsSliderV3(data) {
    return (Array.isArray(data.customData.animation) ||
        typeof data.customData.disableNoteGravity === 'boolean' ||
        typeof data.customData.disableNoteLook === 'boolean' ||
        typeof data.customData.disableBadCutDirection === 'boolean' ||
        typeof data.customData.disableBadCutSaberType === 'boolean' ||
        typeof data.customData.disableBadCutSpeed === 'boolean' ||
        Array.isArray(data.customData.flip) ||
        typeof data.customData.uninteractable === 'boolean' ||
        Array.isArray(data.customData.localRotation) ||
        typeof data.customData.noteJumpMovementSpeed === 'number' ||
        typeof data.customData.noteJumpStartBeatOffset === 'number' ||
        Array.isArray(data.customData.coordinates) ||
        Array.isArray(data.customData.tailCoordinates) ||
        Array.isArray(data.customData.worldRotation) ||
        typeof data.customData.worldRotation === 'number' ||
        typeof data.customData.link === 'string');
}
/**
 * Checks if obstacle has Chroma in beatmap v3.
 */
function hasChromaObstacleV3(data) {
    return Array.isArray(data.customData.color);
}
/**
 * Checks if obstacle has Noodle Extensions in beatmap v3.
 */
function hasNoodleExtensionsObstacleV3(data) {
    return (Array.isArray(data.customData.animation) ||
        typeof data.customData.uninteractable === 'boolean' ||
        Array.isArray(data.customData.localRotation) ||
        typeof data.customData.noteJumpMovementSpeed === 'number' ||
        typeof data.customData.noteJumpStartBeatOffset === 'number' ||
        Array.isArray(data.customData.coordinates) ||
        Array.isArray(data.customData.worldRotation) ||
        Array.isArray(data.customData.size));
}
/**
 * Checks if obstacle has Mapping Extensions in beatmap v3.
 */
function hasMappingExtensionsObstacleV3(data) {
    return (data.posY < 0 ||
        data.posY > 2 ||
        data.posX <= -1000 ||
        data.posX >= 1000 ||
        data.width <= -1000 ||
        data.width >= 1000 ||
        data.height <= -1000 ||
        data.height >= 1000);
}
/**
 * Checks if arc has Mapping Extensions in beatmap v3.
 */
function hasMappingExtensionsArc(data) {
    return (data.posY > 2 ||
        data.posY < 0 ||
        data.posX <= -1000 ||
        data.posX >= 1000 ||
        (data.direction >= 1000 && data.direction <= 1360) ||
        (data.tailDirection >= 1000 && data.tailDirection <= 1360));
}
/**
 * Checks if bomb note has Mapping Extensions in beatmap v3.
 */
function hasMappingExtensionsBombNote(data) {
    return data.posX > 3 || data.posX < 0 || data.posY > 2 || data.posY < 0;
}
/**
 * Checks if chain has Mapping Extensions in beatmap v3.
 */
function hasMappingExtensionsChain(data) {
    return (data.posY > 2 ||
        data.posY < 0 ||
        data.posX <= -1000 ||
        data.posX >= 1000 ||
        (data.direction >= 1000 && data.direction <= 1360));
}
/**
 * Checks if note has Mapping Extensions.
 */
function hasMappingExtensionsNote(data) {
    return (data.posX > 3 ||
        data.posX < 0 ||
        data.posY > 2 ||
        data.posY < 0 ||
        (data.direction >= 1000 && data.direction <= 1360) ||
        (data.direction >= 2000 && data.direction <= 2360));
}
