"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScoreValue = exports.ScoreType = void 0;
exports.calculateScore = calculateScore;
const helpers_js_1 = require("../../utils/math/helpers.js");
/** Scoring type of note. */
var ScoreType;
(function (ScoreType) {
    ScoreType[ScoreType["NONE"] = 0] = "NONE";
    ScoreType[ScoreType["NORMAL"] = 1] = "NORMAL";
    ScoreType[ScoreType["ARC_HEAD"] = 2] = "ARC_HEAD";
    ScoreType[ScoreType["ARC_TAIL"] = 3] = "ARC_TAIL";
    ScoreType[ScoreType["CHAIN_HEAD"] = 4] = "CHAIN_HEAD";
    ScoreType[ScoreType["CHAIN_ELEMENT"] = 5] = "CHAIN_ELEMENT";
})(ScoreType || (exports.ScoreType = ScoreType = {}));
/** Score value given by each scoring type of note. */
exports.ScoreValue = {
    [ScoreType.NONE]: 0,
    [ScoreType.NORMAL]: 115,
    [ScoreType.ARC_HEAD]: 115,
    [ScoreType.ARC_TAIL]: 115,
    [ScoreType.CHAIN_HEAD]: 85,
    [ScoreType.CHAIN_ELEMENT]: 20,
};
/** Calculate max score from beatmap. */
function calculateScore(beatmap) {
    let total = 0;
    let multiplier = 1;
    const elements = generateScoreElement(beatmap);
    for (let i = 0; i < elements.length; i++) {
        if (multiplier < 8 && i === -3 + multiplier * 4) {
            multiplier *= 2;
        }
        total += exports.ScoreValue[elements[i].type] * multiplier;
    }
    return total;
}
function generateScoreElement(beatmap) {
    return [
        createNoteElement(beatmap),
        createChainElement(beatmap.difficulty.chains),
    ]
        .flat()
        .sort((a, b) => a.time - b.time);
}
// for now I only care about arc head/tail and chain head, arc takes priority
// FIXME: use epsilon dammit
function createNoteElement(beatmap) {
    const mapArcHead = new Map();
    const mapArcTail = new Map();
    const mapChainHead = new Map();
    for (let i = 0; i < beatmap.difficulty.arcs.length; i++) {
        const arc = beatmap.difficulty.arcs[i];
        if (!mapArcHead.has(arc.time)) {
            mapArcHead.set(arc.time, []);
        }
        if (!mapArcTail.has(arc.tailTime)) {
            mapArcTail.set(arc.tailTime, []);
        }
        mapArcHead.get(arc.time).push(arc);
        mapArcTail.get(arc.tailTime).push(arc);
    }
    for (let i = 0; i < beatmap.difficulty.chains.length; i++) {
        const chain = beatmap.difficulty.chains[i];
        if (!mapChainHead.has(chain.time)) {
            mapChainHead.set(chain.time, []);
        }
        mapChainHead.get(chain.time).push(chain);
    }
    return beatmap.difficulty.colorNotes.map((n) => {
        const hasArc = mapArcHead
            .get(n.time)
            ?.some((a) => a.color === n.color &&
            a.tailPosX === n.posX &&
            a.tailPosY === n.posY) ||
            mapArcTail
                .get(n.time)
                ?.some((a) => a.color === n.color &&
                a.tailPosX === n.posX &&
                a.tailPosY === n.posY);
        if (hasArc) {
            return {
                type: ScoreType.ARC_HEAD,
                time: n.time,
            };
        }
        const hasChain = mapChainHead
            .get(n.time)
            ?.some((c) => c.color === n.color && c.posX === n.posX && c.posY === n.posY);
        if (hasChain) {
            return {
                type: ScoreType.CHAIN_HEAD,
                time: n.time,
            };
        }
        return {
            type: ScoreType.NORMAL,
            time: n.time,
        };
    });
}
function createChainElement(chains) {
    return chains.flatMap((c) => {
        if (c.sliceCount > 1) {
            return new Array(c.sliceCount - 1).map((_, i) => {
                return {
                    type: ScoreType.CHAIN_ELEMENT,
                    time: (0, helpers_js_1.lerp)((0, helpers_js_1.normalize)(i + 1, 0, c.sliceCount), c.time, c.tailTime),
                };
            });
        }
        return [];
    });
}
