import type { INote } from '../../types/beatmap/v2/note.js';
import type { IBaseObject as IV2BaseObject } from '../../types/beatmap/v2/object.js';
import type { IBaseObject as IV3BaseObject } from '../../types/beatmap/v3/baseObject.js';
import type { IGridObject } from '../../types/beatmap/v3/gridObject.js';
import type { IWrapBaseObject } from '../../types/beatmap/wrapper/baseObject.js';
import type { IWrapGridObject } from '../../types/beatmap/wrapper/gridObject.js';
/**
 * Pass this to wrapper object array `sort` function as an argument.
 *
 * ```ts
 * data.basicEvents.sort(sortObjectFn);
 * ```
 */
export declare function sortObjectFn<T extends Pick<IWrapBaseObject, 'time'>>(a: T, b: T): number;
/**
 * Pass this to wrapper note type array `sort` function as an argument.
 *
 * ```ts
 * data.chains.sort(sortNoteFn);
 * ```
 */
export declare function sortNoteFn<T extends Pick<IWrapGridObject, 'time' | 'posX' | 'posY' | 'customData'>>(a: T, b: T): number;
/**
 * Pass this to schema v1 or v2 object array `sort` function as an argument.
 *
 * ```ts
 * data._events.sort(sortV2ObjectFn);
 * ```
 */
export declare function sortV2ObjectFn<T extends Pick<IV2BaseObject, '_time'>>(a: T, b: T): number;
/**
 * Pass this to schema v1 or v2 note type array `sort` function as an argument.
 *
 * ```ts
 * data._notes.sort(sortV2NoteFn);
 * ```
 */
export declare function sortV2NoteFn<T extends Pick<INote, '_time' | '_lineIndex' | '_lineLayer' | '_customData'>>(a: T, b: T): number;
/**
 * Pass this to schema v3 object array `sort` function as an argument.
 *
 * ```ts
 * data.basicBeatmapEvents.sort(sortV3ObjectFn);
 * ```
 */
export declare function sortV3ObjectFn<T extends Pick<IV3BaseObject, 'b'>>(a: T, b: T): number;
/**
 * Pass this to schema v3 note type array `sort` function as an argument.
 *
 * ```ts
 * data.arcs.sort(sortV3NoteFn);
 * ```
 */
export declare function sortV3NoteFn<T extends Pick<IGridObject, 'b' | 'x' | 'y' | 'customData'>>(a: T, b: T): number;
//# sourceMappingURL=sort.d.ts.map