"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tag = tag;
exports.loadBeatmap = loadBeatmap;
// deno-lint-ignore-file no-explicit-any
const logger_js_1 = require("../../logger.js");
const version_js_1 = require("../helpers/version.js");
const converter_js_1 = require("../mapping/converter.js");
const schema_js_1 = require("../mapping/schema.js");
const json_js_1 = require("../validator/json.js");
function tag(name) {
    return ['loader', name];
}
const defaultOptions = {
    forceConvert: true,
    schemaCheck: {},
    sort: true,
    preprocess: [],
    postprocess: [],
};
function loadBeatmap(type, json, version, options = {}) {
    const optD = (typeof version !== 'number' ? version : options) ?? options ?? {};
    const opt = {
        forceConvert: optD.forceConvert ?? defaultOptions.forceConvert,
        schemaCheck: { ...defaultOptions.schemaCheck, ...optD.schemaCheck },
        sort: optD.sort ?? defaultOptions.sort,
        preprocess: optD.preprocess ?? defaultOptions.preprocess,
        postprocess: optD.postprocess ?? defaultOptions.postprocess,
    };
    const [pretransformer, ...preprocesses] = opt.preprocess;
    let serial = pretransformer
        ? pretransformer(json, version)
        : json;
    preprocesses.forEach((fn, i) => {
        logger_js_1.logger.tInfo(tag('loadBeatmap'), 'Running preprocess function #' + (i + 1));
        serial = fn(serial);
    });
    const jsonVerStr = (0, version_js_1.retrieveVersion)(serial)?.at(0);
    let jsonVer;
    if (jsonVerStr) {
        jsonVer = parseInt(jsonVerStr);
    }
    else {
        jsonVer = +(0, version_js_1.implicitVersion)(type).at(0);
        logger_js_1.logger.tWarn(tag('loadBeatmap'), 'Could not identify beatmap version from JSON, assume implicit version', jsonVer);
    }
    let attribute;
    if (opt.schemaCheck.enabled)
        (0, json_js_1.validateJSON)(type, serial, jsonVer, opt.schemaCheck);
    attribute = (0, schema_js_1.deserializeBeatmap)(type, jsonVer, serial);
    if (version && jsonVer !== version) {
        if (!opt.forceConvert) {
            throw new Error(`Beatmap version unmatched, expected ${version} but received ${jsonVer}`);
        }
        logger_js_1.logger.tWarn(tag('loadBeatmap'), 'Beatmap version unmatched, expected', version, 'but received', jsonVer, 'for version; Converting to beatmap version', version);
        attribute = (0, converter_js_1.convertBeatmap)(type, version, attribute, attribute.version);
    }
    if (opt.sort && 'sort' in attribute && typeof attribute.sort === 'function') {
        attribute.sort();
    }
    const [posttransformer, ...postprocesses] = [...opt.postprocess].reverse();
    postprocesses.forEach((fn, i) => {
        logger_js_1.logger.tInfo(tag('loadBeatmap'), 'Running postprocess function #' + (i + 1));
        attribute = fn(attribute);
    });
    const wrapper = posttransformer
        ? posttransformer(attribute, version ?? null)
        : attribute;
    return wrapper;
}
