"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightshowOptimizeMap = exports.difficultyOptimizeMap = exports.infoOptimizeMap = void 0;
exports.optimizeBeatmap = optimizeBeatmap;
const difficulty_js_1 = require("../schema/v1/optimize/difficulty.js");
const info_js_1 = require("../schema/v1/optimize/info.js");
const difficulty_js_2 = require("../schema/v2/optimize/difficulty.js");
const info_js_2 = require("../schema/v2/optimize/info.js");
const difficulty_js_3 = require("../schema/v3/optimize/difficulty.js");
const lightshow_js_1 = require("../schema/v3/optimize/lightshow.js");
const difficulty_js_4 = require("../schema/v4/optimize/difficulty.js");
const info_js_3 = require("../schema/v4/optimize/info.js");
const lightshow_js_2 = require("../schema/v4/optimize/lightshow.js");
/** Optimize function version map for schema beatmap info. */
exports.infoOptimizeMap = {
    1: info_js_1.optimizeInfo,
    2: info_js_2.optimizeInfo,
    4: info_js_3.optimizeInfo,
};
/** Optimize function version map for schema beatmap difficulty. */
exports.difficultyOptimizeMap = {
    1: difficulty_js_1.optimizeDifficulty,
    2: difficulty_js_2.optimizeDifficulty,
    3: difficulty_js_3.optimizeDifficulty,
    4: difficulty_js_4.optimizeDifficulty,
};
/** Optimize function version map for schema beatmap lightshow. */
exports.lightshowOptimizeMap = {
    3: lightshow_js_1.optimizeLightshow,
    4: lightshow_js_2.optimizeLightshow,
};
function optimizeBeatmap(type, ver, data, options) {
    switch (type) {
        case 'info': {
            const optimize = exports.infoOptimizeMap[ver];
            return optimize(data, options);
        }
        case 'difficulty': {
            const optimize = exports.difficultyOptimizeMap[ver];
            return optimize(data, options);
        }
        case 'lightshow': {
            const optimize = exports.lightshowOptimizeMap[ver];
            return optimize(data, options);
        }
        default: {
            return data;
        }
    }
}
