"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightshowSchemaMap = exports.difficultySchemaMap = exports.audioDataSchemaMap = exports.infoSchemaMap = void 0;
exports.serializeBeatmap = serializeBeatmap;
exports.deserializeBeatmap = deserializeBeatmap;
const difficulty_js_1 = require("../schema/v1/difficulty.js");
const info_js_1 = require("../schema/v1/info.js");
const audioData_js_1 = require("../schema/v2/audioData.js");
const difficulty_js_2 = require("../schema/v2/difficulty.js");
const info_js_2 = require("../schema/v2/info.js");
const difficulty_js_3 = require("../schema/v3/difficulty.js");
const lightshow_js_1 = require("../schema/v3/lightshow.js");
const audioData_js_2 = require("../schema/v4/audioData.js");
const difficulty_js_4 = require("../schema/v4/difficulty.js");
const info_js_3 = require("../schema/v4/info.js");
const lightshow_js_2 = require("../schema/v4/lightshow.js");
/** Schema version map for beatmap info. */
exports.infoSchemaMap = {
    1: info_js_1.info,
    2: info_js_2.info,
    4: info_js_3.info,
};
/** Schema version map for beatmap audio data. */
exports.audioDataSchemaMap = {
    2: audioData_js_1.audioData,
    4: audioData_js_2.audioData,
};
/** Schema version map for beatmap difficulty. */
exports.difficultySchemaMap = {
    1: difficulty_js_1.difficulty,
    2: difficulty_js_2.difficulty,
    3: difficulty_js_3.difficulty,
    4: difficulty_js_4.difficulty,
};
/** Schema version map for beatmap lightshow. */
exports.lightshowSchemaMap = {
    3: lightshow_js_1.lightshow,
    4: lightshow_js_2.lightshow,
};
function serializeBeatmap(type, ver, data) {
    switch (type) {
        case 'info': {
            const container = exports.infoSchemaMap[ver];
            return container.serialize(data);
        }
        case 'audioData': {
            const container = exports.audioDataSchemaMap[ver];
            return container.serialize(data);
        }
        case 'difficulty': {
            const container = exports.difficultySchemaMap[ver];
            return container.serialize(data);
        }
        case 'lightshow': {
            const container = exports.lightshowSchemaMap[ver];
            return container.serialize(data);
        }
        default: {
            throw new Error('');
        }
    }
}
function deserializeBeatmap(type, ver, data) {
    switch (type) {
        case 'info': {
            const container = exports.infoSchemaMap[ver];
            return container.deserialize(data);
        }
        case 'audioData': {
            const container = exports.audioDataSchemaMap[ver];
            return container.deserialize(data);
        }
        case 'difficulty': {
            const container = exports.difficultySchemaMap[ver];
            return container.deserialize(data);
        }
        case 'lightshow': {
            const container = exports.lightshowSchemaMap[ver];
            return container.deserialize(data);
        }
        default: {
            throw new Error('');
        }
    }
}
