"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compatDifficulty = compatDifficulty;
const logger_js_1 = require("../../../../logger.js");
const has_js_1 = require("../../../helpers/modded/has.js");
const _common_js_1 = require("./_common.js");
/**
 * Checks if beatmap data is compatible with v1 `Difficulty` schema.
 */
function compatDifficulty(bm, options) {
    const hasIncompat = !!bm.difficulty.arcs.length ||
        !!bm.difficulty.chains.length ||
        !!bm.lightshow.waypoints.length ||
        !!bm.difficulty.bpmEvents.length ||
        !!bm.lightshow.lightColorEventBoxGroups.length ||
        !!bm.lightshow.lightRotationEventBoxGroups.length ||
        !!bm.lightshow.lightTranslationEventBoxGroups.length ||
        !!bm.lightshow.fxEventBoxGroups.length;
    if (hasIncompat) {
        if (options.throwOn.incompatibleObject) {
            throw new Error('Beatmap is not compatible with v1');
        }
        else {
            logger_js_1.logger.tWarn((0, _common_js_1.tag)('compatDifficulty'), 'Beatmap is not compatible with v1, certain data may be lost!');
        }
    }
    const hasME = bm.difficulty.colorNotes.some(has_js_1.hasMappingExtensionsNote) ||
        bm.difficulty.bombNotes.some(has_js_1.hasMappingExtensionsBombNote) ||
        bm.difficulty.obstacles.some(has_js_1.hasMappingExtensionsObstacleV2) ||
        bm.difficulty.rotationEvents.some(has_js_1.hasMappingExtensionsRotationV2);
    if (hasME) {
        if (options.throwOn.mappingExtensions) {
            throw new Error('Beatmap contain Mapping Extensions value');
        }
        else {
            logger_js_1.logger.tWarn((0, _common_js_1.tag)('compatDifficulty'), 'Beatmap contain Mapping Extensions value and require Mapping Extensions mod to function!');
        }
    }
}
