"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.difficulty = void 0;
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const basicEvent_js_1 = require("./basicEvent.js");
const bombNote_js_1 = require("./bombNote.js");
const bpmEvent_js_1 = require("./bpmEvent.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const colorNote_js_1 = require("./colorNote.js");
const obstacle_js_1 = require("./obstacle.js");
const rotationEvent_js_1 = require("./rotationEvent.js");
/**
 * Schema serialization for v1 `Difficulty`.
 */
exports.difficulty = {
    serialize(data, options) {
        return {
            _version: '1.5.0',
            // FIXME: none of these shouldve ever existed, why
            _beatsPerMinute: options?.bpm ?? 120,
            _beatsPerBar: options?.beatsPerBar ?? 4,
            _shuffle: options?.shuffle ?? 0,
            _shufflePeriod: options?.shufflePeriod ?? 0.5,
            _noteJumpSpeed: options?.njs ?? 0,
            _noteJumpStartBeatOffset: options?.njsOffset ?? 0,
            _notes: [
                ...data.difficulty.colorNotes.map((x) => {
                    return colorNote_js_1.colorNote.serialize(x);
                }),
                ...data.difficulty.bombNotes.map((x) => {
                    return bombNote_js_1.bombNote.serialize(x);
                }),
            ],
            _obstacles: data.difficulty.obstacles.map((x) => {
                return obstacle_js_1.obstacle.serialize(x);
            }),
            _events: [
                ...data.lightshow.basicEvents.map((x) => {
                    return basicEvent_js_1.basicEvent.serialize(x);
                }),
                ...data.lightshow.colorBoostEvents.map((x) => {
                    return colorBoostEvent_js_1.colorBoostEvent.serialize(x);
                }),
                ...data.difficulty.rotationEvents.map((x) => {
                    return rotationEvent_js_1.rotationEvent.serialize(x);
                }),
                ...data.difficulty.bpmEvents.map((x) => {
                    return bpmEvent_js_1.bpmEvent.serialize(x);
                }),
            ],
            _time: data.difficulty.customData._time,
            _BPMChanges: data.difficulty.customData._bpmChanges,
            _bookmarks: data.difficulty.customData._bookmarks,
        };
    },
    deserialize(data, options) {
        const colorNotes = [];
        const bombNotes = [];
        const _notes = data._notes || [];
        for (let i = 0; i < _notes.length; i++) {
            const obj = _notes[i];
            if (obj?._type === 3) {
                bombNotes.push(bombNote_js_1.bombNote.deserialize(obj));
            }
            else {
                colorNotes.push(colorNote_js_1.colorNote.deserialize(obj));
            }
        }
        const basicEvents = [];
        const colorBoostEvents = [];
        const rotationEvents = [];
        const bpmEvents = [];
        const _events = data._events || [];
        for (let i = 0; i < _events.length; i++) {
            const obj = _events[i];
            switch (obj?._type) {
                case 5:
                    colorBoostEvents.push(colorBoostEvent_js_1.colorBoostEvent.deserialize(obj));
                    break;
                case 14:
                case 15:
                    rotationEvents.push(rotationEvent_js_1.rotationEvent.deserialize(obj));
                    break;
                case 100:
                case 10:
                    bpmEvents.push(bpmEvent_js_1.bpmEvent.deserialize(obj));
                    break;
                default:
                    basicEvents.push(basicEvent_js_1.basicEvent.deserialize(obj));
            }
        }
        return (0, beatmap_js_1.createBeatmap)({
            version: 1,
            filename: options?.filename ?? 'EasyStandard.dat',
            lightshowFilename: options?.lightshowFilename ?? 'EasyLightshow.dat',
            difficulty: (0, difficulty_js_1.createDifficulty)({
                colorNotes,
                bombNotes,
                obstacles: data._obstacles?.map((x) => obstacle_js_1.obstacle.deserialize(x)),
                rotationEvents,
                bpmEvents,
                customData: {
                    _bpmChanges: data._BPMChanges,
                    _bookmarks: data._bookmarks,
                    _time: data._time,
                },
            }),
            lightshow: (0, lightshow_js_1.createLightshow)({
                basicEvents,
                colorBoostEvents,
                useNormalEventsAsCompatibleEvents: true,
            }),
        });
    },
};
