"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.info = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const info_js_1 = require("../../core/info.js");
const environment_js_1 = require("../../helpers/environment.js");
const infoBeatmap_js_1 = require("./infoBeatmap.js");
/**
 * Schema serialization for v1 `Info`.
 */
exports.info = {
    serialize(data) {
        return {
            songName: data.song.title,
            songSubName: data.song.subTitle,
            authorName: data.song.author,
            beatsPerMinute: data.audio.bpm,
            previewStartTime: data.audio.previewStartTime,
            previewDuration: data.audio.previewDuration,
            coverImagePath: data.coverImageFilename,
            environmentName: data.environmentBase.normal ||
                data.environmentNames.find((e) => !(0, environment_js_1.is360Environment)(e)) ||
                'DefaultEnvironment',
            difficultyLevels: data.difficulties.map((x) => {
                return infoBeatmap_js_1.infoBeatmap.serialize(x);
            }),
            oneSaber: data.difficulties.some((m) => m.characteristic === 'OneSaber'),
            contributors: (0, json_js_1.deepCopy)(data.customData._contributors),
            customEnvironment: data.customData._customEnvironment,
            customEnvironmentHash: data.customData._customEnvironmentHash,
        };
    },
    deserialize(data, options) {
        const difficulty = data.difficultyLevels?.find((e) => {
            return e?.audioPath;
        });
        return (0, info_js_1.createInfo)({
            version: 1,
            filename: options?.filename,
            song: {
                title: data.songName,
                subTitle: data.songSubName,
                author: data.authorName,
            },
            audio: {
                filename: difficulty?.audioPath ?? options?.audio?.filename,
                audioDataFilename: options?.audio?.audioDataFilename,
                bpm: data.beatsPerMinute,
                lufs: options?.audio?.lufs,
                duration: options?.audio?.duration,
                previewStartTime: data.previewStartTime,
                previewDuration: data.previewDuration,
            },
            songPreviewFilename: difficulty?.audioPath ?? options?.audio?.filename,
            coverImageFilename: data.coverImagePath,
            environmentBase: { normal: data.environmentName },
            difficulties: data.difficultyLevels?.map((x) => {
                return infoBeatmap_js_1.infoBeatmap.deserialize(x);
            }) ?? [],
            customData: {
                _contributors: data.contributors,
                _customEnvironment: data.customEnvironment,
                _customEnvironmentHash: data.customEnvironmentHash,
            },
        });
    },
};
