"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.infoBeatmap = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const infoBeatmap_js_1 = require("../../core/infoBeatmap.js");
const difficulty_js_1 = require("../../shared/difficulty.js");
/**
 * Schema serialization for v1 `Info Beatmap`.
 */
exports.infoBeatmap = {
    serialize(data, options) {
        return {
            difficulty: data.difficulty,
            difficultyRank: difficulty_js_1.DifficultyRanking[data.difficulty],
            audioPath: options?.audio?.filename ?? 'song.ogg',
            jsonPath: data.filename,
            characteristic: data.characteristic,
            offset: data.customData._editorOffset,
            oldOffset: data.customData._editorOldOffset,
            chromaToggle: data.customData._chromaToggle,
            customColors: data.customData._customColors,
            difficultyLabel: data.customData._difficultyLabel,
            colorLeft: (0, json_js_1.shallowCopy)(data.customData._colorLeft),
            colorRight: (0, json_js_1.shallowCopy)(data.customData._colorRight),
            envColorLeft: (0, json_js_1.shallowCopy)(data.customData._envColorLeft),
            envColorRight: (0, json_js_1.shallowCopy)(data.customData._envColorRight),
            obstacleColor: (0, json_js_1.shallowCopy)(data.customData._obstacleColor),
        };
    },
    deserialize(data, options) {
        return (0, infoBeatmap_js_1.createInfoBeatmap)({
            characteristic: data.characteristic,
            difficulty: data.difficulty,
            authors: {
                mappers: options?.authors?.mappers,
                lighters: options?.authors?.lighters,
            },
            filename: data.jsonPath,
            lightshowFilename: options?.lightshowFilename,
            njs: options?.njs,
            njsOffset: options?.njsOffset,
            customData: {
                _editorOffset: data.offset,
                _editorOldOffset: data.oldOffset,
                _chromaToggle: data.chromaToggle,
                _customColors: data.customColors,
                _difficultyLabel: data.difficultyLabel,
                _colorLeft: (0, json_js_1.shallowCopy)(data.colorLeft),
                _colorRight: (0, json_js_1.shallowCopy)(data.colorRight),
                _envColorLeft: (0, json_js_1.shallowCopy)(data.envColorLeft),
                _envColorRight: (0, json_js_1.shallowCopy)(data.envColorRight),
                _obstacleColor: (0, json_js_1.shallowCopy)(data.obstacleColor),
            },
        });
    },
};
