"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.difficulty = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const sort_js_1 = require("../../helpers/sort.js");
const version_js_1 = require("../../helpers/version.js");
const arc_js_1 = require("./arc.js");
const basicEvent_js_1 = require("./basicEvent.js");
const basicEventTypesWithKeywords_js_1 = require("./basicEventTypesWithKeywords.js");
const bombNote_js_1 = require("./bombNote.js");
const bpmEvent_js_1 = require("./bpmEvent.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const colorNote_js_1 = require("./colorNote.js");
const obstacle_js_1 = require("./obstacle.js");
const rotationEvent_js_1 = require("./rotationEvent.js");
const waypoint_js_1 = require("./waypoint.js");
/**
 * Schema serialization for v2 `Difficulty`.
 */
exports.difficulty = {
    serialize(data) {
        return {
            _version: '2.6.0',
            _notes: [
                ...data.difficulty.colorNotes.map((x) => {
                    return colorNote_js_1.colorNote.serialize(x);
                }),
                ...data.difficulty.bombNotes.map((x) => {
                    return bombNote_js_1.bombNote.serialize(x);
                }),
            ].sort(sort_js_1.sortV2NoteFn),
            _sliders: data.difficulty.arcs.map((x) => {
                return arc_js_1.arc.serialize(x);
            }),
            _obstacles: data.difficulty.obstacles.map((x) => {
                return obstacle_js_1.obstacle.serialize(x);
            }),
            _events: [
                ...data.lightshow.basicEvents.map((x) => {
                    return basicEvent_js_1.basicEvent.serialize(x);
                }),
                ...data.lightshow.colorBoostEvents.map((x) => {
                    return colorBoostEvent_js_1.colorBoostEvent.serialize(x);
                }),
                ...data.difficulty.rotationEvents.map((x) => {
                    return rotationEvent_js_1.rotationEvent.serialize(x);
                }),
                ...data.difficulty.bpmEvents.map((x) => {
                    return bpmEvent_js_1.bpmEvent.serialize(x);
                }),
            ].sort(sort_js_1.sortV2ObjectFn),
            _waypoints: data.lightshow.waypoints.map((x) => {
                return waypoint_js_1.waypoint.serialize(x);
            }),
            _specialEventsKeywordFilters: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.serialize(data.lightshow.basicEventTypesWithKeywords),
            _customData: (0, json_js_1.deepCopy)(data.difficulty.customData),
        };
    },
    deserialize(data, options) {
        const colorNotes = [];
        const bombNotes = [];
        const _notes = data._notes || [];
        for (let i = 0; i < _notes.length; i++) {
            const obj = _notes[i];
            if (obj?._type === 3) {
                bombNotes.push(bombNote_js_1.bombNote.deserialize(obj));
            }
            else {
                colorNotes.push(colorNote_js_1.colorNote.deserialize(obj));
            }
        }
        const preV25 = (0, version_js_1.compareVersion)(data._version || '2.0.0', '2.5.0');
        const basicEvents = [];
        const colorBoostEvents = [];
        const rotationEvents = [];
        const bpmEvents = [];
        const _events = data._events || [];
        for (let i = 0; i < _events.length; i++) {
            const obj = _events[i];
            switch (obj?._type) {
                case 5:
                    colorBoostEvents.push(colorBoostEvent_js_1.colorBoostEvent.deserialize(obj));
                    break;
                case 14:
                case 15:
                    rotationEvents.push(rotationEvent_js_1.rotationEvent.deserialize(obj));
                    break;
                case 100:
                    bpmEvents.push(bpmEvent_js_1.bpmEvent.deserialize(obj));
                    break;
                default: {
                    const evt = basicEvent_js_1.basicEvent.deserialize(obj);
                    if (preV25 < 0) {
                        if (obj._type === 10) {
                            bpmEvents.push(bpmEvent_js_1.bpmEvent.deserialize(obj));
                        }
                        else {
                            evt.floatValue = 1;
                            basicEvents.push(evt);
                        }
                    }
                    else {
                        basicEvents.push(evt);
                    }
                }
            }
        }
        return (0, beatmap_js_1.createBeatmap)({
            version: 2,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: (0, difficulty_js_1.createDifficulty)({
                colorNotes,
                bombNotes,
                obstacles: data._obstacles?.map((x) => {
                    return obstacle_js_1.obstacle.deserialize(x);
                }),
                rotationEvents,
                bpmEvents,
                customData: data._customData,
            }),
            lightshow: (0, lightshow_js_1.createLightshow)({
                waypoints: data._waypoints?.map((x) => {
                    return waypoint_js_1.waypoint.deserialize(x);
                }),
                basicEvents,
                colorBoostEvents,
                useNormalEventsAsCompatibleEvents: true,
                basicEventTypesWithKeywords: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.deserialize(data._specialEventsKeywordFilters ?? {}),
            }),
        });
    },
};
