"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeDifficulty = optimizeDifficulty;
const helpers_js_1 = require("../../../../utils/math/helpers.js");
const optimize_js_1 = require("../../../helpers/optimize.js");
const json_js_1 = require("../../../../utils/misc/json.js");
/**
 * Optimize v2 `Difficulty` schema data.
 */
function optimizeDifficulty(
// deno-lint-ignore no-explicit-any
data, options) {
    for (let i1 = 0; i1 < data._notes.length; i1++) {
        const o1 = data._notes[i1];
        if (options.floatTrim) {
            o1._time = (0, helpers_js_1.round)(o1._time, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1._customData, `difficulty._notes[${i1}]._customData`, options);
        if ((0, json_js_1.isEmpty)(o1._customData)) {
            delete o1._customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < data._obstacles.length; i1++) {
        const o1 = data._obstacles[i1];
        if (options.floatTrim) {
            o1._time = (0, helpers_js_1.round)(o1._time, options.floatTrim);
            o1._duration = (0, helpers_js_1.round)(o1._duration, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1._customData, `difficulty._obstacles[${i1}]._customData`, options);
        if ((0, json_js_1.isEmpty)(o1._customData)) {
            delete o1._customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < data._sliders.length; i1++) {
        const o1 = data._sliders[i1];
        if (options.floatTrim) {
            o1._headTime = (0, helpers_js_1.round)(o1._headTime, options.floatTrim);
            o1._tailTime = (0, helpers_js_1.round)(o1._tailTime, options.floatTrim);
            o1._headControlPointLengthMultiplier = (0, helpers_js_1.round)(o1._headControlPointLengthMultiplier, options.floatTrim);
            o1._tailControlPointLengthMultiplier = (0, helpers_js_1.round)(o1._tailControlPointLengthMultiplier, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1._customData, `difficulty._sliders[${i1}]._customData`, options);
        if ((0, json_js_1.isEmpty)(o1._customData)) {
            delete o1._customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < data._waypoints.length; i1++) {
        const o1 = data._waypoints[i1];
        if (options.floatTrim) {
            o1._time = (0, helpers_js_1.round)(o1._time, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1._customData, `difficulty._waypoints[${i1}]._customData`, options);
        if ((0, json_js_1.isEmpty)(o1._customData)) {
            delete o1._customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < data._events.length; i1++) {
        const o1 = data._events[i1];
        if (options.floatTrim && o1._type !== 100) {
            o1._time = (0, helpers_js_1.round)(o1._time, options.floatTrim);
            o1._floatValue = (0, helpers_js_1.round)(o1._floatValue, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1._customData, `difficulty._events[${i1}]._customData`, options);
        if ((0, json_js_1.isEmpty)(o1._customData)) {
            delete o1._customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    (0, optimize_js_1.deepClean)(data._customData, 'difficulty._customData', options);
    if ((0, json_js_1.isEmpty)(data._customData)) {
        delete data._customData;
    }
}
