"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeInfo = optimizeInfo;
const helpers_js_1 = require("../../../../utils/math/helpers.js");
const optimize_js_1 = require("../../../helpers/optimize.js");
const json_js_1 = require("../../../../utils/misc/json.js");
/**
 * Optimize v2 `Info` schema data.
 */
function optimizeInfo(
// deno-lint-ignore no-explicit-any
data, options) {
    if (options.floatTrim) {
        data._beatsPerMinute = (0, helpers_js_1.round)(data._beatsPerMinute, options.floatTrim);
        data._previewDuration = (0, helpers_js_1.round)(data._previewDuration, options.floatTrim);
        data._previewStartTime = (0, helpers_js_1.round)(data._previewStartTime, options.floatTrim);
        data._shuffle = (0, helpers_js_1.round)(data._shuffle, options.floatTrim);
        data._shufflePeriod = (0, helpers_js_1.round)(data._shufflePeriod, options.floatTrim);
        data._songTimeOffset = (0, helpers_js_1.round)(data._songTimeOffset, options.floatTrim);
    }
    if (options.stringTrim) {
        data._levelAuthorName = data._levelAuthorName.trim();
        data._songAuthorName = data._songAuthorName.trim();
        data._songName = data._songName.trim();
        data._songSubName = data._songSubName.trim();
    }
    for (let it = 0; it < data._colorSchemes.length; it++) {
        const cs = data._colorSchemes[it];
        (0, optimize_js_1.deepClean)(cs.colorScheme, `info._colorSchemes[${it}].colorScheme`, options);
    }
    for (let i1 = 0; i1 < data._difficultyBeatmapSets.length; i1++) {
        const set = data._difficultyBeatmapSets[i1];
        for (let i2 = 0; i2 < set._difficultyBeatmaps.length; i2++) {
            const diff = set._difficultyBeatmaps[i2];
            if (options.floatTrim) {
                diff._noteJumpMovementSpeed = (0, helpers_js_1.round)(diff._noteJumpMovementSpeed, options.floatTrim);
                diff._noteJumpStartBeatOffset = (0, helpers_js_1.round)(diff._noteJumpStartBeatOffset, options.floatTrim);
            }
            (0, optimize_js_1.deepClean)(diff._customData, `info._difficultyBeatmapSets[${i1}]._difficultyBeatmaps[${i2}]._customData`, options);
            if ((0, json_js_1.isEmpty)(diff._customData)) {
                delete diff._customData;
            }
        }
        if (set._customData) {
            (0, optimize_js_1.deepClean)(set._customData, `info._difficultyBeatmapSets[${i1}]._customData`, options);
            if ((0, json_js_1.isEmpty)(set._customData)) {
                delete set._customData;
            }
        }
    }
    (0, optimize_js_1.deepClean)(data._customData, 'info._customData', options);
    if ((0, json_js_1.isEmpty)(data._customData)) {
        delete data._customData;
    }
}
