"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rotationEvent = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const rotationEvent_js_1 = require("../../core/rotationEvent.js");
const constants_js_1 = require("../../shared/constants.js");
/**
 * Schema serialization for v2 `Rotation Event`.
 */
exports.rotationEvent = {
    serialize(data) {
        let r = data.rotation % 360;
        const customData = (0, json_js_1.deepCopy)(data.customData);
        if (r >= -60 && r <= 60 && r % 15 === 0 && r / 15 !== 0) {
            r = constants_js_1.RotationValueEventValue[r] || r + 1360;
        }
        else {
            customData._rotation = r;
            r += 1360;
        }
        return {
            _time: data.time,
            _type: data.executionTime === 1 ? 15 : 14,
            _value: r,
            _floatValue: 0,
            _customData: customData,
        };
    },
    deserialize(data) {
        const value = data._value ?? 0;
        return (0, rotationEvent_js_1.createRotationEvent)({
            time: data._time,
            executionTime: data._type === 15 ? 1 : 0,
            rotation: typeof data._customData?._rotation === 'number'
                ? data._customData._rotation
                : value >= 1000
                    ? (value - 1360) % 360
                    : constants_js_1.EventLaneRotationValue[value],
            customData: data._customData,
        });
    },
};
