"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.difficulty = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const constants_js_1 = require("../../shared/constants.js");
const arc_js_1 = require("./arc.js");
const basicEvent_js_1 = require("./basicEvent.js");
const basicEventTypesWithKeywords_js_1 = require("./basicEventTypesWithKeywords.js");
const bombNote_js_1 = require("./bombNote.js");
const bpmEvent_js_1 = require("./bpmEvent.js");
const chain_js_1 = require("./chain.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const colorNote_js_1 = require("./colorNote.js");
const fxEventBoxGroup_js_1 = require("./fxEventBoxGroup.js");
const lightColorEventBoxGroup_js_1 = require("./lightColorEventBoxGroup.js");
const lightRotationEventBoxGroup_js_1 = require("./lightRotationEventBoxGroup.js");
const lightTranslationEventBoxGroup_js_1 = require("./lightTranslationEventBoxGroup.js");
const obstacle_js_1 = require("./obstacle.js");
const rotationEvent_js_1 = require("./rotationEvent.js");
const waypoint_js_1 = require("./waypoint.js");
/**
 * Schema serialization for v3 `Difficulty`.
 */
exports.difficulty = {
    serialize(data) {
        const json = {
            version: '3.3.0',
            bpmEvents: data.difficulty.bpmEvents.map((x) => {
                return bpmEvent_js_1.bpmEvent.serialize(x);
            }),
            rotationEvents: data.difficulty.rotationEvents.map((x) => {
                return rotationEvent_js_1.rotationEvent.serialize(x);
            }),
            colorNotes: data.difficulty.colorNotes.map((x) => {
                return colorNote_js_1.colorNote.serialize(x);
            }),
            bombNotes: data.difficulty.bombNotes.map((x) => {
                return bombNote_js_1.bombNote.serialize(x);
            }),
            obstacles: data.difficulty.obstacles.map((x) => {
                return obstacle_js_1.obstacle.serialize(x);
            }),
            sliders: data.difficulty.arcs.map((x) => {
                return arc_js_1.arc.serialize(x);
            }),
            burstSliders: data.difficulty.chains.map((x) => {
                return chain_js_1.chain.serialize(x);
            }),
            waypoints: data.lightshow.waypoints.map((x) => {
                return waypoint_js_1.waypoint.serialize(x);
            }),
            basicBeatmapEvents: data.lightshow.basicEvents.map((x) => {
                return basicEvent_js_1.basicEvent.serialize(x);
            }),
            colorBoostBeatmapEvents: data.lightshow.colorBoostEvents.map((x) => {
                return colorBoostEvent_js_1.colorBoostEvent.serialize(x);
            }),
            lightColorEventBoxGroups: data.lightshow.lightColorEventBoxGroups.map((x) => {
                return lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.serialize(x);
            }),
            lightRotationEventBoxGroups: data.lightshow.lightRotationEventBoxGroups.map((x) => {
                return lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.serialize(x);
            }),
            lightTranslationEventBoxGroups: data.lightshow.lightTranslationEventBoxGroups.map((x) => {
                return lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.serialize(x);
            }),
            vfxEventBoxGroups: [],
            basicEventTypesWithKeywords: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.serialize(data.lightshow.basicEventTypesWithKeywords),
            _fxEventsCollection: {
                _fl: [],
                _il: [],
            },
            useNormalEventsAsCompatibleEvents: data.lightshow.useNormalEventsAsCompatibleEvents,
            customData: (0, json_js_1.deepCopy)(data.difficulty.customData),
        };
        for (const obj of data.lightshow.fxEventBoxGroups.map((x) => {
            return fxEventBoxGroup_js_1.fxEventBoxGroup.serialize(x);
        })) {
            json.vfxEventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                obj.object.e.push(box.data);
                for (const evt of box.eventData) {
                    box.data.l.push(json._fxEventsCollection._fl.length);
                    json._fxEventsCollection._fl.push(evt);
                }
            }
        }
        return json;
    },
    deserialize(data, options) {
        const fx = data._fxEventsCollection?._fl;
        return (0, beatmap_js_1.createBeatmap)({
            version: 3,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: (0, difficulty_js_1.createDifficulty)({
                colorNotes: data.colorNotes?.map((x) => {
                    return colorNote_js_1.colorNote.deserialize(x);
                }),
                bombNotes: data.bombNotes?.map((x) => {
                    return bombNote_js_1.bombNote.deserialize(x);
                }),
                obstacles: data.obstacles?.map((x) => {
                    return obstacle_js_1.obstacle.deserialize(x);
                }),
                arcs: data.sliders?.map((x) => {
                    return arc_js_1.arc.deserialize(x);
                }),
                chains: data.burstSliders?.map((x) => {
                    return chain_js_1.chain.deserialize(x);
                }),
                rotationEvents: data.rotationEvents?.map((x) => {
                    return rotationEvent_js_1.rotationEvent.deserialize(x);
                }),
                bpmEvents: data.bpmEvents?.map((x) => {
                    return bpmEvent_js_1.bpmEvent.deserialize(x);
                }),
                customData: data.customData,
            }),
            lightshow: (0, lightshow_js_1.createLightshow)({
                waypoints: data.waypoints?.map((x) => {
                    return waypoint_js_1.waypoint.deserialize(x);
                }),
                basicEvents: data.basicBeatmapEvents?.map((x) => {
                    return basicEvent_js_1.basicEvent.deserialize(x);
                }),
                colorBoostEvents: data.colorBoostBeatmapEvents?.map((x) => {
                    return colorBoostEvent_js_1.colorBoostEvent.deserialize(x);
                }),
                lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map((x) => {
                    return lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.deserialize(x);
                }),
                lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map((x) => {
                    return lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.deserialize(x);
                }),
                lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map((x) => {
                    return lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.deserialize(x);
                }),
                fxEventBoxGroups: data.vfxEventBoxGroups?.map((obj) => fxEventBoxGroup_js_1.fxEventBoxGroup.deserialize({
                    object: { ...obj, t: constants_js_1.FxType.FLOAT },
                    boxData: obj.e?.map((box) => ({
                        data: box,
                        eventData: box.l?.map((idx) => fx[idx]) ?? [],
                    })) ?? [],
                })),
                basicEventTypesWithKeywords: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.deserialize(data.basicEventTypesWithKeywords ?? {}),
                useNormalEventsAsCompatibleEvents: data.useNormalEventsAsCompatibleEvents,
            }),
        });
    },
};
