"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightshow = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const constants_js_1 = require("../../shared/constants.js");
const basicEvent_js_1 = require("./basicEvent.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const fxEventBoxGroup_js_1 = require("./fxEventBoxGroup.js");
const lightColorEventBoxGroup_js_1 = require("./lightColorEventBoxGroup.js");
const lightRotationEventBoxGroup_js_1 = require("./lightRotationEventBoxGroup.js");
const lightTranslationEventBoxGroup_js_1 = require("./lightTranslationEventBoxGroup.js");
/**
 * Schema serialization for v3 `Lightshow`.
 */
exports.lightshow = {
    serialize(data) {
        const json = {
            basicBeatmapEvents: data.lightshow.basicEvents.map((x) => {
                return basicEvent_js_1.basicEvent.serialize(x);
            }),
            colorBoostBeatmapEvents: data.lightshow.colorBoostEvents.map((x) => {
                return colorBoostEvent_js_1.colorBoostEvent.serialize(x);
            }),
            lightColorEventBoxGroups: data.lightshow.lightColorEventBoxGroups.map((x) => {
                return lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.serialize(x);
            }),
            lightRotationEventBoxGroups: data.lightshow.lightRotationEventBoxGroups.map((x) => {
                return lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.serialize(x);
            }),
            lightTranslationEventBoxGroups: data.lightshow.lightTranslationEventBoxGroups.map((x) => {
                return lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.serialize(x);
            }),
            vfxEventBoxGroups: [],
            _fxEventsCollection: {
                _fl: [],
                _il: [],
            },
            customData: (0, json_js_1.deepCopy)(data.lightshow.customData),
        };
        for (const obj of data.lightshow.fxEventBoxGroups.map((x) => {
            return fxEventBoxGroup_js_1.fxEventBoxGroup.serialize(x);
        })) {
            json.vfxEventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                obj.object.e.push(box.data);
                for (const evt of box.eventData) {
                    box.data.l.push(json._fxEventsCollection._fl.length);
                    json._fxEventsCollection._fl.push(evt);
                }
            }
        }
        return json;
    },
    deserialize(data, options) {
        const fx = data._fxEventsCollection?._fl;
        return (0, beatmap_js_1.createBeatmap)({
            version: 3,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: (0, difficulty_js_1.createDifficulty)(),
            lightshow: (0, lightshow_js_1.createLightshow)({
                basicEvents: data.basicBeatmapEvents?.map((x) => {
                    return basicEvent_js_1.basicEvent.deserialize(x);
                }),
                colorBoostEvents: data.colorBoostBeatmapEvents?.map((x) => {
                    return colorBoostEvent_js_1.colorBoostEvent.deserialize(x);
                }),
                lightColorEventBoxGroups: data.lightColorEventBoxGroups?.map((x) => {
                    return lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.deserialize(x);
                }),
                lightRotationEventBoxGroups: data.lightRotationEventBoxGroups?.map((x) => {
                    return lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.deserialize(x);
                }),
                lightTranslationEventBoxGroups: data.lightTranslationEventBoxGroups?.map((x) => {
                    return lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.deserialize(x);
                }),
                fxEventBoxGroups: data.vfxEventBoxGroups?.map((obj) => fxEventBoxGroup_js_1.fxEventBoxGroup.deserialize({
                    object: { ...obj, t: constants_js_1.FxType.FLOAT },
                    boxData: obj.e?.map((box) => ({
                        data: box,
                        eventData: box.l?.map((idx) => fx[idx]) ?? [],
                    })) ?? [],
                })),
                customData: data.customData,
            }),
        });
    },
};
