"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeLightshow = optimizeLightshow;
const helpers_js_1 = require("../../../../utils/math/helpers.js");
const optimize_js_1 = require("../../../helpers/optimize.js");
const json_js_1 = require("../../../../utils/misc/json.js");
/**
 * Optimize v3 `Lightshow` schema data.
 */
function optimizeLightshow(data, options) {
    // deno-lint-ignore no-explicit-any
    const d = data;
    for (let i1 = 0; i1 < d.basicBeatmapEvents.length; i1++) {
        const o1 = d.basicBeatmapEvents[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
            o1.f = (0, helpers_js_1.round)(o1.f, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.basicBeatmapEvents[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d.colorBoostBeatmapEvents.length; i1++) {
        const o1 = d.colorBoostBeatmapEvents[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.colorBoostBeatmapEvents[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (!o1.o)
            delete o1.o;
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d.lightColorEventBoxGroups.length; i1++) {
        const o1 = d.lightColorEventBoxGroups[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        for (let i2 = 0; i2 < o1.e.length; i2++) {
            const o2 = o1.e[i2];
            if (options.floatTrim) {
                o2.r = (0, helpers_js_1.round)(o2.r, options.floatTrim);
                o2.w = (0, helpers_js_1.round)(o2.w, options.floatTrim);
                o2.f.l = (0, helpers_js_1.round)(o2.f.l, options.floatTrim);
            }
            for (let i3 = 0; i3 < o2.e.length; i3++) {
                const o3 = o2.e[i3];
                if (options.floatTrim) {
                    o3.b = (0, helpers_js_1.round)(o3.b, options.floatTrim);
                    o3.s = (0, helpers_js_1.round)(o3.s, options.floatTrim);
                    o3.sb = (0, helpers_js_1.round)(o3.sb, options.floatTrim);
                }
                (0, optimize_js_1.deepClean)(o3.customData, `lightshow.lightColorEventBoxGroups[${i1}].e[${i2}].e[${i3}].customData`, options);
                if ((0, json_js_1.isEmpty)(o3.customData)) {
                    delete o3.customData;
                }
                if (options.purgeZeros)
                    (0, optimize_js_1.purgeZeros)(o3);
            }
            (0, optimize_js_1.deepClean)(o2.f.customData, `lightshow.lightRotationEventBoxGroups[${i2}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.f.customData)) {
                delete o2.f.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2.f);
            (0, optimize_js_1.deepClean)(o2.customData, `lightshow.lightColorEventBoxGroups[${i1}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.customData)) {
                delete o2.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.lightColorEventBoxGroups[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d.lightRotationEventBoxGroups.length; i1++) {
        const o1 = d.lightRotationEventBoxGroups[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        for (let i2 = 0; i2 < o1.e.length; i2++) {
            const o2 = o1.e[i2];
            if (options.floatTrim) {
                o2.s = (0, helpers_js_1.round)(o2.s, options.floatTrim);
                o2.w = (0, helpers_js_1.round)(o2.w, options.floatTrim);
                o2.f.l = (0, helpers_js_1.round)(o2.f.l, options.floatTrim);
            }
            for (let i3 = 0; i3 < o2.l.length; i3++) {
                const o3 = o2.l[i3];
                if (options.floatTrim) {
                    o3.b = (0, helpers_js_1.round)(o3.b, options.floatTrim);
                    o3.r = (0, helpers_js_1.round)(o3.r, options.floatTrim);
                }
                (0, optimize_js_1.deepClean)(o3.customData, `lightshow.lightRotationEventBoxGroups[${i3}].e[${i2}].l[${i3}].customData`, options);
                if ((0, json_js_1.isEmpty)(o3.customData)) {
                    delete o3.customData;
                }
                if (options.purgeZeros)
                    (0, optimize_js_1.purgeZeros)(o3);
            }
            (0, optimize_js_1.deepClean)(o2.f.customData, `lightshow.lightRotationEventBoxGroups[${i2}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.f.customData)) {
                delete o2.f.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2.f);
            (0, optimize_js_1.deepClean)(o2.customData, `lightshow.lightRotationEventBoxGroups[${i2}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.customData)) {
                delete o2.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.lightRotationEventBoxGroups[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d.lightTranslationEventBoxGroups.length; i1++) {
        const o1 = d.lightTranslationEventBoxGroups[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        for (let i2 = 0; i2 < o1.e.length; i2++) {
            const o2 = o1.e[i2];
            if (options.floatTrim) {
                o2.s = (0, helpers_js_1.round)(o2.s, options.floatTrim);
                o2.w = (0, helpers_js_1.round)(o2.w, options.floatTrim);
                o2.f.l = (0, helpers_js_1.round)(o2.f.l, options.floatTrim);
            }
            for (let i3 = 0; i3 < o2.l.length; i3++) {
                const o3 = o2.l[i3];
                if (options.floatTrim) {
                    o3.b = (0, helpers_js_1.round)(o3.b, options.floatTrim);
                    o3.t = (0, helpers_js_1.round)(o3.t, options.floatTrim);
                }
                (0, optimize_js_1.deepClean)(o3.customData, `lightshow.lightTranslationEventBoxGroups[${i1}].e[${i2}].l[${i3}].customData`, options);
                if ((0, json_js_1.isEmpty)(o3.customData)) {
                    delete o3.customData;
                }
                if (options.purgeZeros)
                    (0, optimize_js_1.purgeZeros)(o3);
            }
            (0, optimize_js_1.deepClean)(o2.f.customData, `lightshow.lightRotationEventBoxGroups[${i2}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.f.customData)) {
                delete o2.f.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2.f);
            (0, optimize_js_1.deepClean)(o2.customData, `lightshow.lightTranslationEventBoxGroups[${i1}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.customData)) {
                delete o2.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.lightTranslationEventBoxGroups[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d.vfxEventBoxGroups.length; i1++) {
        const o1 = d.vfxEventBoxGroups[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        for (let i2 = 0; i2 < o1.e.length; i2++) {
            const o2 = o1.e[i2];
            if (options.floatTrim) {
                o2.s = (0, helpers_js_1.round)(o2.s, options.floatTrim);
                o2.w = (0, helpers_js_1.round)(o2.w, options.floatTrim);
                o2.f.l = (0, helpers_js_1.round)(o2.f.l, options.floatTrim);
            }
            (0, optimize_js_1.deepClean)(o2.f.customData, `lightshow.lightRotationEventBoxGroups[${i2}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.f.customData)) {
                delete o2.f.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2.f);
            (0, optimize_js_1.deepClean)(o2.customData, `lightshow.vfxEventBoxGroups[${i1}].e[${i2}].customData`, options);
            if ((0, json_js_1.isEmpty)(o2.customData)) {
                delete o2.customData;
            }
            if (options.purgeZeros)
                (0, optimize_js_1.purgeZeros)(o2);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow.vfxEventBoxGroups[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d._fxEventsCollection._fl.length; i1++) {
        const o1 = d._fxEventsCollection._fl[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
            o1.v = (0, helpers_js_1.round)(o1.v, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow._fxEventsCollection._fl[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    for (let i1 = 0; i1 < d._fxEventsCollection._il.length; i1++) {
        const o1 = d._fxEventsCollection._il[i1];
        if (options.floatTrim) {
            o1.b = (0, helpers_js_1.round)(o1.b, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o1.customData, `lightshow._fxEventsCollection._il[${i1}].customData`, options);
        if ((0, json_js_1.isEmpty)(o1.customData)) {
            delete o1.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o1);
    }
    (0, optimize_js_1.deepClean)(d.customData, `lightshow.customData`, options);
    if ((0, json_js_1.isEmpty)(d.customData)) {
        delete d.customData;
    }
}
