"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.difficulty = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const arc_js_1 = require("./arc.js");
const bombNote_js_1 = require("./bombNote.js");
const chain_js_1 = require("./chain.js");
const colorNote_js_1 = require("./colorNote.js");
const njsEvent_js_1 = require("./njsEvent.js");
const obstacle_js_1 = require("./obstacle.js");
const rotationEvent_js_1 = require("./rotationEvent.js");
/**
 * Schema serialization for v4 `Difficulty`.
 */
exports.difficulty = {
    serialize(data) {
        const json = {
            version: '4.1.0',
            colorNotes: [],
            bombNotes: [],
            obstacles: [],
            chains: [],
            arcs: [],
            colorNotesData: [],
            bombNotesData: [],
            obstaclesData: [],
            chainsData: [],
            arcsData: [],
            njsEvents: [],
            njsEventData: [],
            customData: (0, json_js_1.deepCopy)(data.difficulty.customData),
        };
        for (const jsonObj of data.difficulty.colorNotes.map((x) => {
            return colorNote_js_1.colorNote.serialize(x);
        })) {
            json.colorNotes.push(jsonObj.object);
            jsonObj.object.i = json.colorNotesData.length;
            json.colorNotesData.push(jsonObj.data);
        }
        for (const jsonObj of data.difficulty.bombNotes.map((x) => {
            return bombNote_js_1.bombNote.serialize(x);
        })) {
            json.bombNotes.push(jsonObj.object);
            jsonObj.object.i = json.bombNotesData.length;
            json.bombNotesData.push(jsonObj.data);
        }
        for (const jsonObj of data.difficulty.obstacles.map((x) => {
            return obstacle_js_1.obstacle.serialize(x);
        })) {
            json.obstacles.push(jsonObj.object);
            jsonObj.object.i = json.obstaclesData.length;
            json.obstaclesData.push(jsonObj.data);
        }
        for (const jsonObj of data.difficulty.arcs.map((x) => {
            return arc_js_1.arc.serialize(x);
        })) {
            json.arcs.push(jsonObj.object);
            jsonObj.object.ai = json.arcsData.length;
            json.arcsData.push(jsonObj.data);
            jsonObj.object.hi = json.colorNotesData.length;
            json.colorNotesData.push(jsonObj.headData);
            jsonObj.object.ti = json.colorNotesData.length;
            json.colorNotesData.push(jsonObj.tailData);
        }
        for (const jsonObj of data.difficulty.chains.map((x) => {
            return chain_js_1.chain.serialize(x);
        })) {
            json.chains.push(jsonObj.object);
            jsonObj.object.i = json.colorNotesData.length;
            json.colorNotesData.push(jsonObj.data);
            jsonObj.object.ci = json.chainsData.length;
            json.chainsData.push(jsonObj.chainData);
        }
        for (const jsonObj of data.difficulty.njsEvents.map((x) => {
            return njsEvent_js_1.njsEvent.serialize(x);
        })) {
            json.njsEvents.push(jsonObj.object);
            jsonObj.object.i = json.njsEventData.length;
            json.njsEventData.push(jsonObj.data);
        }
        return json;
    },
    deserialize(data, options) {
        return (0, beatmap_js_1.createBeatmap)({
            version: 4,
            filename: options?.filename,
            lightshowFilename: options?.lightshowFilename,
            difficulty: (0, difficulty_js_1.createDifficulty)({
                colorNotes: data.colorNotes?.map((obj) => {
                    return colorNote_js_1.colorNote.deserialize({
                        object: obj,
                        data: data.colorNotesData?.[obj?.i ?? 0] ?? {},
                    });
                }),
                bombNotes: data.bombNotes?.map((obj) => {
                    return bombNote_js_1.bombNote.deserialize({
                        object: obj,
                        data: data.bombNotesData?.[obj?.i ?? 0] ?? {},
                    });
                }),
                obstacles: data.obstacles?.map((obj) => {
                    return obstacle_js_1.obstacle.deserialize({
                        object: obj,
                        data: data.obstaclesData?.[obj?.i ?? 0] ?? {},
                    });
                }),
                arcs: data.arcs?.map((obj) => {
                    return arc_js_1.arc.deserialize({
                        object: obj,
                        data: data.arcsData?.[obj?.ai ?? 0] ?? {},
                        headData: data.colorNotesData?.[obj?.hi ?? 0] ?? {},
                        tailData: data.colorNotesData?.[obj?.ti ?? 0] ?? {},
                    });
                }),
                chains: data.chains?.map((obj) => {
                    return chain_js_1.chain.deserialize({
                        object: obj,
                        data: data.colorNotesData?.[obj?.i ?? 0] ?? {},
                        chainData: data.chainsData?.[obj?.ci ?? 0] ?? {},
                    });
                }),
                rotationEvents: data.spawnRotations?.map((obj) => {
                    return rotationEvent_js_1.rotationEvent.deserialize({
                        object: obj,
                        data: data.spawnRotationsData?.[obj?.i ?? 0] ?? {},
                    });
                }),
                njsEvents: data.njsEvents?.map((obj) => {
                    return njsEvent_js_1.njsEvent.deserialize({
                        object: obj,
                        data: data.njsEventData?.[obj?.i ?? 0] ?? {},
                    });
                }),
                customData: data.customData,
            }),
            lightshow: (0, lightshow_js_1.createLightshow)(),
        });
    },
};
