"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.info = void 0;
const convertor_js_1 = require("../../../utils/colors/convertor.js");
const json_js_1 = require("../../../utils/misc/json.js");
const info_js_1 = require("../../core/info.js");
const infoBeatmap_js_1 = require("./infoBeatmap.js");
/**
 * Schema serialization for v4 `Info`.
 */
exports.info = {
    serialize(data) {
        return {
            version: '4.0.1',
            song: {
                author: data.song.author,
                title: data.song.title,
                subTitle: data.song.subTitle,
            },
            audio: {
                songFilename: data.audio.filename,
                songDuration: data.audio.duration,
                audioDataFilename: data.audio.audioDataFilename,
                bpm: data.audio.bpm,
                lufs: data.audio.lufs,
                previewStartTime: data.audio.previewStartTime,
                previewDuration: data.audio.previewDuration,
            },
            songPreviewFilename: data.songPreviewFilename,
            coverImageFilename: data.coverImageFilename,
            environmentNames: data.environmentNames.map((e) => e),
            colorSchemes: data.colorSchemes.map((e) => {
                const cs = {
                    colorSchemeName: e.name,
                    overrideNotes: e.overrideNotes,
                    overrideLights: e.overrideLights,
                    saberAColor: (0, convertor_js_1.colorToHex)(e.saberLeftColor),
                    saberBColor: (0, convertor_js_1.colorToHex)(e.saberRightColor),
                    environmentColor0: (0, convertor_js_1.colorToHex)(e.environment0Color),
                    environmentColor1: (0, convertor_js_1.colorToHex)(e.environment1Color),
                    obstaclesColor: (0, convertor_js_1.colorToHex)(e.obstaclesColor),
                    environmentColor0Boost: (0, convertor_js_1.colorToHex)(e.environment0ColorBoost),
                    environmentColor1Boost: (0, convertor_js_1.colorToHex)(e.environment1ColorBoost),
                };
                if (e.environmentWColor) {
                    cs.environmentColorW = (0, convertor_js_1.colorToHex)(e.environmentWColor);
                }
                if (e.environmentWColorBoost) {
                    cs.environmentColorWBoost = (0, convertor_js_1.colorToHex)(e.environmentWColorBoost);
                }
                return cs;
            }),
            difficultyBeatmaps: data.difficulties.map((x) => {
                return infoBeatmap_js_1.infoBeatmap.serialize(x);
            }),
            customData: (0, json_js_1.deepCopy)(data.customData),
        };
    },
    deserialize(data, options) {
        return (0, info_js_1.createInfo)({
            version: 4,
            filename: options?.filename,
            song: {
                author: data.song?.author,
                title: data.song?.title,
                subTitle: data.song?.subTitle,
            },
            audio: {
                filename: data.audio?.songFilename,
                duration: data.audio?.songDuration,
                audioDataFilename: data.audio?.audioDataFilename,
                bpm: data.audio?.bpm,
                lufs: data.audio?.lufs,
                previewStartTime: data.audio?.previewStartTime,
                previewDuration: data.audio?.previewDuration,
            },
            songPreviewFilename: data.songPreviewFilename,
            coverImageFilename: data.coverImageFilename,
            environmentNames: data.environmentNames,
            colorSchemes: data.colorSchemes?.map((e) => {
                const scheme = {
                    name: e?.colorSchemeName ?? '',
                    overrideNotes: e?.overrideNotes ?? data.version === '4.0.0',
                    overrideLights: e?.overrideLights ?? data.version === '4.0.0',
                    saberLeftColor: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.saberAColor), true),
                    saberRightColor: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.saberBColor), true),
                    environment0Color: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColor0), true),
                    environment1Color: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColor1), true),
                    obstaclesColor: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.obstaclesColor), true),
                    environment0ColorBoost: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColor0Boost), true),
                    environment1ColorBoost: (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColor1Boost), true),
                };
                if (e.environmentColorW) {
                    scheme.environmentWColor = (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColorW), true);
                }
                if (e.environmentColorWBoost) {
                    scheme.environmentWColorBoost = (0, convertor_js_1.toColorObject)((0, convertor_js_1.hexToRgba)(e.environmentColorWBoost), true);
                }
                return scheme;
            }),
            difficulties: data.difficultyBeatmaps?.map((d) => {
                return infoBeatmap_js_1.infoBeatmap.deserialize(d);
            }),
            customData: data.customData,
        });
    },
};
