"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightRotationEventBox = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const lightRotationEventBox_js_1 = require("../../core/lightRotationEventBox.js");
const indexFilter_js_1 = require("./indexFilter.js");
const lightRotationEvent_js_1 = require("./lightRotationEvent.js");
/**
 * Schema serialization for v4 `Light Rotation Event Box`.
 */
exports.lightRotationEventBox = {
    serialize(data) {
        return {
            data: {
                w: data.beatDistribution,
                d: data.beatDistributionType,
                s: data.rotationDistribution,
                t: data.rotationDistributionType,
                b: data.affectFirst,
                e: data.easing,
                a: data.axis,
                f: data.flip,
                customData: (0, json_js_1.deepCopy)(data.customData),
            },
            eventData: data.events.map((x) => {
                return lightRotationEvent_js_1.lightRotationEvent.serialize(x);
            }),
            filterData: indexFilter_js_1.indexFilter.serialize(data.filter),
        };
    },
    deserialize(data) {
        return (0, lightRotationEventBox_js_1.createLightRotationEventBox)({
            filter: indexFilter_js_1.indexFilter.deserialize(data.filterData ?? {}),
            beatDistribution: data.data?.w,
            beatDistributionType: data.data?.d,
            rotationDistribution: data.data?.s,
            rotationDistributionType: data.data?.t,
            affectFirst: data.data?.b,
            easing: data.data?.e,
            axis: data.data?.a,
            flip: data.data?.f,
            events: data.eventData?.map((x) => {
                return lightRotationEvent_js_1.lightRotationEvent.deserialize(x);
            }),
            customData: data.data?.customData,
        });
    },
};
