"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightTranslationEventBox = void 0;
const json_js_1 = require("../../../utils/misc/json.js");
const lightTranslationEventBox_js_1 = require("../../core/lightTranslationEventBox.js");
const indexFilter_js_1 = require("./indexFilter.js");
const lightTranslationEvent_js_1 = require("./lightTranslationEvent.js");
/**
 * Schema serialization for v4 `Light Translation Event Box`.
 */
exports.lightTranslationEventBox = {
    serialize(data) {
        return {
            data: {
                w: data.beatDistribution,
                d: data.beatDistributionType,
                s: data.gapDistribution,
                t: data.gapDistributionType,
                b: data.affectFirst,
                e: data.easing,
                a: data.axis,
                f: data.flip,
                customData: (0, json_js_1.deepCopy)(data.customData),
            },
            eventData: data.events.map((x) => {
                return lightTranslationEvent_js_1.lightTranslationEvent.serialize(x);
            }),
            filterData: indexFilter_js_1.indexFilter.serialize(data.filter),
        };
    },
    deserialize(data) {
        return (0, lightTranslationEventBox_js_1.createLightTranslationEventBox)({
            filter: indexFilter_js_1.indexFilter.deserialize(data.filterData ?? {}),
            beatDistribution: data.data?.w,
            beatDistributionType: data.data?.d,
            gapDistribution: data.data?.s,
            gapDistributionType: data.data?.t,
            affectFirst: data.data?.b,
            easing: data.data?.e,
            axis: data.data?.a,
            flip: data.data?.f,
            events: data.eventData?.map((x) => {
                return lightTranslationEvent_js_1.lightTranslationEvent.deserialize(x);
            }),
            customData: data.data?.customData,
        });
    },
};
