"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lightshow = void 0;
// deno-lint-ignore-file no-explicit-any
const constants_js_1 = require("../../../types/beatmap/shared/constants.js");
const json_js_1 = require("../../../utils/misc/json.js");
const beatmap_js_1 = require("../../core/beatmap.js");
const difficulty_js_1 = require("../../core/difficulty.js");
const lightshow_js_1 = require("../../core/lightshow.js");
const basicEventTypesWithKeywords_js_1 = require("../v3/basicEventTypesWithKeywords.js");
const basicEvent_js_1 = require("./basicEvent.js");
const colorBoostEvent_js_1 = require("./colorBoostEvent.js");
const fxEventBoxGroup_js_1 = require("./fxEventBoxGroup.js");
const lightColorEventBoxGroup_js_1 = require("./lightColorEventBoxGroup.js");
const lightRotationEventBoxGroup_js_1 = require("./lightRotationEventBoxGroup.js");
const lightTranslationEventBoxGroup_js_1 = require("./lightTranslationEventBoxGroup.js");
const waypoint_js_1 = require("./waypoint.js");
/**
 * Schema serialization for v4 `Lightshow`.
 */
exports.lightshow = {
    serialize(data) {
        const json = {
            version: '4.0.0',
            waypoints: [],
            waypointsData: [],
            basicEvents: [],
            basicEventsData: [],
            colorBoostEvents: [],
            colorBoostEventsData: [],
            eventBoxGroups: [],
            indexFilters: [],
            lightColorEventBoxes: [],
            lightColorEvents: [],
            lightRotationEventBoxes: [],
            lightRotationEvents: [],
            lightTranslationEventBoxes: [],
            lightTranslationEvents: [],
            fxEventBoxes: [],
            floatFxEvents: [],
            basicEventTypesWithKeywords: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.serialize(data.lightshow.basicEventTypesWithKeywords),
            useNormalEventsAsCompatibleEvents: data.lightshow.useNormalEventsAsCompatibleEvents,
            customData: (0, json_js_1.deepCopy)(data.lightshow.customData),
        };
        for (const jsonObj of data.lightshow.waypoints.map((x) => {
            return waypoint_js_1.waypoint.serialize(x);
        })) {
            json.waypoints.push(jsonObj.object);
            jsonObj.object.i = json.waypointsData.length;
            json.waypointsData.push(jsonObj.data);
        }
        for (const jsonObj of data.lightshow.basicEvents.map((x) => {
            return basicEvent_js_1.basicEvent.serialize(x);
        })) {
            json.basicEvents.push(jsonObj.object);
            jsonObj.object.i = json.basicEventsData.length;
            json.basicEventsData.push(jsonObj.data);
        }
        for (const jsonObj of data.lightshow.colorBoostEvents.map((x) => {
            return colorBoostEvent_js_1.colorBoostEvent.serialize(x);
        })) {
            json.colorBoostEvents.push(jsonObj.object);
            jsonObj.object.i = json.colorBoostEventsData.length;
            json.colorBoostEventsData.push(jsonObj.data);
        }
        for (const obj of data.lightshow.lightColorEventBoxGroups.map((x) => {
            return lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.serialize(x);
        })) {
            json.eventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                const list = [];
                for (const evt of box.eventData) {
                    list.push({
                        b: evt.time,
                        i: json.lightColorEvents.length,
                    });
                    json.lightColorEvents.push(evt.data);
                }
                obj.object.e.push({
                    e: json.lightColorEventBoxes.length,
                    f: json.indexFilters.length,
                    l: list,
                    customData: {},
                });
                json.lightColorEventBoxes.push(box.data);
                json.indexFilters.push(box.filterData);
            }
        }
        for (const obj of data.lightshow.lightRotationEventBoxGroups.map((x) => {
            return lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.serialize(x);
        })) {
            json.eventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                const list = [];
                for (const evt of box.eventData) {
                    list.push({
                        b: evt.time,
                        i: json.lightRotationEvents.length,
                    });
                    json.lightRotationEvents.push(evt.data);
                }
                obj.object.e.push({
                    e: json.lightRotationEventBoxes.length,
                    f: json.indexFilters.length,
                    l: list,
                    customData: {},
                });
                json.lightRotationEventBoxes.push(box.data);
                json.indexFilters.push(box.filterData);
            }
        }
        for (const obj of data.lightshow.lightTranslationEventBoxGroups.map((x) => {
            return lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.serialize(x);
        })) {
            json.eventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                const list = [];
                for (const evt of box.eventData) {
                    list.push({
                        b: evt.time,
                        i: json.lightTranslationEvents.length,
                    });
                    json.lightTranslationEvents.push(evt.data);
                }
                obj.object.e.push({
                    e: json.lightTranslationEventBoxes.length,
                    f: json.indexFilters.length,
                    l: list,
                    customData: {},
                });
                json.lightTranslationEventBoxes.push(box.data);
                json.indexFilters.push(box.filterData);
            }
        }
        for (const obj of data.lightshow.fxEventBoxGroups.map((x) => {
            return fxEventBoxGroup_js_1.fxEventBoxGroup.serialize(x);
        })) {
            json.eventBoxGroups.push(obj.object);
            for (const box of obj.boxData) {
                const list = [];
                for (const evt of box.eventData) {
                    list.push({ b: evt.time, i: json.floatFxEvents.length });
                    json.floatFxEvents.push(evt.data);
                }
                obj.object.e.push({
                    e: json.fxEventBoxes.length,
                    f: json.indexFilters.length,
                    l: list,
                    customData: {},
                });
                json.fxEventBoxes.push(box.data);
                json.indexFilters.push(box.filterData);
            }
        }
        return json;
    },
    deserialize(data, options) {
        const lightshow = (0, lightshow_js_1.createLightshow)({
            waypoints: data.waypoints?.map((obj) => waypoint_js_1.waypoint.deserialize({
                object: obj,
                data: data.waypointsData?.[obj?.i || 0] ?? {},
            })),
            basicEvents: data.basicEvents?.map((obj) => basicEvent_js_1.basicEvent.deserialize({
                object: obj,
                data: data.basicEventsData?.[obj?.i || 0] ?? {},
            })),
            colorBoostEvents: data.colorBoostEvents?.map((obj) => colorBoostEvent_js_1.colorBoostEvent.deserialize({
                object: obj,
                data: data.colorBoostEventsData?.[obj?.i || 0] ?? {},
            })),
            basicEventTypesWithKeywords: basicEventTypesWithKeywords_js_1.basicEventTypesWithKeywords.deserialize(data.basicEventTypesWithKeywords ?? {}),
            useNormalEventsAsCompatibleEvents: !!data.useNormalEventsAsCompatibleEvents,
            customData: data.customData,
        });
        const indFil = data.indexFilters ?? [];
        const lceb = data.lightColorEventBoxes ?? [];
        const lce = data.lightColorEvents ?? [];
        const lreb = data.lightRotationEventBoxes ?? [];
        const lre = data.lightRotationEvents ?? [];
        const lteb = data.lightTranslationEventBoxes ?? [];
        const lte = data.lightTranslationEvents ?? [];
        const fxb = data.fxEventBoxes ?? [];
        const fx = data.floatFxEvents ?? [];
        for (const ebg of data.eventBoxGroups || []) {
            const t = ebg?.t || 0;
            switch (t) {
                case constants_js_1.EventBoxType.COLOR:
                    lightshow.lightColorEventBoxGroups.push(lightColorEventBoxGroup_js_1.lightColorEventBoxGroup.deserialize({
                        object: ebg ?? {},
                        boxData: ebg?.e?.map((e) => ({
                            data: lceb[e.e || 0],
                            filterData: indFil[e.f || 0],
                            eventData: e.l?.map((l) => ({
                                time: l.b ?? 0,
                                data: lce[l.i || 0],
                            })) ?? [],
                        })) ?? [],
                    }));
                    break;
                case constants_js_1.EventBoxType.ROTATION:
                    lightshow.lightRotationEventBoxGroups.push(lightRotationEventBoxGroup_js_1.lightRotationEventBoxGroup.deserialize({
                        object: ebg,
                        boxData: ebg?.e?.map((e) => ({
                            data: lreb[e.e || 0],
                            filterData: indFil[e.f || 0],
                            eventData: e.l?.map((l) => ({
                                time: l.b ?? 0,
                                data: lre[l.i || 0],
                            })) ?? [],
                        })) ?? [],
                    }));
                    break;
                case constants_js_1.EventBoxType.TRANSLATION:
                    lightshow.lightTranslationEventBoxGroups.push(lightTranslationEventBoxGroup_js_1.lightTranslationEventBoxGroup.deserialize({
                        object: ebg,
                        boxData: ebg?.e?.map((e) => ({
                            data: lteb[e.e || 0],
                            filterData: indFil[e.f || 0],
                            eventData: e.l?.map((l) => ({
                                time: l.b ?? 0,
                                data: lte[l.i || 0],
                            })) ?? [],
                        })) ?? [],
                    }));
                    break;
                case constants_js_1.EventBoxType.FX_FLOAT:
                    lightshow.fxEventBoxGroups.push(fxEventBoxGroup_js_1.fxEventBoxGroup.deserialize({
                        object: ebg,
                        boxData: ebg?.e?.map((e) => ({
                            data: fxb[e.e || 0],
                            filterData: indFil[e.f || 0],
                            eventData: e.l?.map((l) => ({
                                time: l.b ?? 0,
                                data: fx[l.i || 0],
                            })) ?? [],
                        })) ?? [],
                    }));
                    break;
            }
        }
        return (0, beatmap_js_1.createBeatmap)({
            version: 4,
            filename: options?.filename,
            lightshowFilename: options?.filename,
            difficulty: (0, difficulty_js_1.createDifficulty)(),
            lightshow: lightshow,
        });
    },
};
