"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeDifficulty = optimizeDifficulty;
const helpers_js_1 = require("../../../../utils/math/helpers.js");
const optimize_js_1 = require("../../../helpers/optimize.js");
const json_js_1 = require("../../../../utils/misc/json.js");
/**
 * Optimize v4 `Difficulty` schema data.
 */
function optimizeDifficulty(data, options) {
    if (options.deduplicate) {
        const [newNoteColorData, remapColorNoteIdx] = (0, optimize_js_1.remapDedupe)(data.colorNotesData ?? []);
        const [newBombNoteData, remapBombNoteIdx] = (0, optimize_js_1.remapDedupe)(data.bombNotesData ?? []);
        const [newObstacleData, remapObstacleIdx] = (0, optimize_js_1.remapDedupe)(data.obstaclesData ?? []);
        const [newChainData, remapChainIdx] = (0, optimize_js_1.remapDedupe)(data.chainsData ?? []);
        const [newArcData, remapArcIdx] = (0, optimize_js_1.remapDedupe)(data.arcsData ?? []);
        const [newSRData, remapSRIdx] = (0, optimize_js_1.remapDedupe)(data.spawnRotationsData ?? []);
        if (data.colorNotes) {
            for (let i = 0; i < data.colorNotes.length; i++) {
                const d = data.colorNotes[i];
                d.i = remapColorNoteIdx.get(d.i);
            }
        }
        if (data.bombNotes) {
            for (let i = 0; i < data.bombNotes.length; i++) {
                const d = data.bombNotes[i];
                d.i = remapBombNoteIdx.get(d.i);
            }
        }
        if (data.obstacles) {
            for (let i = 0; i < data.obstacles.length; i++) {
                const d = data.obstacles[i];
                d.i = remapObstacleIdx.get(d.i);
            }
        }
        if (data.chains) {
            for (let i = 0; i < data.chains.length; i++) {
                const d = data.chains[i];
                d.ci = remapChainIdx.get(d.ci);
                d.i = remapColorNoteIdx.get(d.i);
            }
        }
        if (data.arcs) {
            for (let i = 0; i < data.arcs.length; i++) {
                const d = data.arcs[i];
                d.ai = remapArcIdx.get(d.ai);
                d.hi = remapColorNoteIdx.get(d.hi);
                d.ti = remapColorNoteIdx.get(d.ti);
            }
        }
        if (data.spawnRotations) {
            for (let i = 0; i < data.spawnRotations.length; i++) {
                const d = data.spawnRotations[i];
                d.i = remapSRIdx.get(d.i);
            }
        }
        data.colorNotesData = newNoteColorData;
        data.bombNotesData = newBombNoteData;
        data.obstaclesData = newObstacleData;
        data.chainsData = newChainData;
        data.arcsData = newArcData;
        if (newSRData.length)
            data.spawnRotationsData = newSRData;
        else
            delete data.spawnRotationsData;
    }
    for (let i = 0; i < data.arcs.length; i++) {
        const o = data.arcs[i];
        if (options.floatTrim) {
            o.hb = (0, helpers_js_1.round)(o.hb, options.floatTrim);
            o.hr = (0, helpers_js_1.round)(o.hr, options.floatTrim);
            o.tb = (0, helpers_js_1.round)(o.tb, options.floatTrim);
            o.tr = (0, helpers_js_1.round)(o.tr, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.arcs[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.arcsData.length; i++) {
        const o = data.arcsData[i];
        if (options.floatTrim) {
            o.m = (0, helpers_js_1.round)(o.m, options.floatTrim);
            o.tm = (0, helpers_js_1.round)(o.tm, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.arcsData[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.bombNotes.length; i++) {
        const o = data.bombNotes[i];
        if (options.floatTrim) {
            o.b = (0, helpers_js_1.round)(o.b, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.bombNotes[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.bombNotesData.length; i++) {
        const o = data.bombNotesData[i];
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.bombNotesData[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.chains.length; i++) {
        const o = data.chains[i];
        if (options.floatTrim) {
            o.hb = (0, helpers_js_1.round)(o.hb, options.floatTrim);
            o.hr = (0, helpers_js_1.round)(o.hr, options.floatTrim);
            o.tb = (0, helpers_js_1.round)(o.tb, options.floatTrim);
            o.tr = (0, helpers_js_1.round)(o.tr, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.chains[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.chainsData.length; i++) {
        const o = data.chainsData[i];
        if (options.floatTrim) {
            o.s = (0, helpers_js_1.round)(o.s, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.chainsData[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.colorNotes.length; i++) {
        const o = data.colorNotes[i];
        if (options.floatTrim) {
            o.b = (0, helpers_js_1.round)(o.b, options.floatTrim);
            o.r = (0, helpers_js_1.round)(o.r, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.colorNotes[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.colorNotesData.length; i++) {
        const o = data.colorNotesData[i];
        if (options.floatTrim) {
            o.a = (0, helpers_js_1.round)(o.a, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.colorNotesData[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.obstacles.length; i++) {
        const o = data.obstacles[i];
        if (options.floatTrim) {
            o.b = (0, helpers_js_1.round)(o.b, options.floatTrim);
            o.r = (0, helpers_js_1.round)(o.r, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.obstacles[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    for (let i = 0; i < data.obstaclesData.length; i++) {
        const o = data.obstaclesData[i];
        if (options.floatTrim) {
            o.d = (0, helpers_js_1.round)(o.d, options.floatTrim);
        }
        (0, optimize_js_1.deepClean)(o.customData, `difficulty.obstaclesData[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(o.customData)) {
            delete o.customData;
        }
        if (options.purgeZeros)
            (0, optimize_js_1.purgeZeros)(o);
    }
    (0, optimize_js_1.deepClean)(data.customData, `difficulty.customData`, options);
    if ((0, json_js_1.isEmpty)(data.customData)) {
        delete data.customData;
    }
}
