"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeInfo = optimizeInfo;
const helpers_js_1 = require("../../../../utils/math/helpers.js");
const optimize_js_1 = require("../../../helpers/optimize.js");
const json_js_1 = require("../../../../utils/misc/json.js");
/**
 * Optimize v4 `Info` schema data.
 */
function optimizeInfo(data, options) {
    if (options.floatTrim) {
        data.audio.bpm = (0, helpers_js_1.round)(data.audio.bpm, options.floatTrim);
        data.audio.previewDuration = (0, helpers_js_1.round)(data.audio.previewDuration, options.floatTrim);
        data.audio.previewStartTime = (0, helpers_js_1.round)(data.audio.previewStartTime, options.floatTrim);
        data.audio.songDuration = (0, helpers_js_1.round)(data.audio.songDuration, options.floatTrim);
    }
    if (options.stringTrim) {
        data.song.author = data.song.author.trim();
        data.song.title = data.song.title.trim();
        data.song.subTitle = data.song.subTitle.trim();
    }
    for (let i = 0; i < data.difficultyBeatmaps.length; i++) {
        const diff = data.difficultyBeatmaps[i];
        if (options.floatTrim) {
            diff.noteJumpMovementSpeed = (0, helpers_js_1.round)(diff.noteJumpMovementSpeed, options.floatTrim);
            diff.noteJumpStartBeatOffset = (0, helpers_js_1.round)(diff.noteJumpStartBeatOffset, options.floatTrim);
        }
        if (options.stringTrim) {
            diff.beatmapAuthors.lighters = diff.beatmapAuthors.lighters.map((s) => s.trim());
            diff.beatmapAuthors.mappers = diff.beatmapAuthors.mappers.map((s) => s.trim());
        }
        (0, optimize_js_1.deepClean)(diff.customData, `info.difficultyBeatmaps[${i}].customData`, options);
        if ((0, json_js_1.isEmpty)(diff.customData)) {
            delete diff.customData;
        }
    }
    (0, optimize_js_1.deepClean)(data.customData, `info.customData`, options);
    if ((0, json_js_1.isEmpty)(data.customData)) {
        delete data.customData;
    }
}
