"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compatibilityCheck = compatibilityCheck;
// deno-lint-ignore-file no-explicit-any
const logger_js_1 = require("../../logger.js");
const compatibility_js_1 = require("../mapping/compatibility.js");
function tag(name) {
    return ['validator', name];
}
const defaultOptions = {
    enabled: true,
    throwOn: { incompatibleObject: true, mappingExtensions: false },
};
/**
 * Check if the beatmap is compatible with the version schema.
 */
function compatibilityCheck(type, data, version, options) {
    const opt = {
        enabled: options?.enabled ?? defaultOptions.enabled,
        throwOn: {
            ...defaultOptions.throwOn,
            ...options?.throwOn,
        },
    };
    let compatibilityMap = {};
    switch (type) {
        case 'info':
            compatibilityMap = compatibility_js_1.infoCompatibilityMap;
            break;
        case 'audioData':
            compatibilityMap = compatibility_js_1.audioDataCompatibilityMap;
            break;
        case 'difficulty':
            compatibilityMap = compatibility_js_1.difficultyCompatibilityMap;
            break;
        case 'lightshow':
            compatibilityMap = compatibility_js_1.lightshowCompatibilityMap;
            break;
    }
    const compatFn = compatibilityMap[version];
    if (compatFn) {
        logger_js_1.logger.tInfo(tag('compatibilityCheck'), 'Checking beatmap compatibility for ' + type + ' with version', version);
        compatFn(data, opt);
    }
    return data;
}
