"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateJSON = validateJSON;
const logger_js_1 = require("../../logger.js");
const validator_js_1 = require("../mapping/validator.js");
const schema_js_1 = require("./schema.js");
function tag(name) {
    return ['validator', name];
}
const defaultOptions = {
    enabled: true,
    throwOn: {
        unused: false,
        missing: true,
        ignoreOptional: false,
        nullish: true,
        wrongType: true,
        notInt: false,
        notUnsigned: false,
    },
};
/**
 * Validate the beatmap JSON data against the schema.
 */
function validateJSON(type, data, version, options) {
    const opt = {
        enabled: options?.enabled ?? defaultOptions.enabled,
        throwOn: {
            ...defaultOptions.throwOn,
            ...options?.throwOn,
        },
    };
    logger_js_1.logger.tInfo(tag('validateJSON'), 'Validating beatmap JSON for ' + type + ' with version', version);
    let schema;
    switch (type) {
        case 'info': {
            schema = validator_js_1.infoCheckMap[version];
            break;
        }
        case 'audioData': {
            schema = validator_js_1.audioSchemaMap[version];
            break;
        }
        case 'difficulty': {
            schema = validator_js_1.difficultyCheckMap[version];
            break;
        }
        case 'lightshow': {
            schema = validator_js_1.lightshowCheckMap[version];
            break;
        }
    }
    (0, schema_js_1.schemaCheck)(data, schema, type, undefined, opt.throwOn);
    return data;
}
