"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setColor = setColor;
exports.setColorGradient = setColorGradient;
exports.setColorRandom = setColorRandom;
const logger_js_1 = require("../../../logger.js");
const convertor_js_1 = require("../../../utils/colors/convertor.js");
const helpers_js_1 = require("../../../utils/colors/helpers.js");
const helpers_js_2 = require("../../../utils/math/helpers.js");
const settings_js_1 = require("./settings.js");
function tag(name) {
    return ['ext', 'chroma', 'color', name];
}
function setColor(objects, options) {
    const opt = {
        color: options.color,
        colorType: options.colorType ?? (settings_js_1.settings.colorType || 'hsva'),
    };
    const color = (0, convertor_js_1.convertColorType)(opt.color, opt.colorType);
    objects.forEach((obj) => {
        obj.customData.color = color;
    });
}
function setColorGradient(objects, options) {
    if (!objects.length) {
        logger_js_1.logger.tWarn(tag('setColorGradient'), 'No object(s) received.');
        return;
    }
    const opt = {
        offsetStart: options.offsetStart,
        offsetEnd: options.offsetEnd,
        colorStart: options.colorStart,
        colorEnd: options.colorEnd,
        colorType: options.colorType ?? 'rgba',
        easingColor: options.easingColor ??
            function (x) {
                return x;
            },
    };
    const startTime = objects.at(0).time + opt.offsetStart;
    const endTime = objects.at(-1).time + opt.offsetEnd;
    objects.forEach((obj) => {
        const norm = (0, helpers_js_2.normalize)(obj.time, startTime, endTime);
        const color = (0, helpers_js_1.lerpColor)(opt.colorStart, opt.colorEnd, opt.easingColor(norm), opt.colorType);
        obj.customData.color = color;
    });
}
function setColorRandom(objects, options) {
    const opt = {
        offsetStart: options.offsetStart,
        offsetEnd: options.offsetEnd,
        color1: options.color1,
        color2: options.color2,
        colorType: options.colorType ?? 'rgba',
    };
    let random = 0;
    let prevTime = 0;
    for (let i = 0, l = objects.length; i < l; i++) {
        if (objects[i].time > prevTime + 0.001) {
            random = Math.random();
        }
        const color = (0, helpers_js_1.lerpColor)(opt.color1, opt.color2, random, opt.colorType);
        objects[i].customData._color = color;
        prevTime = objects[i].time;
    }
}
