"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNjs = setNjs;
exports.simultaneousSpawn = simultaneousSpawn;
exports.gradientNjs = gradientNjs;
const njs_js_1 = require("../../../beatmap/helpers/njs.js");
const timeProcessor_js_1 = require("../../../beatmap/helpers/timeProcessor.js");
const logger_js_1 = require("../../../logger.js");
const helpers_js_1 = require("../../../utils/math/helpers.js");
const settings_js_1 = require("./settings.js");
function tag(name) {
    return ['ext', 'NE', 'njs', name];
}
/**
 * Set NJS to object from start to end object.
 *
 * **NOTE:** JD input will override NJS offset.
 */
function setNjs(objects, options) {
    if (!objects.length) {
        logger_js_1.logger.tWarn(tag('setNJS'), 'No object(s) received.');
        return;
    }
    const njs = typeof options.njs === 'number'
        ? new njs_js_1.NoteJumpSpeed(options.timeProc.bpm, options.njs, options.offset)
        : options.njs;
    const offset = njs.calcHjdFromJd(options.jd) - njs.calcHjd(0);
    objects.forEach((o) => {
        o.customData.noteJumpMovementSpeed = njs.value;
        o.customData.noteJumpStartBeatOffset = offset;
    });
}
/**
 * Simultaneously spawn the object from start to end object.
 *
 * Speed determines how fast should note spawn from start to end. (1 is regular speed)
 *
 * **NOTE:** JD input will override NJS offset.
 */
function simultaneousSpawn(objects, options) {
    if (!objects.length) {
        logger_js_1.logger.tWarn(tag('simultaneousSpawn'), 'No object(s) received.');
        return;
    }
    if (!options.speed) {
        logger_js_1.logger.tError(tag('simultaneousSpawn'), 'Speed cannot be 0!');
        options.speed = 1;
    }
    options.spawnBeatOffset = options.spawnBeatOffset ?? 0;
    const njs = typeof options.njs === 'number' ? options.njs : options.njs.value;
    const startTime = objects[0].time;
    objects.forEach((o) => {
        o.customData.noteJumpMovementSpeed = options.njsOverride
            ? o.customData.noteJumpMovementSpeed ?? njs
            : njs;
        const currentNJS = new njs_js_1.NoteJumpSpeed(options.timeProc.bpm, o.customData.noteJumpMovementSpeed);
        const offset = currentNJS.calcHjdFromJd(options.jd) - currentNJS.calcHjd(0);
        o.customData.noteJumpStartBeatOffset = options.spawnBeatOffset +
            offset +
            o.time -
            startTime -
            (o.time - startTime) / options.speed;
    });
}
/**
 * Gradually change NJS for objects from start to end objects.
 *
 * **NOTE:** JD input will override NJS offset.
 */
function gradientNjs(objects, options) {
    if (!objects.length) {
        logger_js_1.logger.tWarn(tag('gradientNJS'), 'No object(s) received.');
        return;
    }
    options.easing = options.easing ??
        function (x) {
            return x;
        };
    let offset;
    if (typeof options.njsOffset !== 'number') {
        if (options.njsOffset) {
            offset = options.njsOffset.offset;
        }
        else {
            offset = settings_js_1.settings.NJS?.offset ?? 0;
        }
    }
    else {
        offset = options.njsOffset;
    }
    const startTime = objects[0].time;
    const endTime = objects.at(-1).time;
    objects.forEach((o) => {
        o.customData.noteJumpMovementSpeed = (0, helpers_js_1.lerp)(options.easing((0, helpers_js_1.normalize)(o.time, startTime, endTime)), options.njsStart, options.njsEnd);
        if (typeof options.jd === 'number') {
            const currNJS = new njs_js_1.NoteJumpSpeed(options.timeProc instanceof timeProcessor_js_1.TimeProcessor ? options.timeProc.bpm : options.timeProc, o.customData.noteJumpMovementSpeed, offset);
            o.customData.noteJumpStartBeatOffset = currNJS.calcHjdFromJd(options.jd) -
                currNJS.calcHjd(0);
        }
    });
}
