"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCircle = createCircle;
exports.drawPolygon = drawPolygon;
exports.drawPath = drawPath;
const trigonometry_js_1 = require("../../../utils/math/trigonometry.js");
/** Create points around circle, higher precision is more accurate. */
function createCircle(radius, precision, angleOffset) {
    const points = [];
    angleOffset = angleOffset || 0;
    const offset = (0, trigonometry_js_1.degToRad)(angleOffset % 360);
    for (let i = 0; i < precision; i++) {
        points.push([
            radius * Math.cos(((2 * Math.PI) / precision + offset) * i),
            radius * Math.sin(((2 * Math.PI) / precision + offset) * i),
        ]);
    }
    return points;
}
function drawPolygon(radius, sides, options) {
    const opt = {
        angleOffset: options?.angleOffset ?? 0,
        type: options?.type ?? 'inscribed',
    };
    if (sides < 3) {
        throw new Error('sides cannot be less than 3');
    }
    if (radius === 0) {
        throw new Error('radius cannot be 0');
    }
    const coordinates = [];
    const rotations = [];
    const sizes = [];
    const offset = (0, trigonometry_js_1.degToRad)(((opt.angleOffset % 360) + 360) % 360);
    let coordinateNext;
    const length = 2 * radius * Math.sin(Math.PI / sides);
    for (let i = 0; i < sides; i++) {
        const coordinate = coordinateNext ?? [
            radius * Math.cos(offset + ((2 * Math.PI) / sides) * i),
            radius * Math.sin(offset + ((2 * Math.PI) / sides) * i),
        ];
        coordinateNext = [
            radius * Math.cos(offset + ((2 * Math.PI) / sides) * (i + 1)),
            radius * Math.sin(offset + ((2 * Math.PI) / sides) * (i + 1)),
        ];
        const rotation = (-90 +
            (0, trigonometry_js_1.radToDeg)(Math.atan2(coordinateNext[1] - coordinate[1], coordinateNext[0] - coordinate[0])) +
            360) %
            360;
        coordinates.push(coordinate);
        rotations.push(rotation);
        sizes.push(length);
    }
    return { coordinates, rotations, sizes };
}
function drawPath(path) {
    const coordinates = [];
    const rotations = [];
    const sizes = [];
    let coordinateNext;
    for (let i = 0; i < path.length - 1; i++) {
        const coordinate = coordinateNext ?? [path[i][0], path[i][1]];
        coordinateNext = [path[i + 1][0], path[i + 1][1]];
        const rotation = (-90 +
            (0, trigonometry_js_1.radToDeg)(Math.atan2(coordinateNext[1] - coordinate[1], coordinateNext[0] - coordinate[0])) +
            360) %
            360;
        const size = Math.sqrt(Math.pow(coordinateNext[0] - coordinate[0], 2) +
            Math.pow(coordinateNext[1] - coordinate[1], 2)) * 2;
        coordinates.push(coordinate);
        rotations.push(rotation);
        sizes.push(size);
    }
    return { coordinates, rotations, sizes };
}
