"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTrack = addTrack;
exports.removeTrack = removeTrack;
function _addTrack(currentTrack, trackToAdd) {
    if (!trackToAdd) {
        return typeof currentTrack === 'string'
            ? currentTrack
            : Array.isArray(currentTrack)
                ? currentTrack.filter((s) => s)
                : undefined;
    }
    if (typeof trackToAdd === 'string') {
        return _addTrack(currentTrack, [trackToAdd]);
    }
    if (typeof currentTrack === 'string') {
        return [currentTrack, ...trackToAdd];
    }
    if (Array.isArray(currentTrack)) {
        currentTrack.push(...trackToAdd);
        return currentTrack.filter((s) => s);
    }
    return [...trackToAdd];
}
function _removeTrack(currentTrack, trackToRemove) {
    if (!trackToRemove || !currentTrack) {
        return typeof currentTrack === 'string'
            ? currentTrack
            : Array.isArray(currentTrack)
                ? currentTrack.filter((s) => s)
                : undefined;
    }
    if (typeof currentTrack === 'string') {
        return currentTrack === trackToRemove ? undefined : currentTrack;
    }
    if (typeof trackToRemove === 'string') {
        return _removeTrack(currentTrack, [trackToRemove]);
    }
    for (const track of trackToRemove) {
        currentTrack.splice(currentTrack.indexOf(track), 1);
    }
    return currentTrack.length ? currentTrack.filter((s) => s) : undefined;
}
function addTrack(objects, track) {
    if (!Array.isArray(objects)) {
        objects.customData ||= {};
        objects.customData.track = _addTrack(objects.customData.track, track);
    }
    else {
        objects.forEach((obj) => {
            addTrack(obj, track);
        });
    }
    return objects;
}
function removeTrack(objects, track) {
    if (!Array.isArray(objects)) {
        if (objects.customData) {
            objects.customData.track = _removeTrack(objects.customData.track, track);
        }
    }
    else {
        objects.forEach((obj) => {
            removeTrack(obj, track);
        });
    }
    return objects;
}
