"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parity = void 0;
// rotation 0 means down vertical note
// TODO: proper rotation check based on position
// TODO: AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA there's still more work needed for parity check
// TODO: cleanup the implementation
const baseNote_js_1 = require("../../beatmap/helpers/core/baseNote.js");
const constants_js_1 = require("../../beatmap/shared/constants.js");
const note_js_1 = require("../placement/note.js");
const parity_js_1 = require("./types/parity.js");
const noteInitParity = {
    [constants_js_1.NoteColor.RED]: {
        forehand: [
            constants_js_1.NoteDirection.DOWN,
            constants_js_1.NoteDirection.RIGHT,
            constants_js_1.NoteDirection.DOWN_LEFT,
            constants_js_1.NoteDirection.DOWN_RIGHT,
        ],
        backhand: [
            constants_js_1.NoteDirection.UP,
            constants_js_1.NoteDirection.LEFT,
            constants_js_1.NoteDirection.UP_LEFT,
            constants_js_1.NoteDirection.UP_RIGHT,
        ],
    },
    [constants_js_1.NoteColor.BLUE]: {
        forehand: [
            constants_js_1.NoteDirection.DOWN,
            constants_js_1.NoteDirection.LEFT,
            constants_js_1.NoteDirection.DOWN_LEFT,
            constants_js_1.NoteDirection.DOWN_RIGHT,
        ],
        backhand: [
            constants_js_1.NoteDirection.UP,
            constants_js_1.NoteDirection.RIGHT,
            constants_js_1.NoteDirection.UP_LEFT,
            constants_js_1.NoteDirection.UP_RIGHT,
        ],
    },
};
const noteInitRotation = {
    [constants_js_1.NoteColor.RED]: [0, 0, 90, 90, 45, -45, 45, -45],
    [constants_js_1.NoteColor.BLUE]: [0, 0, -90, -90, -45, 45, -45, 45],
};
const noteParityRotation = {
    0: {
        forehand: [180, 0, -90, 90, -135, 135, -45, 45],
        backhand: [0, 180, 90, -90, 45, -45, 135, -135],
    },
    1: {
        forehand: [-180, 0, -90, 90, -135, 135, -45, 45],
        backhand: [0, -180, 90, -90, 45, -45, 135, -135],
    },
};
// TODO: probably body class for leaning
class Parity {
    constructor(notes, bombs, type, warningThreshold, errorThreshold, allowedRotation, parity) {
        Object.defineProperty(this, "state", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "color", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "rotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "position", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "warningThreshold", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "errorThreshold", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "allowedRotation", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.color = type;
        this.warningThreshold = warningThreshold;
        this.errorThreshold = errorThreshold;
        this.allowedRotation = allowedRotation;
        if (parity && parity !== 'neutral') {
            this.state = parity;
            this.rotation = 0;
        }
        else {
            this.state = this.predictStartState(notes, bombs, type);
            this.rotation = this.predictStartRotation(notes, type);
        }
        this.position = this.predictStartPosition(notes, type);
    }
    check(noteContext, bombContext) {
        if (this.state === 'neutral') {
            return 'none';
        }
        if (!noteContext.length) {
            throw new Error('no notes were given');
        }
        const startTime = noteContext[0].time;
        const noteType = this.color;
        let currentState = this.state;
        let currentRotation = this.rotation;
        bombContext.forEach((bomb) => {
            if (bomb.time - 0.001 > startTime) {
                return;
            }
            if (bomb.posY === constants_js_1.PosY.BOTTOM) {
                if (bomb.posX === (noteType ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT)) {
                    currentState = 'backhand';
                    currentRotation = 0;
                }
            }
            if (bomb.posY === constants_js_1.PosY.TOP) {
                if (bomb.posX === (noteType ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT)) {
                    currentState = 'forehand';
                    currentRotation = 0;
                }
            }
        });
        let prevNote;
        let expectedDirection = constants_js_1.NoteDirection.ANY;
        for (const note of noteContext) {
            if (note.direction !== constants_js_1.NoteDirection.ANY) {
                expectedDirection = note.direction;
            }
            if (prevNote && expectedDirection === constants_js_1.NoteDirection.ANY) {
                expectedDirection = (0, note_js_1.predictDirection)(note, prevNote);
            }
            prevNote = note;
        }
        if (expectedDirection === constants_js_1.NoteDirection.ANY) {
            return 'none';
        }
        const parityRotation = noteParityRotation[noteType][parity_js_1.ParitySwitch[currentState]][expectedDirection];
        if ((currentRotation > parityRotation
            ? currentRotation - parityRotation
            : parityRotation - currentRotation) > 180) {
            return 'error';
        }
        if (parityRotation < Parity.CONSTRAINT_ROTATION[noteType][0] + this.errorThreshold ||
            parityRotation > Parity.CONSTRAINT_ROTATION[noteType][1] - this.errorThreshold) {
            return 'error';
        }
        if (parityRotation < Parity.CONSTRAINT_ROTATION[noteType][0] + this.warningThreshold ||
            parityRotation > Parity.CONSTRAINT_ROTATION[noteType][1] - this.warningThreshold) {
            return 'warning';
        }
        if ((currentRotation > parityRotation
            ? currentRotation - parityRotation
            : parityRotation - currentRotation) > this.allowedRotation) {
            return 'warning';
        }
        return 'none';
    }
    next(noteContext, bombContext) {
        if (this.check(noteContext, bombContext) !== 'error') {
            switch (this.state) {
                case 'forehand': {
                    this.state = 'backhand';
                    break;
                }
                case 'backhand': {
                    this.state = 'forehand';
                    break;
                }
                case 'neutral': {
                    for (let i = 0; i < noteContext.length; i++) {
                        const note = noteContext[i];
                        if (noteInitParity[note.color].forehand.includes(note.direction)) {
                            this.state = 'backhand';
                            break;
                        }
                        if (noteInitParity[note.color].backhand.includes(note.direction)) {
                            this.state = 'forehand';
                            break;
                        }
                        if (this.state === 'neutral' && note.direction === constants_js_1.NoteDirection.ANY) {
                            if (note.posY === 0) {
                                this.state = 'backhand';
                            }
                            if (note.posY > 0) {
                                this.state = 'forehand';
                            }
                        }
                    }
                    break;
                }
            }
        }
        if (this.state === 'neutral') {
            throw new Error('parity unexpected input');
        }
        const startTime = noteContext[0].time;
        const noteType = this.color;
        bombContext.forEach((bomb) => {
            if (bomb.time - 0.001 > startTime) {
                return;
            }
            if (bomb.posY === constants_js_1.PosY.BOTTOM) {
                if (bomb.posX === (noteType ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT)) {
                    this.state = 'forehand';
                    this.rotation = 0;
                }
            }
            if (bomb.posY === constants_js_1.PosY.TOP) {
                if (bomb.posX === (noteType ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT)) {
                    this.state = 'backhand';
                    this.rotation = 0;
                }
            }
        });
        let prevNote;
        let expectedDirection = constants_js_1.NoteDirection.ANY;
        for (const note of noteContext) {
            if (note.direction !== constants_js_1.NoteDirection.ANY) {
                expectedDirection = note.direction;
            }
            if (prevNote && expectedDirection === constants_js_1.NoteDirection.ANY) {
                expectedDirection = (0, note_js_1.predictDirection)(note, prevNote);
            }
            prevNote = note;
        }
        if (expectedDirection !== constants_js_1.NoteDirection.ANY) {
            this.rotation = noteParityRotation[this.color][this.state][expectedDirection];
        }
    }
    predictStartState(notes, bombs, type) {
        const nc = [...notes, ...bombs].sort((a, b) => a.time - b.time);
        let startParity = 'neutral';
        for (let i = 0, len = nc.length; i < len; i++) {
            if ((0, baseNote_js_1.isUnsetNoteColor)(nc[i].color)) {
                if (nc[i].posY === constants_js_1.PosY.BOTTOM) {
                    if (nc[i].posX === type ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT) {
                        startParity = 'backhand';
                    }
                }
                if (nc[i].posY === constants_js_1.PosY.TOP) {
                    if (nc[i].posX === type ? constants_js_1.PosX.MIDDLE_RIGHT : constants_js_1.PosX.MIDDLE_LEFT) {
                        startParity = 'forehand';
                    }
                }
                continue;
            }
            let note = nc[i];
            if (note.color === Math.abs(type - 1)) {
                continue;
            }
            if (note.color === type) {
                if (startParity !== 'neutral') {
                    break;
                }
                const startTime = note.time;
                for (let j = i; j < nc.length; j++) {
                    if (nc[j].time > note.time + 0.001 && startTime < note.time + 0.001) {
                        break;
                    }
                    note = nc[j];
                    if (noteInitParity[note.color].forehand.includes(note.direction)) {
                        return 'backhand';
                    }
                    if (noteInitParity[note.color].backhand.includes(note.direction)) {
                        return 'forehand';
                    }
                    if (startParity === 'neutral' && note.direction === constants_js_1.NoteDirection.ANY) {
                        if (note.posY === constants_js_1.PosY.BOTTOM) {
                            startParity = 'backhand';
                        }
                        if (note.posY > constants_js_1.PosY.BOTTOM) {
                            startParity = 'forehand';
                        }
                    }
                }
                break;
            }
        }
        return startParity;
    }
    predictStartRotation(nc, color) {
        let rotation = 0;
        for (let i = 0, len = nc.length; i < len; i++) {
            let note = nc[i];
            if (note.color !== color) {
                continue;
            }
            if (note.color === color) {
                const startTime = note.time;
                for (let j = i; j < nc.length; j++) {
                    if (nc[j].time > note.time + 0.001 && startTime < note.time + 0.001) {
                        break;
                    }
                    note = nc[j];
                    if (note.direction !== constants_js_1.NoteDirection.ANY) {
                        return noteInitRotation[note.color][note.direction];
                    }
                    if (note.direction === constants_js_1.NoteDirection.ANY) {
                        if (note.posY === constants_js_1.PosY.BOTTOM) {
                            if (note.posX === constants_js_1.PosX.LEFT) {
                                rotation = noteInitRotation[note.color][6];
                            }
                            if (note.posX === constants_js_1.PosX.RIGHT) {
                                rotation = noteInitRotation[note.color][7];
                            }
                        }
                        if (note.posY === constants_js_1.PosY.MIDDLE) {
                            if (note.posX === constants_js_1.PosX.LEFT) {
                                rotation = noteInitRotation[note.color][2];
                            }
                            if (note.posX === constants_js_1.PosX.RIGHT) {
                                rotation = noteInitRotation[note.color][3];
                            }
                        }
                        if (note.posY === constants_js_1.PosY.BOTTOM) {
                            if (note.posX === constants_js_1.PosX.LEFT) {
                                rotation = noteInitRotation[note.color][4];
                            }
                            if (note.posX === constants_js_1.PosX.RIGHT) {
                                rotation = noteInitRotation[note.color][5];
                            }
                        }
                    }
                }
                break;
            }
        }
        return rotation;
    }
    // "predict" btw
    predictStartPosition(_, type) {
        return type ? [-0.5, 1] : [0.5, 1];
    }
}
exports.Parity = Parity;
Object.defineProperty(Parity, "CONSTRAINT_ROTATION", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: [
        [-155, 195],
        [-195, 155],
    ]
});
