import type { TimeProcessor } from '../../beatmap/helpers/timeProcessor.js';
import type { IWrapBaseObject } from '../../types/beatmap/wrapper/baseObject.js';
/**
 * Return objects at given time, adjusted by BPM change if provided.
 * ```ts
 * const notesHere = at(notes, 42);
 * console.log(...notesHere);
 * ```
 */
export declare function at<T extends Pick<IWrapBaseObject, 'time'>>(objects: T[], times: number | number[], timeProc?: TimeProcessor | null): T[];
/**
 * Return objects at given time range, adjusted by BPM change if provided.
 * ```ts
 * const notesRange = between(notes, 42, 69);
 * console.log(...notesRange);
 * ```
 */
export declare function between<T extends Pick<IWrapBaseObject, 'time'>>(objects: T[], from: number, to: number, timeProc?: TimeProcessor | null): T[];
/**
 * Return objects before specified time, adjusted by BPM change if provided.
 * ```ts
 * const notesBefore = before(notes, 42);
 * console.log(...notesBefore);
 * ```
 */
export declare function before<T extends Pick<IWrapBaseObject, 'time'>>(objects: T[], before: number, timeProc?: TimeProcessor | null): T[];
/**
 * Return objects after specified time, adjusted by BPM change if provided.
 * ```ts
 * const notesAfter = after(notes, 69);
 * console.log(...notesAfter);
 * ```
 */
export declare function after<T extends Pick<IWrapBaseObject, 'time'>>(objects: T[], after: number, timeProc?: TimeProcessor | null): T[];
//# sourceMappingURL=time.d.ts.map