"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.at = at;
exports.between = between;
exports.before = before;
exports.after = after;
const settings_js_1 = require("./settings.js");
/**
 * Return objects at given time, adjusted by BPM change if provided.
 * ```ts
 * const notesHere = at(notes, 42);
 * console.log(...notesHere);
 * ```
 */
function at(objects, times, timeProc) {
    timeProc = timeProc ?? settings_js_1.settings.timeProcessor;
    if (Array.isArray(times)) {
        return objects.filter((o) => times.some((time) => timeProc ? timeProc.adjustTime(o.time) === time : o.time === time));
    }
    return objects.filter((o) => timeProc ? timeProc.adjustTime(o.time) === times : o.time === times);
}
/**
 * Return objects at given time range, adjusted by BPM change if provided.
 * ```ts
 * const notesRange = between(notes, 42, 69);
 * console.log(...notesRange);
 * ```
 */
function between(objects, from, to, timeProc) {
    timeProc = timeProc ?? settings_js_1.settings.timeProcessor;
    return objects.filter((o) => timeProc
        ? timeProc.adjustTime(o.time) >= from &&
            timeProc.adjustTime(o.time) <= to
        : o.time >= from && o.time <= to);
}
/**
 * Return objects before specified time, adjusted by BPM change if provided.
 * ```ts
 * const notesBefore = before(notes, 42);
 * console.log(...notesBefore);
 * ```
 */
function before(objects, before, timeProc) {
    timeProc = timeProc ?? settings_js_1.settings.timeProcessor;
    return objects.filter((o) => timeProc ? timeProc.adjustTime(o.time) < before : o.time < before);
}
/**
 * Return objects after specified time, adjusted by BPM change if provided.
 * ```ts
 * const notesAfter = after(notes, 69);
 * console.log(...notesAfter);
 * ```
 */
function after(objects, after, timeProc) {
    timeProc = timeProc ?? settings_js_1.settings.timeProcessor;
    return objects.filter((o) => timeProc ? timeProc.adjustTime(o.time) > after : o.time > after);
}
