"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.countEvent = countEvent;
const basicEvent_js_1 = require("../../beatmap/helpers/core/basicEvent.js");
const has_js_1 = require("../../beatmap/helpers/modded/has.js");
const environment_js_1 = require("../../beatmap/shared/environment.js");
const mod_js_1 = require("../../mod.js");
/**
 * Count number of type of events with their properties in given array and return a event count object.
 * ```ts
 * const list = count(events);
 * console.log(list);
 * ```
 */
function countEvent(events, boost, environment = 'DefaultEnvironment', version = 2) {
    const commonEvent = environment_js_1.EventList[environment]?.[0] ?? environment_js_1.EventList['DefaultEnvironment'][0];
    const eventCount = {};
    for (let i = commonEvent.length - 1; i >= 0; i--) {
        eventCount[commonEvent[i]] = {
            total: 0,
            chroma: 0,
            chromaOld: 0,
        };
    }
    const hasChroma = version >= 3 ? has_js_1.hasChromaEventV3 : has_js_1.hasChromaEventV2;
    eventCount[5] = {
        total: boost.length,
        chroma: 0,
        chromaOld: 0,
    };
    for (let i = events.length - 1; i >= 0; i--) {
        if ((0, mod_js_1.isValidEventType)(events[i].type)) {
            if (!eventCount[events[i].type]) {
                eventCount[events[i].type] = {
                    total: 0,
                    chroma: 0,
                    chromaOld: 0,
                };
            }
            eventCount[events[i].type].total++;
            if (hasChroma(events[i])) {
                eventCount[events[i].type].chroma++;
            }
            if ((0, basicEvent_js_1.isOldChromaEventValue)(events[i].value)) {
                eventCount[events[i].type].chromaOld++;
            }
        }
    }
    return eventCount;
}
