import type { GetAngleFn } from '../../types/beatmap/shared/functions.js';
import type { IWrapBaseNote } from '../../types/beatmap/wrapper/baseNote.js';
import type { IWrapBaseSlider } from '../../types/beatmap/wrapper/baseSlider.js';
import type { IWrapBombNote } from '../../types/beatmap/wrapper/bombNote.js';
import type { IWrapColorNote } from '../../types/beatmap/wrapper/colorNote.js';
import type { IWrapGridObject } from '../../types/beatmap/wrapper/gridObject.js';
import type { ICountNote, ICountStatsNote } from './types/stats.js';
/**
 * Count number of red, blue, and bomb notes with their properties in given array and return a note count object.
 *
 * ```ts
 * const list = count(notes);
 * console.log(list);
 * ```
 */
export declare function countNote<T extends IWrapColorNote | IWrapBaseSlider>(notes: T[], version?: number): ICountNote;
export declare function countBomb<T extends IWrapBombNote>(bombs: T[], version?: number): ICountStatsNote;
/**
 * Count number of specified pos X in a given array and return a counted number of pos X.
 *
 * ```ts
 * const xCount = countX(notes, 0);
 * ```
 */
export declare function countX<T extends IWrapGridObject>(objs: T[], x: number): number;
/**
 * Count number of specified pos Y in a given array and return a counted number of pos Y.
 *
 * ```ts
 * const yCount = countY(notes, 0);
 * ```
 */
export declare function countY<T extends IWrapGridObject>(objs: T[], y: number): number;
/**
 * Count number of specified pos X and Y in a given array and return a counted number of pos X and Y.
 *
 * ```ts
 * const xyCount = countXY(notes, 0, 0);
 * ```
 */
export declare function countXY<T extends IWrapGridObject>(objs: T[], x: number, y: number): number;
/**
 * Count number of specified `direction` in a given array and return a counted number of `direction`.
 * ```ts
 * const cdCount = countDirection(notes, 0);
 * ```
 */
export declare function countDirection<T extends IWrapBaseNote>(notes: T[], cd: number): number;
/**
 * Count number of specified angle in a given array and return a counted number of angle.
 *
 * ```ts
 * const angleCount = countAngle(notes, 0);
 * ```
 */
export declare function countAngle<T extends IWrapBaseNote>(notes: T[], angle: number, fn?: GetAngleFn<T>): number;
//# sourceMappingURL=note.d.ts.map