"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.countObstacle = countObstacle;
const obstacle_js_1 = require("../../beatmap/helpers/core/obstacle.js");
const has_js_1 = require("../../beatmap/helpers/modded/has.js");
/**
 * Count number of type of obstacles with their properties in given array and return a obstacle count object.
 * ```ts
 * const list = count(walls);
 * console.log(list);
 * ```
 */
function countObstacle(obstacles, version = 2) {
    const obstacleCount = {
        total: 0,
        interactive: 0,
        chroma: 0,
        noodleExtensions: 0,
        mappingExtensions: 0,
    };
    const hasChroma = version >= 3 ? has_js_1.hasChromaObstacleV3 : has_js_1.hasChromaObstacleV2;
    const hasNoodle = version >= 3 ? has_js_1.hasNoodleExtensionsObstacleV3 : has_js_1.hasNoodleExtensionsObstacleV2;
    const hasME = version >= 3 ? has_js_1.hasMappingExtensionsObstacleV3 : has_js_1.hasMappingExtensionsObstacleV2;
    for (let i = obstacles.length - 1; i > -1; i--) {
        obstacleCount.total++;
        if ((0, obstacle_js_1.isInteractiveObstacle)(obstacles[i])) {
            obstacleCount.interactive++;
        }
        if (hasChroma(obstacles[i])) {
            obstacleCount.chroma++;
        }
        if (hasNoodle(obstacles[i])) {
            obstacleCount.noodleExtensions++;
        }
        if (hasME(obstacles[i])) {
            obstacleCount.mappingExtensions++;
        }
    }
    return obstacleCount;
}
