import type { TimeProcessor } from '../../beatmap/helpers/timeProcessor.js';
import type { CharacteristicName } from '../../types/beatmap/shared/characteristic.js';
import type { DifficultyName } from '../../types/beatmap/shared/difficulty.js';
import type { ISwingAnalysis, ISwingAnalysisBaseNote, ISwingAnalysisBeatmap, ISwingCount } from './types/swing.js';
/**
 * Count the number of swings in period.
 */
export declare function count<T extends ISwingAnalysisBaseNote>(colorNotes: T[], duration: number, timeProc: TimeProcessor): ISwingCount;
/**
 * Generate swing analysis from beatmap.
 *
 * Port from Uninstaller's Swings Per Second tool
 */
export declare function info<T extends ISwingAnalysisBeatmap>(beatmap: T, timeProc: TimeProcessor, characteristic: CharacteristicName, difficulty: DifficultyName): ISwingAnalysis<ISwingAnalysisBaseNote>;
/**
 * Get first swings analysis that exceeds 40% progression drop.
 */
export declare function getProgressionMax<T extends ISwingAnalysisBaseNote>(spsArray: ISwingAnalysis<T>[], minThreshold: number): {
    result: ISwingAnalysis<T>;
    comparedTo?: ISwingAnalysis<T>;
} | null;
/**
 * Get first swings analysis that does not reach 10% progression drop.
 */
export declare function getProgressionMin<T extends ISwingAnalysisBaseNote>(spsArray: ISwingAnalysis<T>[], minThreshold: number): {
    result: ISwingAnalysis<T>;
    comparedTo?: ISwingAnalysis<T>;
} | null;
/**
 * Calculate total percentage drop from highest to lowest swings analysis.
 */
export declare function calcSpsTotalPercDrop<T extends ISwingAnalysisBaseNote>(spsArray: ISwingAnalysis<T>[]): number;
/**
 * Get lowest SPS.
 */
export declare function getSpsLowest<T extends ISwingAnalysisBaseNote>(spsArray: ISwingAnalysis<T>[]): number;
/**
 * Get highest SPS.
 */
export declare function getSpsHighest<T extends ISwingAnalysisBaseNote>(spsArray: ISwingAnalysis<T>[]): number;
//# sourceMappingURL=info.d.ts.map