"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _a, _Logger_logLevel, _Logger_tagPrint, _Logger_untagged;
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.Logger = void 0;
// deno-lint-ignore-file no-explicit-any
// i got super annoyed about dependencies that i decided to yoink it from deno
function yellow(str) {
    return `\x1b[33m${str}\x1b[39m`;
}
function red(str) {
    return `\x1b[31m${str}\x1b[39m`;
}
function dim(str) {
    return `\x1b[2m${str}\x1b[22m`;
}
var LogLevels;
(function (LogLevels) {
    LogLevels[LogLevels["NONE"] = 0] = "NONE";
    LogLevels[LogLevels["ERROR"] = 1] = "ERROR";
    LogLevels[LogLevels["WARN"] = 2] = "WARN";
    LogLevels[LogLevels["INFO"] = 3] = "INFO";
    LogLevels[LogLevels["DEBUG"] = 4] = "DEBUG";
    LogLevels[LogLevels["TRACE"] = 5] = "TRACE";
})(LogLevels || (LogLevels = {}));
// really simple logger
/** Simple logging system class. */
class Logger {
    constructor() {
        _Logger_logLevel.set(this, LogLevels.INFO);
        _Logger_tagPrint.set(this, (tags, level) => `${_a.LogPrefixes.get(level)} ${dim('>')} [${dim(tags.join('::'))}]`);
        _Logger_untagged.set(this, 'script');
    }
    set logLevel(value) {
        __classPrivateFieldSet(this, _Logger_logLevel, value, "f");
        this.tInfo(['logger', 'logLevel'], `Log level set to ${_a.LogPrefixes.get(value)}`);
    }
    get logLevel() {
        return __classPrivateFieldGet(this, _Logger_logLevel, "f");
    }
    set tagPrint(fn) {
        __classPrivateFieldSet(this, _Logger_tagPrint, fn, "f");
        this.tInfo(['logger', 'tagPrint'], `Update tag print function`);
    }
    get tagPrint() {
        return __classPrivateFieldGet(this, _Logger_tagPrint, "f");
    }
    set untagged(value) {
        __classPrivateFieldSet(this, _Logger_untagged, value.trim(), "f");
        this.tInfo(['logger', 'untagged'], `Update untagged string to ${__classPrivateFieldGet(this, _Logger_untagged, "f")}`);
    }
    get untagged() {
        return __classPrivateFieldGet(this, _Logger_untagged, "f");
    }
    log(level, tags, args) {
        if (level > __classPrivateFieldGet(this, _Logger_logLevel, "f"))
            return;
        const tag = this.tagPrint(tags, level);
        switch (level) {
            case LogLevels.TRACE:
                return tag ? console.trace(tag, ...args) : console.trace(...args);
            case LogLevels.DEBUG:
                return tag ? console.debug(tag, ...args) : console.debug(...args);
            case LogLevels.INFO:
                return tag ? console.info(tag, ...args) : console.info(...args);
            case LogLevels.WARN:
                return tag ? console.warn(tag, ...args) : console.warn(...args);
            case LogLevels.ERROR:
                return tag ? console.error(tag, ...args) : console.error(...args);
            default:
                return tag ? console.log(tag, ...args) : console.log(...args);
        }
    }
    /**
     * Set logging level to filter various information.
     * ```ts
     * 0 -> None
     * 1 -> Error
     * 2 -> Warn
     * 3 -> Info
     * 4 -> Debug
     * 5 -> Trace
     * ```
     */
    setLevel(level) {
        level = Math.min(Math.max(level, 0), 5);
        __classPrivateFieldSet(this, _Logger_logLevel, level, "f");
        this.tInfo(['logger', 'setLevel'], `Log level set to ${_a.LogPrefixes.get(level)}`);
    }
    tTrace(tags, ...args) {
        this.log(LogLevels.TRACE, tags, args);
    }
    tDebug(tags, ...args) {
        this.log(LogLevels.DEBUG, tags, args);
    }
    tInfo(tags, ...args) {
        this.log(LogLevels.INFO, tags, args);
    }
    tWarn(tags, ...args) {
        this.log(LogLevels.WARN, tags, args);
    }
    tError(tags, ...args) {
        this.log(LogLevels.ERROR, tags, args);
    }
    trace(...args) {
        this.tTrace([__classPrivateFieldGet(this, _Logger_untagged, "f")], ...args);
    }
    debug(...args) {
        this.tDebug([__classPrivateFieldGet(this, _Logger_untagged, "f")], ...args);
    }
    info(...args) {
        this.tInfo([__classPrivateFieldGet(this, _Logger_untagged, "f")], ...args);
    }
    warn(...args) {
        this.tWarn([__classPrivateFieldGet(this, _Logger_untagged, "f")], ...args);
    }
    error(...args) {
        this.tError([__classPrivateFieldGet(this, _Logger_untagged, "f")], ...args);
    }
}
exports.Logger = Logger;
_a = Logger, _Logger_logLevel = new WeakMap(), _Logger_tagPrint = new WeakMap(), _Logger_untagged = new WeakMap();
Object.defineProperty(Logger, "LogLevels", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: LogLevels
});
Object.defineProperty(Logger, "LogPrefixes", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: new Map([
        [LogLevels.TRACE, 'TRACE'],
        [LogLevels.DEBUG, 'DEBUG'],
        [LogLevels.INFO, 'INFO'],
        [LogLevels.WARN, yellow('WARN')],
        [LogLevels.ERROR, red('!!ERROR!!')],
        [LogLevels.NONE, 'NONE'],
    ])
});
/**
 * Logging instance used in the module.
 *
 * Can be replaced as needed.
 */
// deno-lint-ignore prefer-const
exports.logger = new Logger();
