"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.customDataUpdate = customDataUpdate;
const eventToV2_js_1 = __importDefault(require("../beatmap/converter/customData/eventToV2.js"));
const eventToV3_js_1 = __importDefault(require("../beatmap/converter/customData/eventToV3.js"));
const objectToV2_js_1 = __importDefault(require("../beatmap/converter/customData/objectToV2.js"));
const objectToV3_js_1 = __importDefault(require("../beatmap/converter/customData/objectToV3.js"));
const basicEvent_js_1 = require("../beatmap/helpers/core/basicEvent.js");
const logger_js_1 = require("../logger.js");
const convertor_js_1 = require("../utils/colors/convertor.js");
function tag(name) {
    return ['patch', 'customDataUpdate', name];
}
function v2(data) {
    logger_js_1.logger.tDebug(tag('v2'), ' Patching notes');
    data.difficulty.colorNotes.forEach((n) => {
        n.customData = (0, objectToV2_js_1.default)(n.customData);
    });
    data.difficulty.bombNotes.forEach((n) => {
        n.customData = (0, objectToV2_js_1.default)(n.customData);
    });
    logger_js_1.logger.tDebug(tag('v2'), ' Patching obstacles');
    data.difficulty.obstacles.forEach((o) => {
        o.customData = (0, objectToV2_js_1.default)(o.customData);
    });
    logger_js_1.logger.tDebug(tag('v2'), ' Patching events');
    data.lightshow.basicEvents.forEach((e) => {
        e.customData = (0, eventToV2_js_1.default)(e.customData);
        if ((0, basicEvent_js_1.isLaserRotationEventType)(e.type)) {
            if (typeof e.customData._preciseSpeed !== 'number') {
                delete e.customData._speed;
            }
        }
        else {
            delete e.customData._preciseSpeed;
        }
    });
}
function v3(data) {
    logger_js_1.logger.tDebug(tag('v3'), ' Patching color notes');
    data.difficulty.colorNotes.forEach((n) => {
        n.customData = (0, objectToV3_js_1.default)(n.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching bomb notes');
    data.difficulty.bombNotes.forEach((b) => {
        b.customData = (0, objectToV3_js_1.default)(b.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching obstacles');
    data.difficulty.obstacles.forEach((o) => {
        o.customData = (0, objectToV3_js_1.default)(o.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching fake color notes');
    data.difficulty.customData.fakeColorNotes?.forEach((n) => {
        n.customData = (0, objectToV3_js_1.default)(n.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching fake bomb notes');
    data.difficulty.customData.fakeBombNotes?.forEach((b) => {
        b.customData = (0, objectToV3_js_1.default)(b.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching fake obstacles');
    data.difficulty.customData.fakeObstacles?.forEach((o) => {
        o.customData = (0, objectToV3_js_1.default)(o.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching basic events');
    data.lightshow.basicEvents.forEach((e) => {
        e.customData = (0, eventToV3_js_1.default)(e.customData);
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching bookmarks');
    data.difficulty.customData.bookmarks?.forEach((b) => {
        if ('_time' in b && typeof b._time === 'number') {
            b.b ??= b._time;
            delete b._time;
        }
        if ('_name' in b && typeof b._name === 'string') {
            b.n ??= b._name;
            delete b._name;
        }
        if ('_color' in b && Array.isArray(b._color)) {
            try {
                b.c ??= (0, convertor_js_1.colorFrom)(b._color);
            }
            catch {
                // just try anyway
            }
            delete b._color;
        }
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching BPM changes');
    data.difficulty.customData.BPMChanges?.forEach((bpmc) => {
        if ('_time' in bpmc && typeof bpmc._time === 'number') {
            bpmc.b ??= bpmc._time;
            delete bpmc._time;
        }
        if ('_BPM' in bpmc && typeof bpmc._BPM === 'number') {
            bpmc.m ??= bpmc._BPM;
            delete bpmc._BPM;
        }
        if ('_beatsPerBar' in bpmc && typeof bpmc._beatsPerBar === 'number') {
            bpmc.p ??= bpmc._beatsPerBar;
            delete bpmc._beatsPerBar;
        }
        if ('_metronomeOffset' in bpmc &&
            typeof bpmc._metronomeOffset === 'number') {
            bpmc.o ??= bpmc._metronomeOffset;
            delete bpmc._metronomeOffset;
        }
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching environment');
    data.difficulty.customData.environment?.forEach((env) => {
        if ('lightID' in env && typeof env.lightID === 'number') {
            const id = env.lightID;
            if (env.components) {
                if (env.components.ILightWithId) {
                    env.components.ILightWithId.lightID ??= id;
                }
                else {
                    env.components.ILightWithId = { lightID: id };
                }
            }
            else {
                env.components = { ILightWithId: { lightID: id } };
            }
            delete env.lightID;
        }
    });
    logger_js_1.logger.tDebug(tag('v3'), ' Patching point definitions');
    if (Array.isArray(data.customData.pointDefinitions)) {
        const fixedObj = {};
        data.customData.pointDefinitions.forEach((pd) => (fixedObj[pd.name] = pd.points));
        data.customData.pointDefinitions = fixedObj;
    }
}
/**
 * Update custom data for beatmap given version.
 */
function customDataUpdate(data, version) {
    logger_js_1.logger.tInfo(['patch', 'customDataUpdate'], 'Patching custom data for beatmap v' + version + '...');
    switch (version) {
        case 2:
            v2(data);
            break;
        case 3:
            v3(data);
            break;
    }
}
