"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.beatmap = beatmap;
const arc_js_1 = require("../../beatmap/core/arc.js");
const basicEvent_js_1 = require("../../beatmap/core/basicEvent.js");
const bombNote_js_1 = require("../../beatmap/core/bombNote.js");
const bpmEvent_js_1 = require("../../beatmap/core/bpmEvent.js");
const chain_js_1 = require("../../beatmap/core/chain.js");
const colorBoostEvent_js_1 = require("../../beatmap/core/colorBoostEvent.js");
const colorNote_js_1 = require("../../beatmap/core/colorNote.js");
const fxEventBox_js_1 = require("../../beatmap/core/fxEventBox.js");
const fxEventBoxGroup_js_1 = require("../../beatmap/core/fxEventBoxGroup.js");
const fxEventFloat_js_1 = require("../../beatmap/core/fxEventFloat.js");
const fxEventInt_js_1 = require("../../beatmap/core/fxEventInt.js");
const indexFilter_js_1 = require("../../beatmap/core/indexFilter.js");
const lightColorEvent_js_1 = require("../../beatmap/core/lightColorEvent.js");
const lightColorEventBox_js_1 = require("../../beatmap/core/lightColorEventBox.js");
const lightColorEventBoxGroup_js_1 = require("../../beatmap/core/lightColorEventBoxGroup.js");
const lightRotationEvent_js_1 = require("../../beatmap/core/lightRotationEvent.js");
const lightRotationEventBox_js_1 = require("../../beatmap/core/lightRotationEventBox.js");
const lightRotationEventBoxGroup_js_1 = require("../../beatmap/core/lightRotationEventBoxGroup.js");
const lightTranslationEvent_js_1 = require("../../beatmap/core/lightTranslationEvent.js");
const lightTranslationEventBox_js_1 = require("../../beatmap/core/lightTranslationEventBox.js");
const lightTranslationEventBoxGroup_js_1 = require("../../beatmap/core/lightTranslationEventBoxGroup.js");
const obstacle_js_1 = require("../../beatmap/core/obstacle.js");
const rotationEvent_js_1 = require("../../beatmap/core/rotationEvent.js");
const waypoint_js_1 = require("../../beatmap/core/waypoint.js");
const basicEvent_js_2 = require("../../beatmap/helpers/core/basicEvent.js");
const constants_js_1 = require("../../beatmap/shared/constants.js");
const logger_js_1 = require("../../logger.js");
const customDataEvent_js_1 = require("./customDataEvent.js");
const customDataObject_js_1 = require("./customDataObject.js");
const helpers_js_1 = require("./helpers.js");
function fixBpmEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, bpmEvent_js_1.BPMEvent.defaultValue.time);
    obj.bpm = (0, helpers_js_1.fixFloat)(obj.bpm);
}
function fixRotationEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, rotationEvent_js_1.RotationEvent.defaultValue.time);
    obj.executionTime = (0, helpers_js_1.fixInt)(obj.executionTime, rotationEvent_js_1.RotationEvent.defaultValue.executionTime, [0, 1]);
    obj.rotation = (0, helpers_js_1.fixFloat)(obj.rotation, rotationEvent_js_1.RotationEvent.defaultValue.rotation);
}
function fixColorNote(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, colorNote_js_1.ColorNote.defaultValue.time);
    obj.color = (0, helpers_js_1.fixInt)(obj.color, [0, 1], [0, 1]);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, colorNote_js_1.ColorNote.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, colorNote_js_1.ColorNote.defaultValue.posY);
    obj.direction = (0, helpers_js_1.fixInt)(obj.direction, colorNote_js_1.ColorNote.defaultValue.direction);
    obj.angleOffset = (0, helpers_js_1.fixInt)(obj.angleOffset, colorNote_js_1.ColorNote.defaultValue.angleOffset);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixFakeColorNote(obj) {
    obj.b = (0, helpers_js_1.fixFloat)(obj.b, colorNote_js_1.ColorNote.defaultValue.time);
    obj.x = (0, helpers_js_1.fixInt)(obj.x, colorNote_js_1.ColorNote.defaultValue.posX);
    obj.y = (0, helpers_js_1.fixInt)(obj.y, colorNote_js_1.ColorNote.defaultValue.posY);
    obj.c = (0, helpers_js_1.fixInt)(obj.c, [0, 1], [0, 1]);
    obj.d = (0, helpers_js_1.fixInt)(obj.d, colorNote_js_1.ColorNote.defaultValue.direction);
    obj.a = (0, helpers_js_1.fixInt)(obj.a, colorNote_js_1.ColorNote.defaultValue.angleOffset);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixBombNote(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, bombNote_js_1.BombNote.defaultValue.time);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, bombNote_js_1.BombNote.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, bombNote_js_1.BombNote.defaultValue.posY);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixFakeBombNote(obj) {
    obj.b = (0, helpers_js_1.fixFloat)(obj.b, bombNote_js_1.BombNote.defaultValue.time);
    obj.x = (0, helpers_js_1.fixInt)(obj.x, bombNote_js_1.BombNote.defaultValue.posX);
    obj.y = (0, helpers_js_1.fixInt)(obj.y, bombNote_js_1.BombNote.defaultValue.posY);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixObstacle(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, obstacle_js_1.Obstacle.defaultValue.time);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, obstacle_js_1.Obstacle.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, obstacle_js_1.Obstacle.defaultValue.posY);
    obj.duration = (0, helpers_js_1.fixFloat)(obj.duration, obstacle_js_1.Obstacle.defaultValue.duration);
    obj.width = (0, helpers_js_1.fixInt)(obj.width, obstacle_js_1.Obstacle.defaultValue.width);
    obj.height = (0, helpers_js_1.fixInt)(obj.height, obstacle_js_1.Obstacle.defaultValue.height);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixFakeObstacle(obj) {
    obj.b = (0, helpers_js_1.fixFloat)(obj.b, obstacle_js_1.Obstacle.defaultValue.time);
    obj.x = (0, helpers_js_1.fixInt)(obj.x, obstacle_js_1.Obstacle.defaultValue.posX);
    obj.y = (0, helpers_js_1.fixInt)(obj.y, obstacle_js_1.Obstacle.defaultValue.posY);
    obj.d = (0, helpers_js_1.fixFloat)(obj.d, obstacle_js_1.Obstacle.defaultValue.duration);
    obj.w = (0, helpers_js_1.fixInt)(obj.w, obstacle_js_1.Obstacle.defaultValue.width);
    obj.h = (0, helpers_js_1.fixInt)(obj.h, obstacle_js_1.Obstacle.defaultValue.height);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixSlider(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, arc_js_1.Arc.defaultValue.time);
    obj.color = (0, helpers_js_1.fixInt)(obj.color, [0, 1], [0, 1]);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, arc_js_1.Arc.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, arc_js_1.Arc.defaultValue.posY);
    obj.direction = (0, helpers_js_1.fixInt)(obj.direction, arc_js_1.Arc.defaultValue.direction);
    obj.lengthMultiplier = (0, helpers_js_1.fixFloat)(obj.lengthMultiplier, arc_js_1.Arc.defaultValue.lengthMultiplier);
    obj.tailTime = (0, helpers_js_1.fixFloat)(obj.tailTime, arc_js_1.Arc.defaultValue.tailTime);
    obj.tailPosX = (0, helpers_js_1.fixInt)(obj.tailPosX, arc_js_1.Arc.defaultValue.tailPosX);
    obj.tailPosY = (0, helpers_js_1.fixInt)(obj.tailPosY, arc_js_1.Arc.defaultValue.tailPosY);
    obj.tailDirection = (0, helpers_js_1.fixInt)(obj.tailDirection, arc_js_1.Arc.defaultValue.tailDirection);
    obj.tailLengthMultiplier = (0, helpers_js_1.fixFloat)(obj.tailLengthMultiplier, arc_js_1.Arc.defaultValue.tailLengthMultiplier);
    obj.midAnchor = (0, helpers_js_1.fixInt)(obj.midAnchor, arc_js_1.Arc.defaultValue.midAnchor);
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixChain(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, chain_js_1.Chain.defaultValue.time);
    obj.color = (0, helpers_js_1.fixInt)(obj.color, [0, 1], [0, 1]);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, chain_js_1.Chain.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, chain_js_1.Chain.defaultValue.posY);
    obj.direction = (0, helpers_js_1.fixInt)(obj.direction, chain_js_1.Chain.defaultValue.direction);
    obj.tailTime = (0, helpers_js_1.fixFloat)(obj.tailTime, chain_js_1.Chain.defaultValue.tailTime);
    obj.tailPosX = (0, helpers_js_1.fixInt)(obj.tailPosX, chain_js_1.Chain.defaultValue.tailPosX);
    obj.tailPosY = (0, helpers_js_1.fixInt)(obj.tailPosY, chain_js_1.Chain.defaultValue.tailPosY);
    obj.sliceCount = (0, helpers_js_1.fixInt)(obj.sliceCount, chain_js_1.Chain.defaultValue.sliceCount) || 1;
    obj.squish = (0, helpers_js_1.fixFloat)(obj.squish, chain_js_1.Chain.defaultValue.squish) || 0.001;
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixFakeChain(obj) {
    obj.b = (0, helpers_js_1.fixFloat)(obj.b, chain_js_1.Chain.defaultValue.time);
    obj.c = (0, helpers_js_1.fixInt)(obj.c, [0, 1], [0, 1]);
    obj.x = (0, helpers_js_1.fixInt)(obj.x, chain_js_1.Chain.defaultValue.posX);
    obj.y = (0, helpers_js_1.fixInt)(obj.y, chain_js_1.Chain.defaultValue.posY);
    obj.d = (0, helpers_js_1.fixInt)(obj.d, chain_js_1.Chain.defaultValue.direction);
    obj.tb = (0, helpers_js_1.fixFloat)(obj.tb, chain_js_1.Chain.defaultValue.tailTime);
    obj.tx = (0, helpers_js_1.fixInt)(obj.tx, chain_js_1.Chain.defaultValue.tailPosX);
    obj.ty = (0, helpers_js_1.fixInt)(obj.ty, chain_js_1.Chain.defaultValue.tailPosY);
    obj.sc = (0, helpers_js_1.fixInt)(obj.sc, chain_js_1.Chain.defaultValue.sliceCount) || 1;
    obj.s = (0, helpers_js_1.fixFloat)(obj.s, chain_js_1.Chain.defaultValue.squish) || 0.001;
    (0, customDataObject_js_1.fixCustomDataObject)(obj.customData);
}
function fixWaypoint(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, waypoint_js_1.Waypoint.defaultValue.time);
    obj.posX = (0, helpers_js_1.fixInt)(obj.posX, waypoint_js_1.Waypoint.defaultValue.posX);
    obj.posY = (0, helpers_js_1.fixInt)(obj.posY, waypoint_js_1.Waypoint.defaultValue.posY);
    obj.direction = (0, helpers_js_1.fixInt)(obj.direction, waypoint_js_1.Waypoint.defaultValue.direction);
}
function fixBasicEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, basicEvent_js_1.BasicEvent.defaultValue.time);
    obj.type = (0, helpers_js_1.fixInt)(obj.type, basicEvent_js_1.BasicEvent.defaultValue.type);
    obj.value = (0, helpers_js_1.fixInt)(obj.value, basicEvent_js_1.BasicEvent.defaultValue.value);
    obj.floatValue = (0, helpers_js_1.fixFloat)(obj.floatValue, basicEvent_js_1.BasicEvent.defaultValue.floatValue);
    (0, customDataEvent_js_1.fixCustomDataEvent)(obj.customData);
}
function fixColorBoostEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, colorBoostEvent_js_1.ColorBoostEvent.defaultValue.time);
    obj.toggle = (0, helpers_js_1.fixBoolean)(obj.toggle, colorBoostEvent_js_1.ColorBoostEvent.defaultValue.toggle);
}
function fixIndexFilter(obj) {
    obj.type = (0, helpers_js_1.fixInt)(obj.type, indexFilter_js_1.IndexFilter.defaultValue.type, [1, 2]);
    obj.p0 = (0, helpers_js_1.fixInt)(obj.p0, indexFilter_js_1.IndexFilter.defaultValue.p0);
    obj.p1 = (0, helpers_js_1.fixInt)(obj.p1, indexFilter_js_1.IndexFilter.defaultValue.p1);
    obj.reverse = (0, helpers_js_1.fixInt)(obj.reverse, indexFilter_js_1.IndexFilter.defaultValue.reverse, [0, 1]);
    obj.chunks = (0, helpers_js_1.fixInt)(obj.chunks, indexFilter_js_1.IndexFilter.defaultValue.chunks);
    obj.random = (0, helpers_js_1.fixInt)(obj.random, indexFilter_js_1.IndexFilter.defaultValue.random, [0, 1, 2, 3]);
    obj.seed = (0, helpers_js_1.fixInt)(obj.seed, indexFilter_js_1.IndexFilter.defaultValue.seed);
    obj.limit = (0, helpers_js_1.fixFloat)(obj.limit, indexFilter_js_1.IndexFilter.defaultValue.limit, 0, 1);
    obj.limitAffectsType = (0, helpers_js_1.fixInt)(obj.limitAffectsType, indexFilter_js_1.IndexFilter.defaultValue.limitAffectsType, [0, 1, 2, 3]);
}
function fixLightColorEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightColorEvent_js_1.LightColorEvent.defaultValue.time);
    obj.frequency = (0, helpers_js_1.fixInt)(obj.frequency, lightColorEvent_js_1.LightColorEvent.defaultValue.frequency);
    obj.color = obj.previous === 1
        ? (0, helpers_js_1.fixInt)(obj.color, -1, [-1, 0, 1, 2])
        : (0, helpers_js_1.fixInt)(obj.color, lightColorEvent_js_1.LightColorEvent.defaultValue.color, [0, 1, 2]);
    obj.easing = (0, helpers_js_1.fixInt)(obj.easing, lightColorEvent_js_1.LightColorEvent.defaultValue.easing, [0, 1, 2]);
    obj.previous = (0, helpers_js_1.fixInt)(obj.previous, lightColorEvent_js_1.LightColorEvent.defaultValue.previous, [0, 1]);
    obj.brightness = (0, helpers_js_1.fixFloat)(obj.brightness, lightColorEvent_js_1.LightColorEvent.defaultValue.brightness);
    obj.strobeBrightness = (0, helpers_js_1.fixFloat)(obj.strobeBrightness, lightColorEvent_js_1.LightColorEvent.defaultValue.strobeBrightness);
    obj.strobeFade = (0, helpers_js_1.fixInt)(obj.strobeFade, lightColorEvent_js_1.LightColorEvent.defaultValue.strobeFade, [0, 1]);
}
function fixLightColorEventBox(obj) {
    fixIndexFilter(obj.filter);
    obj.beatDistribution = (0, helpers_js_1.fixFloat)(obj.beatDistribution, lightColorEventBox_js_1.LightColorEventBox.defaultValue.beatDistribution);
    obj.beatDistributionType = (0, helpers_js_1.fixInt)(obj.beatDistributionType, lightColorEventBox_js_1.LightColorEventBox.defaultValue.beatDistributionType, [1, 2]);
    obj.brightnessDistribution = (0, helpers_js_1.fixFloat)(obj.brightnessDistribution, lightColorEventBox_js_1.LightColorEventBox.defaultValue.brightnessDistribution);
    obj.brightnessDistributionType = (0, helpers_js_1.fixInt)(obj.brightnessDistributionType, lightColorEventBox_js_1.LightColorEventBox.defaultValue.brightnessDistributionType, [1, 2]);
    obj.affectFirst = (0, helpers_js_1.fixInt)(obj.affectFirst, lightColorEventBox_js_1.LightColorEventBox.defaultValue.affectFirst, [0, 1]);
    obj.events.forEach(fixLightColorEvent);
}
function fixLightColorEventBoxGroup(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightColorEventBoxGroup_js_1.LightColorEventBoxGroup.defaultValue.time);
    obj.id = (0, helpers_js_1.fixInt)(obj.id, lightColorEventBoxGroup_js_1.LightColorEventBoxGroup.defaultValue.id);
    obj.boxes.forEach(fixLightColorEventBox);
}
function fixLightRotationEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightRotationEvent_js_1.LightRotationEvent.defaultValue.time);
    obj.easing = (0, helpers_js_1.fixInt)(obj.easing, lightRotationEvent_js_1.LightRotationEvent.defaultValue.easing);
    obj.loop = (0, helpers_js_1.fixInt)(obj.loop, lightRotationEvent_js_1.LightRotationEvent.defaultValue.loop);
    obj.direction = (0, helpers_js_1.fixInt)(obj.direction, lightRotationEvent_js_1.LightRotationEvent.defaultValue.direction, [0, 1, 2]);
    obj.previous = (0, helpers_js_1.fixInt)(obj.previous, lightRotationEvent_js_1.LightRotationEvent.defaultValue.previous, [0, 1]);
    obj.rotation = (0, helpers_js_1.fixFloat)(obj.rotation, lightRotationEvent_js_1.LightRotationEvent.defaultValue.rotation);
}
function fixLightRotationEventBox(obj) {
    fixIndexFilter(obj.filter);
    obj.beatDistribution = (0, helpers_js_1.fixFloat)(obj.beatDistribution, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.beatDistribution);
    obj.beatDistributionType = (0, helpers_js_1.fixInt)(obj.beatDistributionType, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.beatDistributionType, [1, 2]);
    obj.rotationDistribution = (0, helpers_js_1.fixFloat)(obj.rotationDistribution, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.rotationDistribution);
    obj.rotationDistributionType = (0, helpers_js_1.fixInt)(obj.rotationDistributionType, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.rotationDistributionType, [1, 2]);
    obj.axis = (0, helpers_js_1.fixInt)(obj.axis, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.axis, [0, 1, 2]);
    obj.flip = (0, helpers_js_1.fixInt)(obj.flip, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.flip, [0, 1]);
    obj.affectFirst = (0, helpers_js_1.fixInt)(obj.affectFirst, lightRotationEventBox_js_1.LightRotationEventBox.defaultValue.affectFirst, [0, 1]);
    obj.events.forEach(fixLightRotationEvent);
}
function fixLightRotationEventBoxGroup(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightRotationEventBoxGroup_js_1.LightRotationEventBoxGroup.defaultValue.time);
    obj.id = (0, helpers_js_1.fixInt)(obj.id, lightRotationEventBoxGroup_js_1.LightRotationEventBoxGroup.defaultValue.id);
    obj.boxes.forEach(fixLightRotationEventBox);
}
function fixLightTranslationEvent(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightTranslationEvent_js_1.LightTranslationEvent.defaultValue.time);
    obj.easing = (0, helpers_js_1.fixInt)(obj.easing, lightTranslationEvent_js_1.LightTranslationEvent.defaultValue.easing);
    obj.previous = (0, helpers_js_1.fixInt)(obj.previous, lightTranslationEvent_js_1.LightTranslationEvent.defaultValue.previous, [0, 1]);
    obj.translation = (0, helpers_js_1.fixFloat)(obj.translation, lightTranslationEvent_js_1.LightTranslationEvent.defaultValue.translation);
}
function fixLightTranslationEventBox(obj) {
    fixIndexFilter(obj.filter);
    obj.beatDistribution = (0, helpers_js_1.fixFloat)(obj.beatDistribution, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.beatDistribution);
    obj.beatDistributionType = (0, helpers_js_1.fixInt)(obj.beatDistributionType, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.beatDistributionType, [1, 2]);
    obj.gapDistribution = (0, helpers_js_1.fixFloat)(obj.gapDistribution, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.gapDistribution);
    obj.gapDistributionType = (0, helpers_js_1.fixInt)(obj.gapDistributionType, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.gapDistributionType, [1, 2]);
    obj.axis = (0, helpers_js_1.fixInt)(obj.axis, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.axis, [0, 1, 2]);
    obj.flip = (0, helpers_js_1.fixInt)(obj.flip, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.flip, [0, 1]);
    obj.affectFirst = (0, helpers_js_1.fixInt)(obj.affectFirst, lightTranslationEventBox_js_1.LightTranslationEventBox.defaultValue.affectFirst, [0, 1]);
    obj.events.forEach(fixLightTranslationEvent);
}
function fixLightTranslationEventBoxGroup(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, lightTranslationEventBoxGroup_js_1.LightTranslationEventBoxGroup.defaultValue.time);
    obj.id = (0, helpers_js_1.fixInt)(obj.id, lightTranslationEventBoxGroup_js_1.LightTranslationEventBoxGroup.defaultValue.id);
    obj.boxes.forEach(fixLightTranslationEventBox);
}
function fixFxEventBox(obj) {
    fixIndexFilter(obj.filter);
    obj.beatDistribution = (0, helpers_js_1.fixFloat)(obj.beatDistribution, fxEventBox_js_1.FxEventBox.defaultValue.beatDistribution);
    obj.beatDistributionType = (0, helpers_js_1.fixInt)(obj.beatDistributionType, fxEventBox_js_1.FxEventBox.defaultValue.beatDistributionType, [1, 2]);
    obj.fxDistribution = (0, helpers_js_1.fixFloat)(obj.fxDistribution, fxEventBox_js_1.FxEventBox.defaultValue.fxDistribution);
    obj.fxDistributionType = (0, helpers_js_1.fixInt)(obj.fxDistributionType, fxEventBox_js_1.FxEventBox.defaultValue.fxDistributionType, [1, 2]);
    obj.affectFirst = (0, helpers_js_1.fixInt)(obj.affectFirst, fxEventBox_js_1.FxEventBox.defaultValue.affectFirst, [0, 1]);
    obj.events.forEach(fixFxEventFloat);
}
function fixFxEventBoxGroup(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, fxEventBoxGroup_js_1.FxEventBoxGroup.defaultValue.time);
    obj.id = (0, helpers_js_1.fixInt)(obj.id, fxEventBoxGroup_js_1.FxEventBoxGroup.defaultValue.id);
    obj.boxes.forEach(fixFxEventBox);
}
// FIXME: prolly never gonna be used
function _fixFxEventInt(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, fxEventInt_js_1.FxEventInt.defaultValue.time);
    obj.previous = (0, helpers_js_1.fixInt)(obj.previous, fxEventInt_js_1.FxEventInt.defaultValue.previous, [0, 1]);
    obj.value = (0, helpers_js_1.fixFloat)(obj.value, fxEventInt_js_1.FxEventInt.defaultValue.value);
}
function fixFxEventFloat(obj) {
    obj.time = (0, helpers_js_1.fixFloat)(obj.time, fxEventFloat_js_1.FxEventFloat.defaultValue.time);
    obj.easing = (0, helpers_js_1.fixInt)(obj.easing, fxEventFloat_js_1.FxEventFloat.defaultValue.easing);
    obj.previous = (0, helpers_js_1.fixInt)(obj.previous, fxEventFloat_js_1.FxEventFloat.defaultValue.previous, [0, 1]);
    obj.value = (0, helpers_js_1.fixFloat)(obj.value, fxEventFloat_js_1.FxEventFloat.defaultValue.value);
}
/**
 * Verifies and corrects data type for beatmap data.
 */
function beatmap(data) {
    logger_js_1.logger.tInfo(['patch', 'dataCorrection', 'beatmap', 'main'], 'Verifying and correcting data type...');
    data.difficulty.bpmEvents.forEach(fixBpmEvent);
    data.difficulty.rotationEvents.forEach(fixRotationEvent);
    data.difficulty.colorNotes.forEach(fixColorNote);
    data.difficulty.bombNotes.forEach(fixBombNote);
    data.difficulty.obstacles.forEach(fixObstacle);
    data.difficulty.arcs.forEach(fixSlider);
    data.difficulty.chains.forEach(fixChain);
    data.lightshow.waypoints.forEach(fixWaypoint);
    data.difficulty.customData.fakeColorNotes?.forEach(fixFakeColorNote);
    data.difficulty.customData.fakeBombNotes?.forEach(fixFakeBombNote);
    data.difficulty.customData.fakeObstacles?.forEach(fixFakeObstacle);
    data.difficulty.customData.fakeChains?.forEach(fixFakeChain);
    data.lightshow.basicEvents.forEach(fixBasicEvent);
    data.lightshow.colorBoostEvents.forEach(fixColorBoostEvent);
    data.lightshow.lightColorEventBoxGroups.forEach(fixLightColorEventBoxGroup);
    data.lightshow.lightRotationEventBoxGroups.forEach(fixLightRotationEventBoxGroup);
    data.lightshow.lightTranslationEventBoxGroups.forEach(fixLightTranslationEventBoxGroup);
    data.lightshow.fxEventBoxGroups.forEach(fixFxEventBoxGroup);
    data.lightshow.useNormalEventsAsCompatibleEvents = (0, helpers_js_1.fixBoolean)(data.lightshow.useNormalEventsAsCompatibleEvents);
    const boost = data.lightshow.basicEvents.filter((ev) => (0, basicEvent_js_2.isColorBoostEventType)(ev.type));
    const laneRotation = data.lightshow.basicEvents.filter((ev) => (0, basicEvent_js_2.isLaneRotationEventType)(ev.type));
    data.lightshow.basicEvents = data.lightshow.basicEvents.filter((ev) => !((0, basicEvent_js_2.isColorBoostEventType)(ev.type) || (0, basicEvent_js_2.isLaneRotationEventType)(ev.type)));
    data.lightshow.colorBoostEvents.push(...boost.map((ev) => ({
        time: ev.time,
        toggle: ev.value ? true : false,
        customData: {},
    })));
    data.difficulty.rotationEvents.push(...laneRotation.map((ev) => ({
        time: ev.time,
        executionTime: ev.type == 15 ? constants_js_1.ExecutionTime.LATE : constants_js_1.ExecutionTime.EARLY,
        rotation: ev.customData._rotation ??
            (ev.value >= 1000 && ev.value <= 1720
                ? ev.value - 1360
                : ev.value >= 0 && ev.value <= 7
                    ? constants_js_1.EventLaneRotationValue[ev.value]
                    : 0),
        customData: {},
    })));
}
