"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.info = info;
const environment_js_1 = require("../../beatmap/helpers/environment.js");
const environment_js_2 = require("../../beatmap/shared/environment.js");
const logger_js_1 = require("../../logger.js");
const helpers_js_1 = require("../../utils/math/helpers.js");
const helpers_js_2 = require("./helpers.js");
function fixEnvironment(str, all = false) {
    if (typeof str === 'string') {
        if (str === 'Origins')
            return 'OriginsEnvironment';
        if (Object.keys(environment_js_2.EnvironmentRename)
            .filter((env) => all || !(0, environment_js_1.is360Environment)(env))
            .includes(str)) {
            return str;
        }
    }
    return 'DefaultEnvironment';
}
function fixColorObject(val, req) {
    if (typeof val === 'object' && !Array.isArray(val) && val) {
        if (req) {
            return {
                r: (0, helpers_js_2.fixFloat)(val.r, 0),
                g: (0, helpers_js_2.fixFloat)(val.g, 0),
                b: (0, helpers_js_2.fixFloat)(val.b, 0),
                a: (0, helpers_js_2.fixFloat)(val.a, 1),
            };
        }
        else {
            return {
                r: (0, helpers_js_2.fixFloat)(val.r, 0),
                g: (0, helpers_js_2.fixFloat)(val.g, 0),
                b: (0, helpers_js_2.fixFloat)(val.b, 0),
            };
        }
    }
    return req ? { r: 0, g: 0, b: 0, a: 1 } : { r: 0, g: 0, b: 0 };
}
/**
 * Verifies and corrects data type for beatmap info.
 */
function info(data) {
    logger_js_1.logger.tInfo(['patch', 'dataCorrection', 'info'], 'Verifying and correcting data type for beatmap info...');
    data.song.title = (0, helpers_js_2.fixString)(data.song.title, 'Unknown');
    data.song.subTitle = (0, helpers_js_2.fixString)(data.song.subTitle, 'Unknown');
    data.song.author = (0, helpers_js_2.fixString)(data.song.author, 'Unknown');
    data.audio.bpm = (0, helpers_js_2.fixFloat)(data.audio.bpm, 120);
    data.audio.audioOffset = (0, helpers_js_2.fixFloat)(data.audio.audioOffset, 0);
    data.audio.shuffle = (0, helpers_js_2.fixFloat)(data.audio.shuffle);
    data.audio.shufflePeriod = (0, helpers_js_2.fixFloat)(data.audio.shufflePeriod);
    data.audio.previewStartTime = (0, helpers_js_2.fixFloat)(data.audio.previewStartTime, 12);
    data.audio.previewDuration = (0, helpers_js_2.fixFloat)(data.audio.previewDuration, 10);
    data.audio.filename = (0, helpers_js_2.fixString)(data.audio.filename, 'song.ogg');
    data.coverImageFilename = (0, helpers_js_2.fixString)(data.coverImageFilename, 'cover.png');
    data.environmentNames = data.environmentNames.map((n) => fixEnvironment(n, true));
    if (Array.isArray(data.environmentNames)) {
        data.environmentNames = data.environmentNames
            .filter((v) => v)
            .map((v) => fixEnvironment(v, true));
    }
    else
        data.environmentNames = [];
    if (Array.isArray(data.colorSchemes)) {
        data.colorSchemes = data.colorSchemes
            .filter((v) => v)
            .map((v) => {
            v.overrideNotes = (0, helpers_js_2.fixBoolean)(v.overrideNotes);
            v.overrideLights = (0, helpers_js_2.fixBoolean)(v.overrideLights);
            v.name = (0, helpers_js_2.fixString)(v.name, 'Unknown');
            v.saberLeftColor = fixColorObject(v.saberLeftColor, true);
            v.saberRightColor = fixColorObject(v.saberRightColor, true);
            v.environment0Color = fixColorObject(v.environment0Color, true);
            v.environment1Color = fixColorObject(v.environment1Color, true);
            v.obstaclesColor = fixColorObject(v.obstaclesColor, true);
            v.environment0ColorBoost = fixColorObject(v.environment0ColorBoost, true);
            v.environment1ColorBoost = fixColorObject(v.environment1ColorBoost, true);
            return v;
        });
    }
    else
        data.colorSchemes = [];
    for (const d of data.difficulties) {
        d.colorSchemeId = (0, helpers_js_1.clamp)((0, helpers_js_2.fixInt)(d.colorSchemeId, 0), 0, data.environmentNames.length - 1);
        d.environmentId = (0, helpers_js_1.clamp)((0, helpers_js_2.fixInt)(d.environmentId, 0), 0, data.colorSchemes.length - 1);
        d.njs = (0, helpers_js_2.fixFloat)(d.njs, 0);
        d.njsOffset = (0, helpers_js_2.fixFloat)(d.njsOffset, 0);
    }
}
