import type { GenericAudioDataFilename } from '../types/beatmap/shared/filename.js';
import type { InferBeatmap, InferBeatmapSerial, InferBeatmapVersion } from '../types/beatmap/shared/infer.js';
import type { IReadOptions } from '../types/bsmap/reader.js';
import type { LooseAutocomplete } from '../types/utils.js';
/**
 * Asynchronously read beatmap audio data file.
 * ```ts
 * readAudioDataFile('AudioData.dat', 4).then((data) => console.log(data));
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readAudioDataFile<TVersion extends InferBeatmapVersion<'audioData'>, TWrapper extends Record<string, any> = InferBeatmap<'audioData'>, TSerial extends Record<string, any> = InferBeatmapSerial<'audioData', TVersion>>(path: LooseAutocomplete<GenericAudioDataFilename>, version?: TVersion | null, options?: IReadOptions<'audioData', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
export declare function readAudioDataFile<TVersion extends InferBeatmapVersion<'audioData'>, TWrapper extends Record<string, any> = InferBeatmap<'audioData'>, TSerial extends Record<string, any> = InferBeatmapSerial<'audioData', TVersion>>(path: LooseAutocomplete<GenericAudioDataFilename>, options?: IReadOptions<'audioData', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
/**
 * Synchronously read beatmap audio data file.
 * ```ts
 * const audioData = readAudioDataFileSync('AudioData.dat', 4);
 * console.log(audioData);
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readAudioDataFileSync<TVersion extends InferBeatmapVersion<'audioData'>, TWrapper extends Record<string, any> = InferBeatmap<'audioData'>, TSerial extends Record<string, any> = InferBeatmapSerial<'audioData', TVersion>>(path: LooseAutocomplete<GenericAudioDataFilename>, version?: TVersion | null, options?: IReadOptions<'audioData', TVersion, TWrapper, TSerial>): TWrapper;
export declare function readAudioDataFileSync<TVersion extends InferBeatmapVersion<'audioData'>, TWrapper extends Record<string, any> = InferBeatmap<'audioData'>, TSerial extends Record<string, any> = InferBeatmapSerial<'audioData', TVersion>>(path: LooseAutocomplete<GenericAudioDataFilename>, options?: IReadOptions<'audioData', TVersion, TWrapper, TSerial>): TWrapper;
//# sourceMappingURL=audioData.d.ts.map