import type { GenericBeatmapFilename } from '../types/beatmap/shared/filename.js';
import type { InferBeatmap, InferBeatmapSerial, InferBeatmapVersion } from '../types/beatmap/shared/infer.js';
import type { IReadOptions } from '../types/bsmap/reader.js';
import type { LooseAutocomplete } from '../types/utils.js';
/**
 * Asynchronously read beatmap difficulty file.
 * ```ts
 * readDifficultyFile('EasyStandard.dat', 4).then((data) => console.log(data));
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readDifficultyFile<TVersion extends InferBeatmapVersion<'difficulty'>, TWrapper extends Record<string, any> = InferBeatmap<'difficulty'>, TSerial extends Record<string, any> = InferBeatmapSerial<'difficulty', TVersion>>(path: LooseAutocomplete<GenericBeatmapFilename>, version?: TVersion | null, options?: IReadOptions<'difficulty', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
export declare function readDifficultyFile<TVersion extends InferBeatmapVersion<'difficulty'>, TWrapper extends Record<string, any> = InferBeatmap<'difficulty'>, TSerial extends Record<string, any> = InferBeatmapSerial<'difficulty', TVersion>>(path: LooseAutocomplete<GenericBeatmapFilename>, options?: IReadOptions<'difficulty', TVersion, TWrapper, TSerial>): Promise<TWrapper>;
/**
 * Synchronously read beatmap difficulty file.
 * ```ts
 * const difficulty = readDifficultyFileSync('EasyStandard.dat', 4);
 * console.log(difficulty);
 * ```
 *
 * Mismatched beatmap version will be automatically converted, unspecified will leave the version as is but not known.
 */
export declare function readDifficultyFileSync<TVersion extends InferBeatmapVersion<'difficulty'>, TWrapper extends Record<string, any> = InferBeatmap<'difficulty'>, TSerial extends Record<string, any> = InferBeatmapSerial<'difficulty', TVersion>>(path: LooseAutocomplete<GenericBeatmapFilename>, version?: TVersion | null, options?: IReadOptions<'difficulty', TVersion, TWrapper, TSerial>): TWrapper;
export declare function readDifficultyFileSync<TVersion extends InferBeatmapVersion<'difficulty'>, TWrapper extends Record<string, any> = InferBeatmap<'difficulty'>, TSerial extends Record<string, any> = InferBeatmapSerial<'difficulty', TVersion>>(path: LooseAutocomplete<GenericBeatmapFilename>, options?: IReadOptions<'difficulty', TVersion, TWrapper, TSerial>): TWrapper;
//# sourceMappingURL=difficulty.d.ts.map