"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readFromInfo = readFromInfo;
exports.readFromInfoSync = readFromInfoSync;
const difficulty_js_1 = require("./difficulty.js");
const lightshow_js_1 = require("./lightshow.js");
async function readFromInfo(info, options = {}) {
    const ary = [];
    for (const d of info.difficulties) {
        const [posttransformer, ...postprocesses] = [
            ...(options.load?.postprocess ?? []),
        ].reverse();
        const beatmap = await (0, difficulty_js_1.readDifficultyFile)(d.filename, {
            ...options,
            load: {
                postprocess: [
                    ...postprocesses,
                    (x) => {
                        if (x.version === 4) {
                            const light = (0, lightshow_js_1.readLightshowFileSync)(d.lightshowFilename, {
                                directory: options.directory ?? options.directory,
                            });
                            x.lightshow = light.lightshow;
                            x.lightshowFilename = light.lightshowFilename;
                        }
                        return x;
                    },
                    posttransformer,
                ],
            },
        });
        ary.push({ info: d, beatmap });
    }
    return ary;
}
function readFromInfoSync(info, options = {}) {
    const ary = [];
    for (const d of info.difficulties) {
        const [posttransformer, ...postprocesses] = [
            ...(options.load?.postprocess ?? []),
        ].reverse();
        const beatmap = (0, difficulty_js_1.readDifficultyFileSync)(d.filename, {
            ...options,
            load: {
                postprocess: [
                    ...postprocesses,
                    (x) => {
                        if (x.version === 4) {
                            const light = (0, lightshow_js_1.readLightshowFileSync)(d.lightshowFilename, {
                                directory: options.directory,
                            });
                            x.lightshow = light.lightshow;
                            x.lightshowFilename = light.lightshowFilename;
                        }
                        return x;
                    },
                    posttransformer,
                ],
            },
        });
        ary.push({ info: d, beatmap });
    }
    return ary;
}
