"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readJSONFile = readJSONFile;
exports.readJSONFileSync = readJSONFileSync;
exports.writeJSONFile = writeJSONFile;
exports.writeJSONFileSync = writeJSONFileSync;
// deno-lint-ignore-file no-explicit-any
const logger_js_1 = require("../logger.js");
const fs_js_1 = require("./fs.js");
function tag(name) {
    return ['fs', name];
}
function readJSONFile(path) {
    logger_js_1.logger.tInfo(tag('readJSONFile'), `Async reading JSON file from ${path}`);
    return fs_js_1.fs.readTextFile(path).then(JSON.parse);
}
function readJSONFileSync(path) {
    logger_js_1.logger.tInfo(tag('readJSONFileSync'), `Sync reading JSON file from ${path}`);
    return JSON.parse(fs_js_1.fs.readTextFileSync(path));
}
function writeJSONFile(path, json, format) {
    logger_js_1.logger.tInfo(tag('writeJSONFile'), `Async writing JSON file to ${path}`);
    return fs_js_1.fs.writeTextFile(path, JSON.stringify(json, null, format));
}
function writeJSONFileSync(path, json, format) {
    logger_js_1.logger.tInfo(tag('writeJSONFileSync'), `Sync writing JSON file to ${path}`);
    fs_js_1.fs.writeTextFileSync(path, JSON.stringify(json, null, format));
}
