"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fs = void 0;
// @ts-ignore: trick
const _fs_js_1 = __importDefault(require("./_fs.js"));
// @ts-ignore: trick
const _fsp_js_1 = __importDefault(require("./_fsp.js"));
function noFsFunctionProvided() {
    throw new Error('`fs` function not provided; please supply `fs` function inside the `fs` object from the module');
}
// @ts-ignore: trick
const _fs = _fs_js_1.default;
// @ts-ignore: trick
const _fsp = _fsp_js_1.default;
/**
 * Wrapper for use in `read` and `write`.
 *
 * Do not use this as a mean to perform `fs` operations.
 * This module only serve to provide cross-platform compatibility.
 *
 * If your runtime or vendor does not contain `node:` built-in module,
 * you should provide `fs` function inside the `fs` object from the module.
 * If you are creating a web app, you may ignore this and use `load` and `save`.
 */
exports.fs = {
    readTextFile: noFsFunctionProvided,
    readTextFileSync: noFsFunctionProvided,
    writeTextFile: noFsFunctionProvided,
    writeTextFileSync: noFsFunctionProvided,
};
if (typeof Deno !== 'undefined') {
    exports.fs.readTextFile = Deno.readTextFile || noFsFunctionProvided;
    exports.fs.readTextFileSync = Deno.readTextFileSync || noFsFunctionProvided;
    exports.fs.writeTextFile = Deno.writeTextFile || noFsFunctionProvided;
    exports.fs.writeTextFileSync = Deno.writeTextFileSync || noFsFunctionProvided;
}
else {
    if (typeof Bun !== 'undefined') {
        exports.fs.readTextFile = (path) => {
            return Bun.file(path).text();
        };
        exports.fs.writeTextFile = Bun.write;
    }
    else if (_fsp) {
        exports.fs.readTextFile = (path) => {
            return _fsp.readFile(path, 'utf8');
        };
        exports.fs.writeTextFile = (path, data) => {
            return _fsp.writeFile(path, data, 'utf8');
        };
    }
    if (_fs) {
        exports.fs.readTextFileSync = (path) => {
            return _fs?.readFileSync(path, 'utf8');
        };
        exports.fs.writeTextFileSync = (path, data) => {
            return _fs?.writeFileSync(path, data, 'utf8');
        };
    }
}
