import type { MirrorFn } from '../shared/functions.js';
import type { InferBeatmap, InferBeatmapSerial, InferBeatmapVersion } from '../shared/infer.js';
import type { BeatmapFileType } from '../shared/schema.js';
import type { ISchemaCheckOptions } from './schema.js';
export interface ILoadOptions<TFileType extends BeatmapFileType, TVersion extends InferBeatmapVersion<TFileType>, TWrapper = Record<string, any>, TSerial = Record<string, any>> {
    /**
     * Force version conversion if loaded difficulty version is mismatched.
     *
     * @default true
     */
    forceConvert?: boolean;
    /** Data check option when loading. */
    schemaCheck?: Partial<ISchemaCheckOptions>;
    /**
     * Sort object(s) on load.
     *
     * @default true
     */
    sort?: boolean;
    /**
     * Perform any preprocessing when JSON is created or passed.
     *
     * **Warning**: This may result in side-effects if object is passed.
     *
     * @default []
     */
    preprocess?: [
        ((data: TSerial, version?: TVersion | null) => InferBeatmapSerial<TFileType, TVersion>),
        ...MirrorFn<InferBeatmapSerial<TFileType, TVersion>>[]
    ];
    /**
     * Perform any postprocessing after object attribute has been instantiated.
     *
     * @default []
     */
    postprocess?: [
        ...MirrorFn<InferBeatmap<TFileType>>[],
        ((data: InferBeatmap<TFileType>, version?: TVersion | null) => TWrapper)
    ];
}
//# sourceMappingURL=loader.d.ts.map