import type { Member } from '../../utils.js';
/** Base game object position X value. */
export declare const PosX: {
    readonly LEFT: 0;
    readonly MIDDLE_LEFT: 1;
    readonly MIDDLE_RIGHT: 2;
    readonly RIGHT: 3;
};
export type PosX = Member<typeof PosX>;
/** Base game object position Y value. */
export declare const PosY: {
    readonly BOTTOM: 0;
    readonly MIDDLE: 1;
    readonly TOP: 2;
};
export type PosY = Member<typeof PosY>;
/** Base game note color value. */
export declare const NoteColor: {
    readonly NONE: -1;
    readonly RED: 0;
    readonly BLUE: 1;
};
export type NoteColor = Member<typeof NoteColor>;
/** Base game note direction value. */
export declare const NoteDirection: {
    readonly UP: 0;
    readonly DOWN: 1;
    readonly LEFT: 2;
    readonly RIGHT: 3;
    readonly UP_LEFT: 4;
    readonly UP_RIGHT: 5;
    readonly DOWN_LEFT: 6;
    readonly DOWN_RIGHT: 7;
    readonly ANY: 8;
};
export type NoteDirection = Member<typeof NoteDirection>;
/** Base game waypoint offset direction value. */
export declare const OffsetDirection: {
    readonly UP: 0;
    readonly DOWN: 1;
    readonly LEFT: 2;
    readonly RIGHT: 3;
    readonly UP_LEFT: 4;
    readonly UP_RIGHT: 5;
    readonly DOWN_LEFT: 6;
    readonly DOWN_RIGHT: 7;
    readonly NONE: 9;
};
export type OffsetDirection = Member<typeof OffsetDirection>;
/** Base game slider mid anchor mode value used in arc. */
export declare const SliderMidAnchorMode: {
    readonly STRAIGHT: 0;
    readonly CLOCKWISE: 1;
    readonly COUNTER_CLOCKWISE: 2;
};
export type SliderMidAnchorMode = Member<typeof SliderMidAnchorMode>;
/** Base game event light value used in basic event. */
export declare const EventLightValue: {
    readonly OFF: 0;
    readonly BLUE_ON: 1;
    readonly BLUE_FLASH: 2;
    readonly BLUE_FADE: 3;
    readonly BLUE_TRANSITION: 4;
    readonly RED_ON: 5;
    readonly RED_FLASH: 6;
    readonly RED_FADE: 7;
    readonly RED_TRANSITION: 8;
    readonly WHITE_ON: 9;
    readonly WHITE_FLASH: 10;
    readonly WHITE_FADE: 11;
    readonly WHITE_TRANSITION: 12;
};
export type EventLightValue = Member<typeof EventLightValue>;
/** Generic event type value used in basic event. */
export declare const EventType: {
    readonly BACK_LASERS: 0;
    readonly RING_LIGHTS: 1;
    readonly LEFT_LASERS: 2;
    readonly RIGHT_LASERS: 3;
    readonly CENTER_LIGHTS: 4;
    readonly COLOR_BOOST: 5;
    readonly EXTRA_LEFT_LIGHTS: 6;
    readonly EXTRA_RIGHT_LIGHTS: 7;
    readonly RING_ROTATION: 8;
    readonly RING_ZOOM: 9;
    readonly EXTRA_LEFT_LASERS: 10;
    readonly EXTRA_RIGHT_LASERS: 11;
    readonly LEFT_LASER_ROTATION: 12;
    readonly RIGHT_LASER_ROTATION: 13;
    readonly EARLY_LANE_ROTATION: 14;
    readonly LATE_LANE_ROTATION: 15;
    readonly UTILITY_EVENT_0: 16;
    readonly UTILITY_EVENT_1: 17;
    readonly UTILITY_EVENT_2: 18;
    readonly UTILITY_EVENT_3: 19;
    readonly SPECIAL_EVENT_0: 40;
    readonly SPECIAL_EVENT_1: 41;
    readonly SPECIAL_EVENT_2: 42;
    readonly SPECIAL_EVENT_3: 43;
    readonly BPM_CHANGE: 100;
    readonly NJS_CHANGE: 1000;
};
export type EventType = Member<typeof EventType>;
/** Base game execution time value used in spawn rotation event. */
export declare const ExecutionTime: {
    readonly EARLY: 0;
    readonly LATE: 1;
};
export type ExecutionTime = Member<typeof ExecutionTime>;
/** Base game index filter type value. */
export declare const IndexFilterType: {
    readonly DIVISION: 1;
    readonly STEP_AND_OFFSET: 2;
};
export type IndexFilterType = Member<typeof IndexFilterType>;
/** Base game limit also affects type value used in index filter. */
export declare const LimitAlsoAffectsType: {
    readonly NONE: 0;
    readonly DURATION: 1;
    readonly DISTRIBUTION: 2;
    readonly ALL: 3;
};
export type LimitAlsoAffectsType = Member<typeof LimitAlsoAffectsType>;
/** Base game random type value used in index filter. */
export declare const RandomType: {
    readonly NO_RANDOM: 0;
    readonly KEEP_ORDER: 1;
    readonly RANDOM_ELEMENTS: 2;
    readonly ALL: 3;
};
export type RandomType = Member<typeof RandomType>;
/** Base game distribution type value used in light event. */
export declare const DistributionType: {
    readonly WAVE: 1;
    readonly STEP: 2;
};
export type DistributionType = Member<typeof DistributionType>;
/** Base game ease type value used in light event. */
export declare const EaseType: {
    readonly NONE: -1;
    readonly LINEAR: 0;
    readonly IN_QUAD: 1;
    readonly OUT_QUAD: 2;
    readonly INOUT_QUAD: 3;
    readonly IN_SINE: 4;
    readonly OUT_SINE: 5;
    readonly INOUT_SINE: 6;
    readonly IN_CUBIC: 7;
    readonly OUT_CUBIC: 8;
    readonly INOUT_CUBIC: 9;
    readonly IN_QUART: 10;
    readonly OUT_QUART: 11;
    readonly INOUT_QUART: 12;
    readonly IN_QUINT: 13;
    readonly OUT_QUINT: 14;
    readonly INOUT_QUINT: 15;
    readonly IN_EXPO: 16;
    readonly OUT_EXPO: 17;
    readonly INOUT_EXPO: 18;
    readonly IN_CIRC: 19;
    readonly OUT_CIRC: 20;
    readonly INOUT_CIRC: 21;
    readonly IN_BACK: 22;
    readonly OUT_BACK: 23;
    readonly INOUT_BACK: 24;
    readonly IN_ELASTIC: 25;
    readonly OUT_ELASTIC: 26;
    readonly INOUT_ELASTIC: 27;
    readonly IN_BOUNCE: 28;
    readonly OUT_BOUNCE: 29;
    readonly INOUT_BOUNCE: 30;
    readonly BS_INOUT_BACK: 100;
    readonly BS_INOUT_ELASTIC: 101;
    readonly BS_INOUT_BOUNCE: 102;
};
export type EaseType = Member<typeof EaseType>;
/** Base game event color value used in light color event. */
export declare const EventLightColor: {
    readonly NONE: -1;
    readonly RED: 0;
    readonly BLUE: 1;
    readonly WHITE: 2;
};
export type EventLightColor = Member<typeof EventLightColor>;
/** Base game axis value used in rotation/translation event box. */
export declare const Axis: {
    readonly X: 0;
    readonly Y: 1;
    readonly Z: 2;
};
export type Axis = Member<typeof Axis>;
/** Base game light rotation direction value used in light rotation event. */
export declare const LightRotationDirection: {
    readonly AUTOMATIC: 0;
    readonly CLOCKWISE: 1;
    readonly COUNTER_CLOCKWISE: 2;
};
export type LightRotationDirection = Member<typeof LightRotationDirection>;
/** Base game note type value used in v1/v2 note. */
export declare const NoteType: {
    readonly NOTE_A: 0;
    readonly NOTE_B: 1;
    readonly BOMB: 3;
};
export type NoteType = Member<typeof NoteType>;
/** Base game obstacle type value used in v1/v2 obstacle. */
export declare const ObstacleType: {
    readonly FULL_HEIGHT: 0;
    readonly TOP: 1;
    readonly FREE: 2;
};
export type ObstacleType = Member<typeof ObstacleType>;
/** Base game transition type value used in v3 light color event. */
export declare const TransitionType: {
    readonly INSTANT: 0;
    readonly INTERPOLATE: 1;
    readonly EXTEND: 2;
};
export type TransitionType = Member<typeof TransitionType>;
/** Base game FX type value used in v3 FX event group. */
export declare const FxType: {
    readonly INT: 0;
    readonly FLOAT: 1;
    readonly BOOL: 2;
};
export type FxType = Member<typeof FxType>;
/** Base game event box type value used in v4 event box group. */
export declare const EventBoxType: {
    readonly NONE: 0;
    readonly COLOR: 1;
    readonly ROTATION: 2;
    readonly TRANSLATION: 3;
    readonly FX_FLOAT: 4;
};
export type EventBoxType = Member<typeof EventBoxType>;
//# sourceMappingURL=constants.d.ts.map