import type { Nullable } from '../../../utils.js';
import type { Vector2, Vector3 } from '../../../vector.js';
import type { ICustomDataBase } from '../../shared/custom/customData.js';
import type { FloatPointDefinition, Vector3PointDefinition } from '../../shared/custom/heck.js';
import type { PlayerObject } from '../../shared/custom/noodleExtensions.js';
import type { IHeckBase } from './heck.js';
/** Noodle Extensions Object interface for Beatmap Object. */
interface INEObject {
    _position?: Vector2;
    _rotation?: number | Vector3;
    _localRotation?: Vector3;
    /**
     * **Type:** `f32`
     */
    _noteJumpMovementSpeed?: number;
    /**
     * **Type:** `f32`
     */
    _noteJumpStartBeatOffset?: number;
    _fake?: boolean;
    _interactable?: boolean;
}
/**
 * Noodle Extensions Note interface for Beatmap Note.
 */
export interface INENote extends INEObject {
    /**
     * **Type:** `f32`
     */
    _cutDirection?: number;
    _flip?: Vector2;
    _disableNoteGravity?: boolean;
    _disableNoteLook?: boolean;
}
/**
 * Noodle Extensions Obstacle interface for Beatmap Obstacle.
 */
export interface INEObstacle extends INEObject {
    _scale?: Nullable<Vector3>;
}
/**
 * Noodle Extensions Event interface for Beatmap Event.
 */
export interface INEEvent extends ICustomDataBase {
    /**
     * **Type:** `f32`
     */
    _rotation?: number;
}
/**
 * AssignPathAnimation interface for Noodle Extensions Custom Event.
 */
export interface INECustomEventDataAnimateTrack extends Required<IHeckBase> {
    _dissolve?: string | FloatPointDefinition;
    _dissolveArrow?: string | FloatPointDefinition;
    _interactable?: string | FloatPointDefinition;
    _time?: string | FloatPointDefinition;
}
/**
 * AssignPathAnimation interface for Noodle Extensions Custom Event.
 */
export interface INECustomEventDataAssignPathAnimation extends Required<IHeckBase> {
    _dissolve?: string | FloatPointDefinition;
    _dissolveArrow?: string | FloatPointDefinition;
    _interactable?: string | FloatPointDefinition;
    _definitePosition?: string | Vector3PointDefinition;
}
/** AssignPathAnimation interface for Noodle Extensions Custom Event. */
export interface INECustomEventDataAssignTrackParent {
    _childrenTracks: string | string[];
    _parentTrack: string;
    _worldPositionStays?: boolean;
}
/**
 * AssignPlayerToTrack interface for Noodle Extensions Custom Event.
 */
export interface INECustomEventDataAssignPlayerToTrack extends Required<IHeckBase> {
    _target?: PlayerObject;
}
/** Noodle Extensions Animation interface for Noodle Extensions Object. */
export interface INEAnimation {
    _position?: string | Vector3PointDefinition;
    _rotation?: string | Vector3PointDefinition;
    _localRotation?: string | Vector3PointDefinition;
    _scale?: string | Vector3PointDefinition;
    _dissolve?: string | FloatPointDefinition;
    _dissolveArrow?: string | FloatPointDefinition;
    _interactable?: string | FloatPointDefinition;
    _definitePosition?: string | Vector3PointDefinition;
    _time?: string | FloatPointDefinition;
}
export {};
//# sourceMappingURL=noodleExtensions.d.ts.map