import type { LooseAutocomplete } from '../../utils.js';
import type { CharacteristicName } from '../shared/characteristic.js';
import type { DifficultyName } from '../shared/difficulty.js';
import type { Environment360Name, EnvironmentAllName, EnvironmentName, EnvironmentV3Name } from '../shared/environment.js';
import type { IColor } from '../../colors.js';
import type { ICustomDataInfo } from './custom/info.js';
import type { ICustomDataInfoBeatmap } from '../v4/custom/info.js';
import type { IWrapBaseFile } from './baseFile.js';
import type { GenericBeatmapFilename, GenericInfoFilename, GenericLightshowFilename } from '../shared/filename.js';
import type { IWrapBaseItem } from './baseItem.js';
/**
 * Wrapper attribute for beatmap info.
 */
export interface IWrapInfo extends IWrapBaseItem, IWrapBaseFile<GenericInfoFilename> {
    song: IWrapInfoSong;
    audio: IWrapInfoAudio;
    songPreviewFilename: string;
    coverImageFilename: string;
    /**
     * @deprecated use for pre-v4
     */
    environmentBase: {
        normal: EnvironmentName | EnvironmentV3Name | null;
        allDirections: Environment360Name | null;
    };
    environmentNames: EnvironmentAllName[];
    colorSchemes: IWrapInfoColorScheme[];
    difficulties: IWrapInfoBeatmap[];
    customData: ICustomDataInfo;
}
/**
 * Wrapper interface for beatmap info song.
 */
export interface IWrapInfoSong {
    title: string;
    subTitle: string;
    author: string;
}
/**
 * Wrapper interface for beatmap info audio.
 */
export interface IWrapInfoAudio {
    filename: string;
    duration: number;
    audioDataFilename: string;
    bpm: number;
    lufs: number;
    previewStartTime: number;
    previewDuration: number;
    /**
     * Exist for backport compatibility.
     *
     * Highly recommended to not use offset as it causes audio syncing issue.
     *
     * @deprecated
     */
    audioOffset: number;
    /**
     * Exist for backport compatibility.
     *
     * @deprecated
     */
    shuffle: number;
    /**
     * Exist for backport compatibility.
     *
     * @deprecated
     */
    shufflePeriod: number;
}
/**
 * Wrapper interface for beatmap info color scheme.
 */
export interface IWrapInfoColorScheme {
    name: string;
    overrideNotes: boolean;
    overrideLights: boolean;
    saberLeftColor: Required<IColor>;
    saberRightColor: Required<IColor>;
    environment0Color: Required<IColor>;
    environment1Color: Required<IColor>;
    environmentWColor?: Required<IColor>;
    obstaclesColor: Required<IColor>;
    environment0ColorBoost: Required<IColor>;
    environment1ColorBoost: Required<IColor>;
    environmentWColorBoost?: Required<IColor>;
}
/**
 * Wrapper interface for beatmap info beatmap authors.
 */
export interface IWrapInfoBeatmapAuthors {
    mappers: string[];
    lighters: string[];
}
/**
 * Wrapper attribute for beatmap info beatmap.
 */
export interface IWrapInfoBeatmap extends IWrapBaseItem {
    characteristic: CharacteristicName;
    difficulty: DifficultyName;
    filename: LooseAutocomplete<GenericBeatmapFilename>;
    lightshowFilename: LooseAutocomplete<GenericLightshowFilename>;
    authors: IWrapInfoBeatmapAuthors;
    njs: number;
    njsOffset: number;
    colorSchemeId: number;
    environmentId: number;
    customData: ICustomDataInfoBeatmap;
}
//# sourceMappingURL=info.d.ts.map