import type { ColorArray, ColorInput, ColorObject, ColorType, IColor } from '../../types/colors.js';
/** Return RGBA color array from input. */
export declare function colorFrom(r: number, g: number, b: number, a?: number): ColorArray;
export declare function colorFrom(r: number, g: number, b: number, a: number, type: 'rgba'): Required<ColorArray>;
export declare function colorFrom(r: number, g: number, b: number, a: number, type: 'rgba255'): Required<ColorArray>;
export declare function colorFrom(h: number, s: number, v: number, a: number, type: 'hsva'): Required<ColorArray>;
export declare function colorFrom(r: number, g: number, b: number, type: 'rgba'): ColorArray;
export declare function colorFrom(r: number, g: number, b: number, type: 'rgba255'): ColorArray;
export declare function colorFrom(h: number, s: number, v: number, type: 'hsva'): ColorArray;
export declare function colorFrom(value: number): ColorArray;
export declare function colorFrom(value: number, normalise255?: boolean): ColorArray;
export declare function colorFrom(value: number, alpha: number): Required<ColorArray>;
export declare function colorFrom(hex: string): ColorArray;
export declare function colorFrom(color: (number | undefined)[]): ColorArray;
export declare function colorFrom(color: (number | undefined)[], type: 'rgba'): ColorArray;
export declare function colorFrom(color: (number | undefined)[], type: 'rgba255'): ColorArray;
export declare function colorFrom(color: (number | undefined)[], type: 'hsva'): ColorArray;
export declare function colorFrom(color: ColorObject): ColorArray;
export declare function colorFrom(color: IColor): ColorArray;
/**
 * Convert color input to color object.
 * ```ts
 * const color1 = toColorObject([0, 1, 1]); // { r: 0, g: 1, b: 1 }
 * const color2 = toColorObject([0, 1, 1], true); // { r: 0, g: 1, b: 1, a: 1 }
 * ```
 */
export declare function toColorObject(color: ColorInput | IColor, ensureAlpha?: boolean): IColor;
export declare function toColorObject(color: ColorInput | IColor, ensureAlpha: true): Required<IColor>;
/**
 * Convert color input to standard hex string.
 * ```ts
 * const hex = colorToHex([0, 1, 1]); // '#00ffff'
 * ```
 */
export declare function colorToHex(color: ColorInput | IColor): string;
/**
 * Convert hex string to RGBA array.
 * ```ts
 * const rgba = hexToRgba('#00ffff'); // [0, 1, 1, 1]
 * ```
 */
export declare function hexToRgba(hex: string): ColorArray;
/**
 * Convert color input to standard RGBA array.
 * ```ts
 * const rgba = convertColorType([30, 0.75, 1], 'hsva')
 * ```
 *
 * Default color output type is RGBA unless specified otherwise.
 */
export declare function convertColorType(color: ColorInput | IColor, type?: ColorType, output?: 'rgba' | 'hsva'): ColorArray;
/**
 * Convert RGBA value to HSVA array.
 * ```
 * const hsva1 = rgbaToHsva(r, g, b, a);
 * const hsva2 = rgbaToHsva(rgba);
 * ```
 */
export declare function rgbaToHsva(r: number, g: number, b: number, a?: number): ColorArray;
export declare function rgbaToHsva(color: ColorArray): ColorArray;
/**
 * Convert HSVA value to RGBA array.
 * ```
 * const rgba1 = hsvaToRgba(h, s, v, a);
 * const rgba2 = hsvaToRgba(hsva);
 * ```
 */
export declare function hsvaToRgba(hue: number, saturation: number, value: number, alpha?: number): ColorArray;
export declare function hsvaToRgba(color: ColorArray): ColorArray;
export declare function rgbaToLabA(rgbaAry: ColorArray): ColorArray;
/** Convert LAB to Hue */
export declare function labToHue(a: number, b: number): number;
//# sourceMappingURL=convertor.d.ts.map