"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.radToDeg = radToDeg;
exports.degToRad = degToRad;
exports.cartesianCoordToSphericalCoord = cartesianCoordToSphericalCoord;
exports.sphericalCoordToCartesianCoord = sphericalCoordToCartesianCoord;
exports.mod = mod;
exports.shortRotDistance = shortRotDistance;
/** Convert degrees to radians. */
function radToDeg(rad) {
    return rad * (180 / Math.PI);
}
/** Convert radians to degrees. */
function degToRad(deg) {
    return deg * (Math.PI / 180);
}
/**
 * Convert cartesian coordinates to spherical coordinates.
 *
 * Return `[radius, theta, phi]`.
 */
function cartesianCoordToSphericalCoord(x, y, z) {
    const radius = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2) + Math.pow(z, 2));
    return [
        radius,
        Math.acos(z / radius),
        (Math.atan2(y, x) + 2 * Math.PI) % (2 * Math.PI),
    ];
}
/**
 * Convert spherical coordinates to cartesian coordinates.
 *
 * Return `[x, y, z]`
 */
function sphericalCoordToCartesianCoord(radius, theta, phi) {
    return [
        radius * Math.sin(theta) * Math.cos(phi),
        radius * Math.sin(theta) * Math.sin(phi),
        radius * Math.cos(theta),
    ];
}
// thanks Top_Cat#1961
/**
 * Modulo function that can handle negative numbers.
 */
function mod(x, m) {
    if (m < 0) {
        m = -m;
    }
    const r = x % m;
    return r < 0 ? r + m : r;
}
/**
 * Get the shortest distance between two angles.
 */
function shortRotDistance(a, b, m) {
    return Math.min(mod(a - b, m), mod(b - a, m));
}
