import type { Vector2, Vector2Object, Vector3, Vector3Object, Vector4, Vector4Object } from '../../types/vector.js';
type VectorObject = Partial<Vector2Object> | Partial<Vector3Object> | Partial<Vector4Object>;
/** Check if value is `Vector2` */
export declare function isVector2(obj: unknown): obj is Vector2;
/** Check if value is `Vector3` */
export declare function isVector3(obj: unknown): obj is Vector3;
/** Check if value is `Vector4` */
export declare function isVector4(obj: unknown): obj is Vector4;
type VectorArgument = Vector2 | Vector3 | Vector4 | number[];
export declare function vectorAdd<T extends VectorArgument | undefined>(vec: T, value?: number | number[]): T;
export declare function vectorAdd(vec?: Vector2, value?: VectorObject): Vector2;
export declare function vectorAdd(vec?: Vector3, value?: VectorObject): Vector3;
export declare function vectorAdd(vec?: Vector4, value?: VectorObject): Vector4;
export declare function vectorSub<T extends VectorArgument | undefined>(vec: T, value?: number | number[]): T;
export declare function vectorSub(vec?: Vector2, value?: VectorObject): Vector2;
export declare function vectorSub(vec?: Vector3, value?: VectorObject): Vector3;
export declare function vectorSub(vec?: Vector4, value?: VectorObject): Vector4;
export declare function vectorMul<T extends VectorArgument | undefined>(vec: T, value?: number | number[]): T;
export declare function vectorMul(vec?: Vector2, value?: VectorObject): Vector2;
export declare function vectorMul(vec?: Vector3, value?: VectorObject): Vector3;
export declare function vectorMul(vec?: Vector4, value?: VectorObject): Vector4;
export declare function vectorDiv<T extends VectorArgument | undefined>(vec: T, value?: number | number[]): T;
export declare function vectorDiv(vec?: Vector2, value?: VectorObject): Vector2;
export declare function vectorDiv(vec?: Vector3, value?: VectorObject): Vector3;
export declare function vectorDiv(vec?: Vector4, value?: VectorObject): Vector4;
export declare function lerpVector<T extends VectorArgument | undefined>(alpha: number, vec: T, value?: number[]): T;
export declare function lerpVector(valpha: number, ec?: Vector2, value?: VectorObject): Vector2;
export declare function lerpVector(alpha: number, vec?: Vector3, value?: VectorObject): Vector3;
export declare function lerpVector(alpha: number, vec?: Vector4, value?: VectorObject): Vector4;
export declare function vectorDistance<T extends VectorArgument>(v1: T, v2: T): number;
export declare function vectorIsVertical<T extends VectorArgument>(v1: T, v2: T, epsilon?: number): boolean;
export declare function vectorIsHorizontal<T extends VectorArgument>(v1: T, v2: T, epsilon?: number): boolean;
export declare function vectorIsDiagonal<T extends VectorArgument>(v1: T, v2: T, epsilon?: number): boolean;
export declare function vectorIsInline<T extends VectorArgument>(v1: T, v2: T, lapping?: number): boolean;
export declare function vectorIsAdjacent<T extends VectorArgument>(v1: T, v2: T, epsilon?: number): boolean;
export declare function vectorIsWindow<T extends VectorArgument>(v1: T, v2: T, gap?: number): boolean;
export declare function vectorIsSlantedWindow<T extends VectorArgument>(v1: T, v2: T): boolean;
export declare function vectorMagnitude(vec: number[]): number;
export declare function vectorNormalize<T extends number[]>(vec: T): T;
export declare function vectorEqual<T extends VectorArgument>(v1: T, v2: T): boolean;
export declare function vectorNearEqual<T extends VectorArgument>(v1: T, v2: T, tolerance?: number): boolean;
export {};
//# sourceMappingURL=vector.d.ts.map