"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.range = range;
exports.shuffle = shuffle;
exports.interleave = interleave;
exports.pickRandom = pickRandom;
exports.cycle = cycle;
function range(arg0, arg1, arg2, arg3) {
    let start = arg0;
    let inc = arg3;
    let end;
    let step;
    if (typeof arg1 === 'boolean') {
        end = 0;
        inc = arg1;
    }
    else if (typeof arg1 === 'number') {
        end = arg1;
    }
    else {
        end = start;
        start = 0;
    }
    if (typeof arg2 === 'boolean') {
        inc = arg2;
    }
    else {
        step = arg2;
    }
    if (!step)
        step = start > end ? -1 : 1;
    const ary = new Array(Math.abs(Math.ceil((end - start) / step)) + (inc ? 1 : 0));
    for (let i = 0; i < ary.length; i++) {
        ary[i] = start + i * step;
    }
    return ary;
}
/**
 * Shuffle array in-place.
 *
 * Using Fisher–Yates shuffle algorithm.
 */
function shuffle(array, fn = Math.random) {
    for (let i = array.length - 1; i > 0; i--) {
        const j = Math.floor(fn() * (i + 1));
        [array[i], array[j]] = [array[j], array[i]];
    }
    return array;
}
/**
 * Interleave two arrays.
 *
 * ```ts
 * interleave([1, 2, 3], [4, 5, 6]); // [1, 4, 2, 5, 3, 6]
 * ```
 */
function interleave([x, ...xs], ys = []) {
    return x === undefined
        ? ys // base: no x
        : [x, ...interleave(ys, xs)]; // inductive: some x
}
/** Pick random element from array */
function pickRandom(ary, fn = Math.random) {
    return ary[Math.floor(fn() * ary.length)];
}
/** Move to the next item in an iterable object programatically. */
function cycle(iter, current, step = 1) {
    const arr = Object.values(iter);
    const index = arr.indexOf(current);
    if (!(current in arr))
        return current;
    return arr[(index + arr.length + step) % arr.length];
}
