"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.secToMmss = secToMmss;
exports.minToHhmmss = minToHhmmss;
exports.secToMmssms = secToMmssms;
exports.mmssToSec = mmssToSec;
/**
 * Convert number value of second to MM:SS string.
 */
function secToMmss(seconds) {
    if (!seconds) {
        return '0:00';
    }
    const numr = Math.floor(seconds);
    const temp = numr / 60;
    const min = temp < 0 ? `-${Math.ceil(temp).toString()}` : Math.floor(temp).toString();
    const sec = Math.abs(numr % 60)
        .toString()
        .padStart(2, '0');
    return `${min}:${sec}`;
}
/**
 * Convert number value of minute to HH:MM:SS string.
 */
function minToHhmmss(minutes) {
    if (!minutes) {
        return '0:00:00';
    }
    const sec = Math.abs(Math.round((minutes * 60) % 60))
        .toString()
        .padStart(2, '0');
    return `${secToMmss(minutes > 0 ? Math.floor(minutes) : Math.ceil(minutes))}:${sec}`;
}
/**
 * Convert number value of second to MM:SS.SSS string.
 */
function secToMmssms(seconds) {
    if (!seconds) {
        return '0:00.000';
    }
    const dec = (seconds % 1).toString().split('.')[1]?.padEnd(3, '0').slice(0, 3) || '000';
    return `${secToMmss(seconds > 0 ? Math.floor(seconds) : Math.ceil(seconds))}.${dec}`;
}
/**
 * Convert MM:SS string to number value of second.
 */
function mmssToSec(mmss) {
    const [m, s] = mmss.split(':').map((el) => Math.abs(parseInt(el)));
    const sign = mmss.trim().startsWith('-');
    return (sign ? -1 : 1) * (m * 60 + s);
}
